@@ -104,6 +104,11 @@ do_test(char *name)
 	return PM_ERR_INDOM;
     }
 
+    if ((err = pmFetch(1, &pmid, &r1)) < 0) {
+	fprintf(stderr, "pmFetch failed for %s: %s\n", name, pmErrStr(err));
+	return err;
+    }
+
     if ((err = pmGetInDom(desc.indom, &instlist, &namelist)) < 0) {
 	fprintf(stderr, "pmGetInDom failed for %s: %s\n", name, pmErrStr(err));
 	return err;
@@ -135,11 +140,6 @@ do_test(char *name)
 	}
 	free(myname);
     }
-    
-    if ((err = pmFetch(1, &pmid, &r1)) < 0) {
-	fprintf(stderr, "pmFetch failed for %s: %s\n", name, pmErrStr(err));
-	return err;
-    }
 
     if ((err = pmDelProfile(desc.indom, 0, (int *)0)) < 0) {
 	fprintf(stderr, "pmDelProfile failed for %s: %s\n", name, pmErrStr(err));
@@ -451,6 +451,11 @@ Options\n\
 		pmGetProgname(), pmErrStr(sts));
 	    exit(1);
 	}
+	if ((sts = pmSetMode(PM_MODE_FORW, NULL, 1)) < 0) {
+	    fprintf(stderr, "%s: Cannot set archive mode: %s\n",
+		pmGetProgname(), pmErrStr(sts));
+	    exit(1);
+	}
     }
 
     exit(do_test(metricname));
