@@ -73,8 +73,8 @@ _pcp_complete()
         arg_regex="-[dpw]"
     ;;
     pmiectl)
-        all_args="acfimNpV"
-        arg_regex="-[cip]"
+        all_args="aCcfimNpV"
+        arg_regex="-[Ccip]"
     ;;
     pminfo)
         all_args="abcdFfhIKLlMmNnOsTtVvxZz"
@@ -93,12 +93,12 @@ _pcp_complete()
         arg_regex="-[nSTZ]"
     ;;
     pmlogctl)
-        all_args="acfimNpV"
-        arg_regex="-[cip]"
+        all_args="aCcfimNpV"
+        arg_regex="-[Ccip]"
     ;;
     pmlogextract)
-        all_args="cdfmSsTvwxZz"
-        arg_regex="-[cSsTvZ]"
+        all_args="cdfmSsTVvwxZz"
+        arg_regex="-[cSsTVvZ]"
     ;;
     pmlogger)
         all_args="CcHIhKLlmNnoPprsTtUuVvxy"
@@ -133,8 +133,8 @@ _pcp_complete()
         arg_regex="-[04689ABabcEeFfhiJKlNOoPQqSsTtWwXYyZ]"
     ;;
     pmseries)
-        all_args="acdFghIiLlMmnpqSstVvZ"
-        arg_regex="-[cghpZ]"
+        all_args="acdFghIiLlMmnpqSstVvwZ"
+        arg_regex="-[cghpwZ]"
     ;;
     pmstat)
         all_args="AagHhLlnOPpSsTtVxZz"
