@@ -7,7 +7,7 @@
  *	remain fixed across releases, and they may not work, or may
  *	provide different semantics at some point in the future.
  *
- * Copyright (c) 2012-2021 Red Hat.
+ * Copyright (c) 2012-2022 Red Hat.
  * Copyright (c) 2008-2009 Aconex.  All Rights Reserved.
  * Copyright (c) 1995-2002 Silicon Graphics, Inc.  All Rights Reserved.
  *
@@ -376,14 +376,16 @@ PCP_CALL extern int __pmSendError(int, int, int);
 PCP_CALL extern int __pmDecodeError(__pmPDU *, int *);
 PCP_CALL extern int __pmSendXtendError(int, int, int, int);
 PCP_CALL extern int __pmDecodeXtendError(__pmPDU *, int *, int *);
-PCP_CALL extern int __pmSendResult(int, int, const pmResult *);
-PCP_CALL extern int __pmEncodeResult(int, const pmResult *, __pmPDU **);
-PCP_CALL extern int __pmSendHighResResult(int, int, const pmHighResResult *);
-PCP_CALL extern int __pmEncodeHighResResult(const pmHighResResult *, __pmPDU **);
-PCP_CALL extern int __pmDecodeResult(__pmPDU *, pmResult **);
-PCP_CALL extern int __pmDecodeHighResResult(__pmPDU *, pmHighResResult **);
+PCP_CALL extern int __pmSendResult(int, int, const __pmResult *);
+/* see below for __pmEncodeResult() */
+PCP_CALL extern int __pmSendHighResResult(int, int, const __pmResult *);
+PCP_CALL extern int __pmEncodeHighResResult(const __pmResult *, __pmPDU **);
+PCP_CALL extern int __pmDecodeResult(__pmPDU *, __pmResult **);
+PCP_CALL extern int __pmDecodeHighResResult(__pmPDU *, __pmResult **);
+PCP_CALL extern int __pmDecodeValueSet(__pmPDU *, int, __pmPDU *, char *, int, int, int, pmValueSet **);
 PCP_CALL extern int __pmSendProfile(int, int, int, pmProfile *);
 PCP_CALL extern int __pmDecodeProfile(__pmPDU *, int *, pmProfile **);
+PCP_CALL extern int __pmSendFetchPDU(int, int, int, int, pmID *, int);
 PCP_CALL extern int __pmSendFetch(int, int, int, int, pmID *);
 PCP_CALL extern int __pmDecodeFetch(__pmPDU *, int *, void *, int *, pmID **);
 PCP_CALL extern int __pmSendHighResFetch(int, int, int, int, pmID *);
@@ -423,6 +425,7 @@ PCP_CALL extern int __pmDecodeLabelReq(__pmPDU *, int *, int *);
 PCP_CALL extern int __pmSendLabel(int, int, int, int, pmLabelSet *, int);
 PCP_CALL extern int __pmDecodeLabel(__pmPDU *, int *, int *, pmLabelSet **, int *);
 PCP_CALL unsigned int __pmServerGetFeaturesFromPDU(__pmPDU *);
+PCP_CALL extern int __pmFeaturesIPC(int);
 
 /* PDU buffer services */
 PCP_CALL extern __pmPDU *__pmFindPDUBuf(int);
@@ -435,6 +438,7 @@ PCP_DATA extern unsigned int *__pmPDUCntIn;
 PCP_DATA extern unsigned int *__pmPDUCntOut;
 PCP_CALL extern void __pmSetPDUCntBuf(unsigned *, unsigned *);
 PCP_CALL extern void __pmDumpPDUCnt(FILE *);
+PCP_CALL extern void __pmDumpPDUTrace(FILE *);
 
 /* internal IPC protocol stuff */
 typedef int (*__pmConnectHostType)(int, int);
@@ -652,17 +656,22 @@ PCP_CALL extern int __pmGetDomainLabels(int, const char *, pmLabelSet **);
  * used as a trailer
  */
 typedef struct __pmLogHdr {
-    int		len;	/* record length, includes header and trailer */
-    int		type;	/* see TYPE_* #defines below */
+    __int32_t		len;	/* record length, includes header and trailer */
+    __int32_t		type;	/* see TYPE_* #defines below */
 } __pmLogHdr;
 
-#define TYPE_DESC		1	/* header, pmDesc, trailer */
-#define TYPE_INDOM_V2		2	/* header, __pmLogInDom_v2, trailer */
+#define TYPE_DESC		1	/* header, pmDesc, trailer          */
+#define TYPE_INDOM_V2		2	/* header, __pmInDom_v2, trailer    */
+					/*         (with pmTimeval)         */
 #define TYPE_LABEL_V2		3	/* header, __pmLogLabelSet, trailer */
 					/*         (with pmTimeval)         */
-#define TYPE_TEXT		4	/* header, __pmLogText, trailer */
-#define TYPE_INDOM		5	/* header, __pmLogInDom, trailer */
-#define TYPE_INDOM_DELTA	6	/* header, __pmLogInDom, trailer */
+#define TYPE_TEXT		4	/* header, __pmLogText, trailer     */
+#define TYPE_INDOM		5	/* header, __pmInDom_v3, trailer    */
+					/*         ("full" indom variant,   */
+					/*          with __pmTimestamp)     */
+#define TYPE_INDOM_DELTA	6	/* header, __pmInDom_v3, trailer    */
+					/*         ("delta" indom variant,  */
+					/*          with __pmTimestamp)     */
 #define TYPE_LABEL		7	/* header, __pmLogLabelSet, trailer */
 					/*         (with __pmTimestamp)     */
 #define TYPE_MAX TYPE_LABEL
@@ -679,34 +688,33 @@ typedef struct __pmLogHdr {
  *	nameindex[0] .... nameindex[numinst-1]
  *	string (name) table, all null-byte terminated
  *
- * NOTE: 3 types of allocation
- * (1) buf is NULL, 
- *     namelist and instlist have been allocated separately and so must
- *     each be freed.
- * (2) buf is NOT NULL, allinbuf == 1,
- *     all allocations were in the buffer and so only the buffer should
- *     be freed,
- * (3) buf is NOT NULL, allinbuf == 0,
- *     as well as buffer allocation, the namelist has been allocated
- *     separately and so both the buf and namelist should be freed.
- *
- * NOTE: timestamp is pmTimeVal for V2 archives and pmTimespec fo
+ * NOTE: timestamp is pmTimeVal for V2 archives and pmTimespec for
  *       V3 archives
  *
  * NOTE: the same structure is used for TYPE_INDOM_DELTA records, except
  *       the semantics of nameindex[] is changed so that a value of -1
  *       means the corresponding instance has been deleted from the
- *       instance domain, else the corresponding instance has been added
+ *       instance domain (similarly namelist[] becomes NULL once the
+ *       record is loaded), else the corresponding instance has been added
  *       to the instance domain
  */
+
+#define PMLID_SELF	1		/* __pmLogInDom is malloc'd */
+#define PMLID_INSTLIST	2		/* instlist[] is malloc'd */
+#define PMLID_NAMELIST	4		/* namelist[] is malloc'd */
+#define PMLID_NAMES	8		/* namelist[i] strings are malloc'd */
+
 typedef struct __pmLogInDom {
-    struct __pmLogInDom	*next;
+    struct __pmLogInDom	*next;			/* backwards in time */
+    struct __pmLogInDom	*prior;			/* forwards in time */
+    pmInDom		indom;
     __pmTimestamp	stamp;
+    int			isdelta;		/* 1 => delta indom semantics */
     int			numinst;
-    int			*instlist;		/* points into buf[] */
-    char		**namelist;		/* may pint into buf[] */
+    int			alloc;			/* PMLID_... allocation flag bits */
+    int			*instlist;		/* may point into buf[] */
+    char		**namelist;		/* may point into buf[] */
     __int32_t		*buf;			/* on-disk buffer */
-    int			allinbuf; 
 } __pmLogInDom;
 
 /*
@@ -760,7 +768,7 @@ typedef struct {
     int			pid;		/* PID of logger */
     __pmTimestamp	start;		/* start of this log */
     int			vol;		/* current log volume no. */
-    uint32_t		features;	/* current enabled features */
+    __uint32_t		features;	/* current enabled features */
     char		*hostname;	/* hostname at collection host */
     char		*timezone;	/* squashed $TZ at collection host */
     char		*zoneinfo;	/* detailed $TZ at collection host */
@@ -813,6 +821,8 @@ typedef struct {
 #define PM_LOG_STATE_NEW	0
 #define PM_LOG_STATE_INIT	1
 
+PCP_CALL extern int __pmEncodeResult(const __pmLogCtl *, const __pmResult *, __pmPDU **);
+
 /*
  * Minimal information to retain for each archive in a multi-archive context
  */
@@ -835,6 +845,7 @@ typedef struct {
     long		ac_offset;	/* fseek ptr for archives */
     int			ac_vol;		/* volume for ac_offset */
     int			ac_serial;	/* serial access pattern for archives */
+    int			ac_chkfeatures;	/* 1 => check featutre bits */
     __pmHashCtl		ac_pmid_hc;	/* per PMID controls for INTERP */
     double		ac_end;		/* time at end of archive */
     void		*ac_want;	/* used in interp.c */
@@ -867,6 +878,9 @@ typedef struct {
     __pmHashCtl	hashinst;		/* nested hash on inst for this indom */
 } __pmLogTrimInDom;
 
+PCP_CALL extern void __pmFreeLogInDom(__pmLogInDom *);
+PCP_CALL extern __pmLogInDom *__pmDupLogInDom(__pmLogInDom *);
+
 /*
  * Nested per-instance trimming control (potentially one of these for
  * _every_ instance in _every_ "large" indom), accessed as a hash using
@@ -891,7 +905,8 @@ typedef struct {
     __pmPMCDCtl		*c_pmcd;	/* pmcd control for HOST contexts */
     __pmArchCtl		*c_archctl;	/* log control for ARCHIVE contexts */
     __pmTimestamp	c_origin;	/* pmFetch time origin / current time */
-    int			c_delta;	/* for updating origin */
+    __pmTimestamp	c_delta;	/* for updating origin */
+    int			c_direction;	/* signedness of delta (-1/0/1) */
     int			c_sent;		/* profile has been sent to pmcd */
     pmProfile		*c_instprof;	/* instance profile */
     void		*c_dm;		/* derived metrics, if any */
@@ -907,43 +922,50 @@ typedef struct {
 #define __PM_MODE_MASK	0xffff
 
 /* internal archive routines */
-PCP_CALL extern int __pmLogVersion(__pmLogCtl *);
-PCP_CALL extern size_t __pmLogLabelSize(__pmLogCtl *);
+PCP_CALL extern int __pmLogVersion(const __pmLogCtl *);
+PCP_CALL extern size_t __pmLogLabelSize(const __pmLogCtl *);
 PCP_CALL extern int __pmLogChkLabel(__pmArchCtl *, __pmFILE *, __pmLogLabel *, int);
 PCP_CALL extern int __pmLogCreate(const char *, const char *, int, __pmArchCtl *);
 PCP_CALL extern __pmFILE *__pmLogNewFile(const char *, int);
 PCP_CALL extern void __pmLogClose(__pmArchCtl *);
 PCP_CALL extern int __pmLogPutDesc(__pmArchCtl *, const pmDesc *, int, char **);
-PCP_CALL extern int __pmLogPutInDom(__pmArchCtl *, pmInDom, const __pmTimestamp *, int, int *, char **);
+PCP_CALL extern int __pmLogPutInDom(__pmArchCtl *, int, const __pmLogInDom * const);
 PCP_CALL extern int __pmLogPutResult(__pmArchCtl *, __pmPDU *);
 PCP_CALL extern int __pmLogPutResult2(__pmArchCtl *, __pmPDU *);
+PCP_CALL extern int __pmLogPutResult3(__pmArchCtl *, __pmPDU *);
 PCP_CALL extern int __pmLogPutIndex(const __pmArchCtl *, const __pmTimestamp *);
 PCP_CALL extern int __pmLogLoadIndex(__pmLogCtl *);
+PCP_CALL extern int __pmLogEncodeLabels(__pmLogCtl *, unsigned int, unsigned int, int, pmLabelSet *, const __pmTimestamp *, __int32_t **);
 PCP_CALL extern int __pmLogPutLabels(__pmArchCtl *, unsigned int, unsigned int, int, pmLabelSet *, const __pmTimestamp *);
-PCP_CALL extern int __pmLogPutText(__pmArchCtl *, unsigned int , unsigned int, char *, int);
+PCP_CALL extern int __pmLogPutText(__pmArchCtl *, unsigned int, unsigned int, char *, int);
 PCP_CALL extern int __pmLogWriteLabel(__pmFILE *, const __pmLogLabel *);
+PCP_CALL extern int __pmLogWriteMark(__pmArchCtl *, const __pmTimestamp *, const __pmTimestamp *);
 PCP_CALL extern int __pmLogLoadMeta(__pmArchCtl *);
 PCP_CALL extern int __pmLogAddDesc(__pmArchCtl *, const pmDesc *);
-PCP_CALL extern int __pmLogAddInDom(__pmArchCtl *, const __pmTimestamp *, const pmInResult *, __int32_t *, int);
+PCP_CALL extern int __pmLogAddInDom(__pmArchCtl *, int, const __pmLogInDom *, __int32_t *);
+PCP_CALL extern int __pmLogEncodeInDom(__pmLogCtl *, int, const __pmLogInDom * const, __int32_t **);
+PCP_CALL extern __pmLogInDom *__pmLogSearchInDom(__pmLogCtl *, pmInDom, __pmTimestamp *);
+PCP_CALL extern void __pmLogUndeltaInDom(pmInDom, __pmLogInDom *);
 PCP_CALL extern int __pmLogAddPMNSNode(__pmArchCtl *, pmID, const char *);
 PCP_CALL extern int __pmLogAddLabelSets(__pmArchCtl *, const __pmTimestamp *, unsigned int, unsigned int, int, pmLabelSet *);
 PCP_CALL extern int __pmLogAddText(__pmArchCtl *, unsigned int, unsigned int, const char *);
 PCP_CALL extern int __pmLogAddVolume(__pmArchCtl *, unsigned int);
-
 #define PMLOGREAD_NEXT		0
 #define PMLOGREAD_TO_EOF	1
-PCP_CALL extern int __pmLogRead(__pmArchCtl *, int, __pmFILE *, pmResult **, int);
-PCP_CALL extern int __pmLogRead_ctx(__pmContext *, int, __pmFILE *, pmResult **, int);
+PCP_CALL extern int __pmLogRead(__pmArchCtl *, int, __pmFILE *, __pmResult **, int);
+PCP_CALL extern int __pmLogRead_ctx(__pmContext *, int, __pmFILE *, __pmResult **, int);
 PCP_CALL extern int __pmLogChangeVol(__pmArchCtl *, int);
-PCP_CALL extern int __pmLogFetch(__pmContext *, int, pmID *, pmResult **);
+PCP_CALL extern int __pmLogFetch(__pmContext *, int, pmID *, __pmResult **);
 PCP_CALL extern int __pmLogGetInDom(__pmArchCtl *, pmInDom, __pmTimestamp *, int **, char ***);
 PCP_CALL extern int __pmGetArchiveEnd(__pmArchCtl *, __pmTimestamp *);
 PCP_CALL extern int __pmLogLookupDesc(__pmArchCtl *, pmID, pmDesc *);
 #define PMLOGPUTINDOM_DUP       1
 PCP_CALL extern int __pmLogLookupInDom(__pmArchCtl *, pmInDom, __pmTimestamp *, const char *);
 PCP_CALL extern int __pmLogLookupLabel(__pmArchCtl *, unsigned int, unsigned int, pmLabelSet **, const __pmTimestamp *);
-PCP_CALL extern int __pmLogLookupText(__pmArchCtl *, unsigned int , unsigned int, char **);
+PCP_CALL extern int __pmLogLookupText(__pmArchCtl *, unsigned int, unsigned int, char **);
 PCP_CALL extern int __pmLogNameInDom(__pmArchCtl *, pmInDom, __pmTimestamp *, int, char **);
+PCP_CALL extern const char *__pmLogMetaTypeStr(int);
+PCP_CALL extern char *__pmLogMetaTypeStr_r(int, char *, int);
 PCP_CALL extern const char *__pmLogLocalSocketDefault(int, char *buf, size_t bufSize);
 PCP_CALL extern const char *__pmLogLocalSocketUser(int, char *buf, size_t bufSize);
 PCP_CALL extern int __pmLogCompressedSuffix(const char *);
@@ -961,11 +983,18 @@ PCP_CALL extern __pmContext *__pmHandleToPtr(int);
  */
 
 /* pmFetch helper routines, hooks for derivations and local contexts */
+PCP_CALL extern int __pmFetch(__pmContext *, int, pmID *, __pmResult **);
+PCP_CALL extern int __pmFetchLocal(__pmContext *, int, pmID *, __pmResult **);
+PCP_CALL extern int __pmFetchArchive(__pmContext *, __pmResult **);
 PCP_CALL extern int __pmPrepareFetch(__pmContext *, int, const pmID *, pmID **);
-PCP_CALL extern int __pmFinishResult(__pmContext *, int, pmResult **);
+PCP_CALL extern int __pmFinishResult(__pmContext *, int, __pmResult **);
 PCP_CALL extern int __pmFinishHighResResult(__pmContext *, int, pmHighResResult **);
-PCP_CALL extern int __pmFetchLocal(__pmContext *, int, pmID *, pmResult **);
-PCP_CALL extern int __pmHighResFetchLocal(__pmContext *, int, pmID *, pmHighResResult **);
+PCP_CALL extern int __pmFetchLocal(__pmContext *, int, pmID *, __pmResult **);
+PCP_CALL extern int __pmFetchHighResLocal(__pmContext *, int, pmID *, __pmResult **);
+PCP_CALL extern int __pmDecodeResult_ctx(__pmContext *, __pmPDU *, __pmResult **);
+PCP_CALL extern int __pmDecodeHighResResult_ctx(__pmContext *, __pmPDU *, __pmResult **);
+PCP_CALL extern void __pmGetResultSize(int, int, pmValueSet * const *, size_t *, size_t *);
+PCP_CALL extern void __pmSortInstances(__pmResult *);
 
 /* safely insert an atom value into a pmValue */
 PCP_CALL extern int __pmStuffValue(const pmAtomValue *, pmValue *, int);
@@ -1030,8 +1059,8 @@ typedef struct {
 #define PDU_LOG_REQUEST		0x8002
 #define PDU_LOG_STATUS		0x8003
 PCP_CALL extern int __pmConnectLogger(const char *, int *, int *);
-PCP_CALL extern int __pmSendLogControl(int, const pmResult *, int, int, int);
-PCP_CALL extern int __pmDecodeLogControl(const __pmPDU *, pmResult **, int *, int *, int *);
+PCP_CALL extern int __pmSendLogControl(int, const __pmResult *, int, int, int);
+PCP_CALL extern int __pmDecodeLogControl(const __pmPDU *, __pmResult **, int *, int *, int *);
 PCP_CALL extern int __pmSendLogRequest(int, int);
 PCP_CALL extern int __pmDecodeLogRequest(const __pmPDU *, int *);
 PCP_CALL extern int __pmSendLogStatus(int, __pmLoggerStatus *);
@@ -1042,7 +1071,7 @@ PCP_CALL extern void __pmFreeLogStatus(__pmLoggerStatus *, int);
 PCP_CALL extern int __pmLoggerTimeout(void);
 
 /* other interfaces shared by pmlc and pmlogger */
-PCP_CALL extern int __pmControlLog(int, const pmResult *, int, int, int, pmResult **);
+PCP_CALL extern int __pmControlLog(int, const __pmResult *, int, int, int, __pmResult **);
 #define PM_LOG_OFF		0	/* state */
 #define PM_LOG_MAYBE		1
 #define PM_LOG_ON		2
@@ -1270,11 +1299,14 @@ PCP_CALL extern char *__pmZoneinfo(void);
 /* string conversion to value of given type, suitable for pmStore */
 PCP_CALL extern int __pmStringValue(const char *, pmAtomValue *, int);
 
-/* timeval-based delays */
+/* time-based delays */
+PCP_CALL extern void __pmtimespecSleep(struct timespec);
+PCP_CALL extern void __pmtimespecPause(struct timespec);
 PCP_CALL extern void __pmtimevalSleep(struct timeval);
 PCP_CALL extern void __pmtimevalPause(struct timeval);
 
 /* manipulate internal timestamps */
+PCP_CALL extern int __pmGetTimestamp(__pmTimestamp *);
 PCP_CALL extern double __pmTimevalSub(const pmTimeval *, const pmTimeval *);
 PCP_CALL extern double __pmTimespecSub(const pmTimespec *, const pmTimespec *);
 PCP_CALL extern double __pmTimestampSub(const __pmTimestamp *, const __pmTimestamp *);
@@ -1284,9 +1316,14 @@ PCP_CALL extern int __pmTimestampCmp(const __pmTimestamp *, const __pmTimestamp
 
 /* reverse ctime, time interval parsing, time conversions */
 PCP_CALL extern int __pmParseCtime(const char *, struct tm *, char **);
-PCP_CALL extern int __pmParseTime(const char *, struct timeval *, struct timeval *,
-			 struct timeval *, char **);
-PCP_CALL extern int __pmConvertTime(struct tm *, struct timeval *, struct timeval *);
+PCP_CALL extern int __pmParseTime(const char *, struct timeval *,
+				struct timeval *, struct timeval *, char **);
+PCP_CALL extern int __pmParseHighResTime(const char *, struct timespec *,
+				struct timespec *, struct timespec *, char **);
+PCP_CALL extern int __pmConvertTime(struct tm *, struct timeval *,
+				struct timeval *);
+PCP_CALL extern int __pmConvertHighResTime(struct tm *, struct timespec *,
+				struct timespec *);
 PCP_CALL extern time_t __pmMktime(struct tm *);
 
 /* Query server features - used for expressing protocol capabilities */
@@ -1359,9 +1396,24 @@ PCP_CALL extern int __pmUnparseHostAttrsSpec(
 PCP_CALL extern int __pmUrlEncode(const char *inp, const size_t len, char **outp);
 PCP_CALL extern int __pmUrlDecode(const char *inp, const size_t len, char **outp);
 
-/* SSL/TLS/IPv6 support via NSS/NSPR */
-PCP_CALL extern int __pmSecureServerCertificateSetup(const char *, const char *, const char *);
-PCP_CALL extern void __pmSecureServerShutdown(void);
+/* TLS support via OpenSSL */
+typedef struct __pmSecureConfig {
+    char	*cacertfile;	/* authority file */
+    char	*cacertdir;	/* authority directory */
+    char	*certfile;	/* certificate chain */
+    char	*keyfile;	/* certificate key */
+    char	*ciphers;
+    char	*ciphersuites;	
+    char	*clientcertfile;/* certificate chain (client only) */
+    char	*clientkeyfile;	/* certificate key (client only) */
+    char	*clientverify;	/* client certificates (t/f) */
+} __pmSecureConfig;
+
+PCP_CALL extern void __pmSecureConfigInit(void);
+PCP_CALL extern int __pmGetSecureConfig(__pmSecureConfig *);
+PCP_CALL extern void __pmFreeSecureConfig(__pmSecureConfig *);
+PCP_CALL extern void *__pmSecureServerInit(__pmSecureConfig *);
+PCP_CALL extern void __pmSecureServerShutdown(void *, __pmSecureConfig *);
 PCP_CALL extern int __pmSecureServerHandshake(int, int, __pmHashCtl *);
 PCP_CALL extern int __pmSecureClientHandshake(int, int, const char *, __pmHashCtl *);
 
@@ -1383,14 +1435,14 @@ PCP_CALL extern __pmResult *__pmAllocResult(int);
  * after, the start of the __pmResult (rp)
  */
 static inline pmResult *
-__pmOffsetResult(__pmResult *rp)
+__pmOffsetResult(const __pmResult *rp)
 {
    return (pmResult *)(&((char *)rp)[offsetof(__pmResult,numpmid) - offsetof(pmResult,numpmid)]);
 }
 
 /* same for pmHighResResult ... */
 static inline pmHighResResult *
-__pmOffsetHighResResult(__pmResult *rp)
+__pmOffsetHighResResult(const __pmResult *rp)
 {
    return (pmHighResResult *)(&((char *)rp)[offsetof(__pmResult,numpmid) - offsetof(pmHighResResult,numpmid)]);
 }
@@ -1405,6 +1457,7 @@ PCP_CALL extern void __pmFreeProfile(pmProfile *);
 PCP_CALL extern void __pmFreeResultValues(pmResult *);
 PCP_CALL extern void __pmFreeHighResResultValues(pmHighResResult *);
 PCP_CALL extern void __pmFreeDerived(__pmContext *);
+PCP_CALL extern void __pmFreeResult(__pmResult *);
 
 /* diagnostics for formatting or printing miscellaneous data structures */
 PCP_CALL extern void __pmDumpContext(FILE *, int, pmInDom);
@@ -1421,7 +1474,8 @@ PCP_CALL extern void __pmDumpNameNode(FILE *, const __pmnsNode *, int);
 PCP_CALL extern void __pmDumpNameSpace(FILE *, int);
 PCP_CALL extern void __pmDumpResult(FILE *, const pmResult *);
 PCP_CALL extern void __pmPrintResult(FILE *, const __pmResult *);
-PCP_CALL extern void __pmDumpStack(FILE *);
+PCP_CALL extern void __pmDumpStack(void);
+PCP_CALL extern void __pmDumpStackInit(void *);
 PCP_CALL extern void __pmDumpStatusList(FILE *, int, const int *);
 PCP_CALL extern void __pmPrintTimeval(FILE *, const pmTimeval *);
 PCP_CALL extern void __pmPrintTimespec(FILE *, const pmTimespec *);
@@ -1436,6 +1490,7 @@ PCP_CALL extern const char *__pmLabelTypeString(int);
 PCP_CALL extern const char *__pmGetLabelConfigHostName(char *, size_t);
 PCP_CALL extern const char *__pmGetLabelConfigMachineID(char *, size_t);
 PCP_CALL extern const char *__pmGetLabelConfigDomainName(char *, size_t);
+PCP_CALL extern char *__pmLogFeaturesStr(__uint32_t);
 
 /* log file rotation */
 PCP_CALL extern FILE *__pmRotateLog(const char *, const char *, FILE *, int *);
@@ -1471,9 +1526,11 @@ PCP_CALL extern void __pmFreeHighResResult(pmHighResResult *);
 /*
  * Loading archive records from disk ...
  */
-PCP_CALL extern int __pmLogLoadInDom(__pmArchCtl *, int, int, pmInResult *, __pmTimestamp *, __int32_t **);
+PCP_CALL extern int __pmLogLoadInDom(__pmArchCtl *, int, int, __pmLogInDom *, __int32_t **);
 PCP_CALL extern int __pmLogLoadLabel(__pmFILE *, __pmLogLabel *);
 PCP_CALL extern void __pmLogFreeLabel(__pmLogLabel *);
+PCP_CALL extern int __pmLogLoadLabelSet(char *, int, int, __pmTimestamp *,
+					int *, int *, int *, pmLabelSet **);
 
 /*
  * Routines to pack (Put) and unpack (Load) each timestamp format
