@@ -451,20 +451,45 @@ static const char *disabled(void) { return "false"; }
 
 #define STRINGIFY(s)		#s
 #define TO_STRING(s)		STRINGIFY(s)
-static const char *pmapi_version(void) { return TO_STRING(PMAPI_VERSION); }
+static const char *pmapi_version(void) { return TO_STRING(PMAPI_VERSION_3); }
 static const char *pcp_version(void) { return PCP_VERSION; }
 #if defined(HAVE_SECURE_SOCKETS)
-#include "nss.h"
-#include "nspr.h"
-#include "sasl.h"
-static const char *nspr_version(void) { return PR_VERSION; }
-static const char *nss_version(void) { return NSS_VERSION; }
+#include <openssl/opensslv.h>
+#include <sasl/sasl.h>
+static const char *openssl_version(void)
+{
+#ifdef OPENSSL_VERSION_STR
+    return OPENSSL_VERSION_STR;
+#else /* back-compat and not ideal, includes date */
+    return OPENSSL_VERSION_TEXT;
+#endif
+}
 static const char *sasl_version_string(void)
 {
     return TO_STRING(SASL_VERSION_MAJOR.SASL_VERSION_MINOR.SASL_VERSION_STEP);
 }
 #endif
 
+static const char *
+myfeatures(void)
+{
+    static char	*arch_features = NULL;
+    /*
+     * cheat a little using __pmLock_extcall ... just want mutual exclusion
+     * here to call __pmLogFeaturesStr() once and only once
+     */
+    PM_LOCK(__pmLock_extcall);
+    if (arch_features == NULL) {
+	 /*
+	  * hide the "QA" feature ...
+	  */
+	 arch_features = __pmLogFeaturesStr(PM_LOG_FEATURES & ~PM_LOG_FEATURE_QA);
+    }
+    PM_UNLOCK(__pmLock_extcall);
+
+    return arch_features;
+}
+
 static const char *
 ipv6_enabled(void)
 {
@@ -546,8 +571,7 @@ static struct {
 	{ "pcp_version",	pcp_version },
 	{ "pmapi_version",	pmapi_version },
 #if defined(HAVE_SECURE_SOCKETS)
-	{ "nss_version",	nss_version },
-	{ "nspr_version",	nspr_version },
+	{ "openssl_version",	openssl_version },
 	{ "sasl_version",	sasl_version_string },
 #endif
 	{ "multi_threaded",	MULTI_THREAD_ENABLED },
@@ -564,6 +588,8 @@ static struct {
 	{ "lzma_decompress",	LZMA_DECOMPRESS },		/* from pcp-4.0.0 */
 	{ "transparent_decompress", TRANSPARENT_DECOMPRESS },	/* from pcp-4.0.0 */
 	{ "compress_suffixes",	compress_suffix_list },		/* from pcp-4.0.1 */
+	{ "v3_archives",	enabled },			/* from pcp-6.0.0 */
+	{ "archive_features",	myfeatures },			/* from pcp-6.0.0 */
 };
 
 void
