@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2017,2020-2021 Red Hat.
+ * Copyright (c) 2012-2017,2020-2022 Red Hat.
  * Copyright (c) 1995-2002,2004 Silicon Graphics, Inc.  All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or modify it
@@ -48,12 +48,10 @@ typedef struct {
  */
 static __pmHashCtl	pc_hc;
 
-static int LogCheckForNextArchive(__pmContext *, int, pmResult **);
+static int LogCheckForNextArchive(__pmContext *, int, __pmResult **);
 static int LogChangeToNextArchive(__pmContext *);
 static int LogChangeToPreviousArchive(__pmContext *);
 
-static void logFreeLabel(__pmLogLabel *);
-
 #ifdef PM_MULTI_THREAD
 static pthread_mutex_t	logutil_lock = PTHREAD_MUTEX_INITIALIZER;
 #else
@@ -125,7 +123,7 @@ __pmLogLabelVersion(const __pmLogLabel *llp)
 }
 
 int
-__pmLogVersion(__pmLogCtl *lcp)
+__pmLogVersion(const __pmLogCtl *lcp)
 {
     return __pmLogLabelVersion(&lcp->label);
 }
@@ -170,12 +168,13 @@ _logpeek(__pmArchCtl *acp, int vol)
     }
     PM_UNLOCK(logutil_lock);
 
-    if ((sts = __pmLogChkLabel(acp, f, &label, vol)) < 0) {
+    sts = __pmLogChkLabel(acp, f, &label, vol);
+    __pmLogFreeLabel(&label);
+    if (sts < 0) {
 	__pmFclose(f);
 	setoserror(sts);
 	return NULL;
     }
-    logFreeLabel(&label);
 
     return f;
 }
@@ -378,10 +377,28 @@ logFreeHashInDom(__pmHashCtl *hcp)
 	for (hp = hcp->hash[i], prior_hp = NULL; hp != NULL; hp = hp->next) {
 	    for (idp = (__pmLogInDom *)hp->data, prior_idp = NULL;
 		idp != NULL; idp = idp->next) {
+		if ((idp->alloc & ~(PMLID_SELF|PMLID_INSTLIST|PMLID_NAMELIST|PMLID_NAMES)) != 0) {
+		    fprintf(stderr, "logFreeHashInDom(%p): Warning: bogus alloc flags: 0x%x for idp=%p\n",
+			hcp, idp->alloc & ~(PMLID_SELF|PMLID_INSTLIST|PMLID_NAMELIST|PMLID_NAMES), idp);
+		}
+
 		if (idp->buf != NULL)
 		    free(idp->buf);
-		if (idp->allinbuf == 0 && idp->namelist != NULL)
+		if (idp->numinst >= 0) {
+		    if (idp->alloc & PMLID_NAMES && idp->namelist != NULL) {
+			int		j;
+			for (j = 0; j < idp->numinst; j++) {
+			    if (idp->namelist[j] != NULL) {
+				free(idp->namelist[j]);
+				idp->namelist[j] = NULL;
+			    }
+			}
+		    }
+		}
+		if (idp->alloc & PMLID_NAMELIST && idp->namelist != NULL)
 		    free(idp->namelist);
+		if (idp->alloc & PMLID_INSTLIST && idp->instlist != NULL)
+		    free(idp->instlist);
 		if (prior_idp != NULL)
 		    free(prior_idp);
 		prior_idp = idp;
@@ -523,29 +540,10 @@ logFreeHashText(__pmHashCtl *type_ctl)
     free(type_ctl->hash);
 }
 
-static void
-logFreeLabel(__pmLogLabel *label)
-{
-    if (label->hostname != NULL) {
-	free(label->hostname);
-	label->hostname = NULL;
-    }
-
-    if (label->timezone != NULL) {
-	free(label->timezone);
-	label->timezone = NULL;
-    }
-
-    if (label->zoneinfo != NULL) {
-	free(label->zoneinfo);
-	label->zoneinfo = NULL;
-    }
-}
-
 static void
 logFreeMeta(__pmLogCtl *lcp)
 {
-    logFreeLabel(&lcp->label);
+    __pmLogFreeLabel(&lcp->label);
 
     if (lcp->pmns != NULL) {
 	__pmFreePMNS(lcp->pmns);
@@ -792,12 +790,24 @@ __pmLogOpen(const char *name, __pmContext *ctxp)
     int		version;
     int		sts;
 
-    if ((sts = __pmLogFindOpen(ctxp->c_archctl, name)) < 0)
+    if ((sts = __pmLogFindOpen(ctxp->c_archctl, name)) < 0) {
+	if (pmDebugOptions.log && pmDebugOptions.desperate) {
+	    char	errmsg[PM_MAXERRMSGLEN];
+	    fprintf(stderr, "__pmLogOpen(..., %s, ...): __pmLogFindOpen: %s\n",
+		name, pmErrStr_r(sts, errmsg, sizeof(errmsg)));
+	}
 	return sts;
+    }
 
     acp->ac_curvol = -1;
-    if ((sts = __pmLogChangeVol(acp, lcp->minvol)) < 0)
+    if ((sts = __pmLogChangeVol(acp, lcp->minvol)) < 0) {
+	if (pmDebugOptions.log && pmDebugOptions.desperate) {
+	    char	errmsg[PM_MAXERRMSGLEN];
+	    fprintf(stderr, "__pmLogOpen(..., %s, ...): __pmLogChangeVol: %s\n",
+		name, pmErrStr_r(sts, errmsg, sizeof(errmsg)));
+	}
 	goto cleanup;
+    }
     else
 	version = sts;
 
@@ -805,8 +815,14 @@ __pmLogOpen(const char *name, __pmContext *ctxp)
 
     if (lcp->tifp) {
 	sts = __pmLogChkLabel(acp, lcp->tifp, &label, PM_LOG_VOL_TI);
-	if (sts < 0)
+	if (sts < 0) {
+	    if (pmDebugOptions.log && pmDebugOptions.desperate) {
+		char	errmsg[PM_MAXERRMSGLEN];
+		fprintf(stderr, "__pmLogOpen(..., %s, ...): __pmLogChkLabel TI: %s\n",
+		    name, pmErrStr_r(sts, errmsg, sizeof(errmsg)));
+	    }
 	    goto cleanup;
+	}
 	if (sts != version) {
 	    /* mismatch between meta & actual data versions! */
 	    sts = PM_ERR_LABEL;
@@ -818,11 +834,17 @@ __pmLogOpen(const char *name, __pmContext *ctxp)
 	    sts = PM_ERR_LABEL;
 	    goto cleanup;
 	}
-	logFreeLabel(&label);
+	__pmLogFreeLabel(&label);
     }
 
-    if ((sts = __pmLogChkLabel(acp, lcp->mdfp, &label, PM_LOG_VOL_META)) < 0)
+    if ((sts = __pmLogChkLabel(acp, lcp->mdfp, &label, PM_LOG_VOL_META)) < 0) {
+	if (pmDebugOptions.log && pmDebugOptions.desperate) {
+	    char	errmsg[PM_MAXERRMSGLEN];
+	    fprintf(stderr, "__pmLogOpen(..., %s, ...): __pmLogChkLabel META: %s\n",
+		name, pmErrStr_r(sts, errmsg, sizeof(errmsg)));
+	}
 	goto cleanup;
+    }
     else if (sts != version) {	/* version mismatch between meta & ti */
 	sts = PM_ERR_LABEL;
 	goto cleanup;
@@ -832,21 +854,39 @@ __pmLogOpen(const char *name, __pmContext *ctxp)
      * Perform consistency checks between this label and the labels of other
      * archives possibly making up this context.
      */
-    if ((sts = checkLabelConsistency(ctxp, &lcp->label)) < 0)
+    if ((sts = checkLabelConsistency(ctxp, &lcp->label)) < 0) {
+	if (pmDebugOptions.log && pmDebugOptions.desperate) {
+	    char	errmsg[PM_MAXERRMSGLEN];
+	    fprintf(stderr, "__pmLogOpen(..., %s, ...): checkLabelConsistency: %s\n",
+		name, pmErrStr_r(sts, errmsg, sizeof(errmsg)));
+	}
 	goto cleanup;
+    }
 
-    if ((sts = __pmLogLoadMeta(acp)) < 0)
+    if ((sts = __pmLogLoadMeta(acp)) < 0) {
+	if (pmDebugOptions.log && pmDebugOptions.desperate) {
+	    char	errmsg[PM_MAXERRMSGLEN];
+	    fprintf(stderr, "__pmLogOpen(..., %s, ...): __pmLogLoadMeta: %s\n",
+		name, pmErrStr_r(sts, errmsg, sizeof(errmsg)));
+	}
 	goto cleanup;
+    }
 
-    if ((sts = __pmLogLoadIndex(lcp)) < 0)
+    if ((sts = __pmLogLoadIndex(lcp)) < 0) {
+	if (pmDebugOptions.log && pmDebugOptions.desperate) {
+	    char	errmsg[PM_MAXERRMSGLEN];
+	    fprintf(stderr, "__pmLogOpen(..., %s, ...): __pmLogLoadIndex: %s\n",
+		name, pmErrStr_r(sts, errmsg, sizeof(errmsg)));
+	}
 	goto cleanup;
+    }
 
     if (lcp->label.pid != label.pid ||
 	strcmp(lcp->label.hostname, label.hostname) != 0) {
 	    sts = PM_ERR_LABEL;
 	    goto cleanup;
     }
-    logFreeLabel(&label);
+    __pmLogFreeLabel(&label);
 
     PM_LOCK(lcp->lc_lock);
     lcp->refcnt = 0;
@@ -859,7 +899,7 @@ __pmLogOpen(const char *name, __pmContext *ctxp)
 
 cleanup:
     __pmLogClose(acp);
-    logFreeLabel(&label);
+    __pmLogFreeLabel(&label);
     logFreeMeta(lcp);
     return sts;
 }
@@ -888,23 +928,24 @@ logputresult(int version, __pmArchCtl *acp, __pmPDU *pb)
      *
      * If version == 1, pb[] does not have room for trailer len.
      * If version == 2, pb[] does have room for trailer len.
+     * If version == 3, like 2 but different timestamp and into a v3 archive.
      */
     __pmLogCtl		*lcp = acp->ac_log;
+    __pmPDU		*start = &pb[2];
     int			sz;
     int			sts = 0;
     int			save_from;
-    __pmPDU		*start = &pb[2];
 
     if (lcp->state == PM_LOG_STATE_NEW) {
-	int		i;
-	pmTimeval	*tvp; /* TODO: Y2038 */
 	/*
 	 * first result, do the label record
+	 *
+	 * pb[3] => 3 __int32_t's before the timestamp starts
 	 */
-	i = sizeof(__pmPDUHdr) / sizeof(__pmPDU);
-	tvp = (pmTimeval *)&pb[i];
-	lcp->label.start.sec = ntohl(tvp->tv_sec);
-	lcp->label.start.nsec = ntohl(tvp->tv_usec) * 1000;
+	if (version >= 3)
+	    __pmLoadTimestamp((__int32_t *)&pb[3], &lcp->label.start);
+	else
+	    __pmLoadTimeval((__int32_t *)&pb[3], &lcp->label.start);
 	lcp->label.vol = PM_LOG_VOL_TI;
 	__pmLogWriteLabel(lcp->tifp, &lcp->label);
 	lcp->label.vol = PM_LOG_VOL_META;
@@ -942,12 +983,12 @@ logputresult(int version, __pmArchCtl *acp, __pmPDU *pb)
 	}
     }
     else {
-	/* assume version == 2 */
+	/* version == 2 or version == 3 */
 	start[(sz-1)/sizeof(__pmPDU)] = start[0];
 	if ((sts = __pmFwrite(start, 1, sz, acp->ac_mfp)) != sz) {
 	    char	errmsg[PM_MAXERRMSGLEN];
-	    pmprintf("__pmLogPutResult2: write failed: returns %d expecting %d: %s\n",
-	    	sts, sz, osstrerror_r(errmsg, sizeof(errmsg)));
+	    pmprintf("__pmLogPutResult%d: write failed: returns %d expecting %d: %s\n",
+	    	version, sts, sz, osstrerror_r(errmsg, sizeof(errmsg)));
 	    pmflush();
 	    sts = -oserror();
 	}
@@ -979,13 +1020,23 @@ __pmLogPutResult2(__pmArchCtl *acp, __pmPDU *pb)
     return logputresult(2, acp, pb);
 }
 
+/*
+ * new routine, pb[] does have room for trailer, so only 1 write
+ * needed and this time we generate v3 archives
+ */
+int
+__pmLogPutResult3(__pmArchCtl *acp, __pmPDU *pb)
+{
+    return logputresult(3, acp, pb);
+}
+
 /*
  * check if PDU buffer seems even half-way reasonable ...
  * only used when trying to locate end of archive.
  * return 0 for OK, -1 for bad.
  */
 static int
-paranoidCheck(int len, __pmPDU *pb)
+paranoidCheck(int len, int version, __pmPDU *pb)
 {
     int			numpmid;
     size_t		hdrsz;		/* bytes for the PDU head+tail */
@@ -995,45 +1046,66 @@ paranoidCheck(int len, __pmPDU *pb)
     int			vbsize;		/* size of pmValueBlocks */
     int			numval;		/* number of values */
     int			valfmt;
+    __pmPDU		*data;
 
     struct result_t {			/* from p_result.c */
 	__pmPDUHdr		hdr;
 	pmTimeval		timestamp;	/* when returned */
 	int			numpmid;	/* no. of PMIDs to follow */
 	__pmPDU			data[1];	/* zero or more */
-    }			*pp;
+    } *rp;
+
+    struct log_result_v3_t {		/* from p_result.c */
+	__int32_t	len;		/* PDU header -- repacked */
+	__int32_t	type;		/* ditto */
+	__int32_t	from;		/* ditto -- becomes len */
+	__int32_t	sec[2];		/* __pmTimestamp */
+	__int32_t	nsec;
+	__int32_t	numpmid;
+	__int32_t	data[1];	/* zero or more */
+    } *v3rp;
+
+    
     struct vlist_t {			/* from p_result.c */
-	pmID			pmid;
-	int			numval;		/* no. of vlist els to follow, or error */
-	int			valfmt;		/* insitu or pointer */
-	__pmValue_PDU		vlist[1];	/* zero or more */
-    }			*vlp;
+	pmID		pmid;
+	int		numval;		/* no. of vlist els to follow, or error */
+	int		valfmt;		/* insitu or pointer */
+	__pmValue_PDU	vlist[1];	/* zero or more */
+    } *vlp;
 
     /*
-     * to start with, need space for result_t with no data (__pmPDU)
+     * to start with, need space for *result_t with no data (__pmPDU)
      * ... this is the external size, which consists of
      * <header len>
-     * <timestamp> (2 words)
+     * <timestamp>
      * <numpmid>
      * <trailer len>
      *
-     * it is confusing because *pb and result_t include the fake
+     * it is confusing because *pb and *result_t include the fake
      * __pmPDUHdr which is not really in the external file
      */
-    hdrsz = 5 * sizeof(__pmPDU);
+
+    if (version >= PM_LOG_VERS03) {
+	v3rp = (struct log_result_v3_t *)pb;
+	numpmid = ntohl(v3rp->numpmid);
+	hdrsz = 6 * sizeof(__int32_t);
+	data = (__pmPDU *)v3rp->data;
+    } else {
+	rp = (struct result_t *)pb;
+	numpmid = ntohl(rp->numpmid);
+	hdrsz = 5 * sizeof(__pmPDU);
+	data = rp->data;
+    }
 
     if (len < hdrsz) {
 	if (pmDebugOptions.log) {
-	    fprintf(stderr, "\nparanoidCheck: len=%d, min len=%d\n",
-		len, (int)hdrsz);
-	    dumpbuf(len, &pb[3]); /* skip first 3 words, start @ timestamp */
+	    fprintf(stderr, "\nparanoidCheck: len=%d, min len=%zu\n",
+		len, hdrsz);
+	    dumpbuf(len, &pb[3]); /* skip first 3 words (PDU header) */
 	}
 	return -1;
     }
 
-    pp = (struct result_t *)pb;
-    numpmid = ntohl(pp->numpmid);
-
     /*
      * This is a re-implementation of much of __pmDecodeResult()
      */
@@ -1043,7 +1115,7 @@ paranoidCheck(int len, __pmPDU *pb)
 	    if (pmDebugOptions.log) {
 		fprintf(stderr, "\nparanoidCheck: numpmid=%d len=%d, expected len=%d\n",
 		    numpmid, len, (int)hdrsz);
-		dumpbuf(len, &pb[3]); /* skip first 3 words, start @ timestamp */
+		dumpbuf(len, &pb[3]); /* skip first 3 words (PDU header) */
 	    }
 	    return -1;
 	}
@@ -1060,13 +1132,13 @@ paranoidCheck(int len, __pmPDU *pb)
 
     vsize = vbsize = 0;
     for (i = 0; i < numpmid; i++) {
-	vlp = (struct vlist_t *)&pp->data[vsize/sizeof(__pmPDU)];
+	vlp = (struct vlist_t *)&data[vsize/sizeof(__pmPDU)];
 	vsize += sizeof(vlp->pmid) + sizeof(vlp->numval);
 	if (len < hdrsz + vsize + vbsize) {
 	    if (pmDebugOptions.log) {
 		fprintf(stderr, "\nparanoidCheck: vset[%d] len=%d, need len>=%d (%d+%d+%d)\n",
 		    i, len, (int)(hdrsz + vsize + vbsize), (int)hdrsz, vsize, vbsize);
-		dumpbuf(len, &pb[3]); /* skip first 3 words, start @ timestamp */
+		dumpbuf(len, &pb[3]); /* skip first 3 words (PDU header) */
 	    }
 	    return -1;
 	}
@@ -1082,7 +1154,7 @@ paranoidCheck(int len, __pmPDU *pb)
 		if (pmDebugOptions.log) {
 		    fprintf(stderr, "\nparanoidCheck: vset[%d] bad valfmt=%d\n",
 			i, valfmt);
-		    dumpbuf(len, &pb[3]); /* skip first 3 words, start @ timestamp */
+		    dumpbuf(len, &pb[3]); /* skip first 3 words (PDU header) */
 		}
 		return -1;
 	    }
@@ -1106,7 +1178,7 @@ paranoidCheck(int len, __pmPDU *pb)
 			if (pmDebugOptions.log) {
 			    fprintf(stderr, "\nparanoidCheck: vset[%d] val[%d], bad pval index=%d not in range 0..%d\n",
 				i, j, index, (int)(len / sizeof(__pmPDU)));
-			    dumpbuf(len, &pb[3]); /* skip first 3 words, start @ timestamp */
+			    dumpbuf(len, &pb[3]); /* skip first 3 words (PDU header) */
 			}
 			return -1;
 		    }
@@ -1118,7 +1190,7 @@ paranoidCheck(int len, __pmPDU *pb)
 			if (pmDebugOptions.log) {
 			    fprintf(stderr, "\nparanoidCheck: vset[%d] val[%d], bad vlen=%d\n",
 				i, j, vlen);
-			    dumpbuf(len, &pb[3]); /* skip first 3 words, start @ timestamp */
+			    dumpbuf(len, &pb[3]); /* skip first 3 words (PDU header) */
 			}
 			return -1;
 		    }
@@ -1132,7 +1204,7 @@ paranoidCheck(int len, __pmPDU *pb)
 }
 
 static int
-paranoidLogRead(__pmContext *ctxp, int mode, __pmFILE *peekf, pmResult **result)
+paranoidLogRead(__pmContext *ctxp, int mode, __pmFILE *peekf, __pmResult **result)
 {
     return __pmLogRead_ctx(ctxp, mode, peekf, result, PMLOGREAD_TO_EOF);
 }
@@ -1148,25 +1220,19 @@ paranoidLogRead(__pmContext *ctxp, int mode, __pmFILE *peekf, pmResult **result)
  */
 
 int
-__pmLogGenerateMark_ctx(__pmContext *ctxp, int mode, pmResult **result)
+__pmLogGenerateMark_ctx(__pmContext *ctxp, int mode, __pmResult **result)
 {
     __pmLogCtl		*lcp = ctxp->c_archctl->ac_log;
-#if 0		// TODO when *result => __pmResult
     __pmResult		*pr;
-#else
-    pmResult		*pr;
-    __pmResult		*__pr;
-#endif
     int			sts;
     __pmTimestamp	end;
 
     PM_ASSERT_IS_LOCKED(ctxp->c_lock);
 
-    if ((__pr = __pmAllocResult(0)) == NULL) {
+    if ((pr = __pmAllocResult(0)) == NULL) {
 	pmNoMem("generateMark", sizeof(__pmResult), PM_FATAL_ERR);
 	/* NOTREACHED */
     }
-    pr = __pmOffsetResult(__pr);
 
     /*
      * A mark record has numpmid == 0 and the timestamp set to one millisecond
@@ -1178,22 +1244,20 @@ __pmLogGenerateMark_ctx(__pmContext *ctxp, int mode, pmResult **result)
 	    free(pr);
 	    return sts;
 	}
-	pr->timestamp.tv_sec = lcp->endtime.sec;
-	pr->timestamp.tv_usec = lcp->endtime.nsec / 1000;
-	pr->timestamp.tv_usec += 1000;
-	if (pr->timestamp.tv_usec > 1000000) {
-	    pr->timestamp.tv_usec -= 1000000;
-	    pr->timestamp.tv_sec++;
+	pr->timestamp = lcp->endtime;
+	pr->timestamp.nsec += 1000000;
+	if (pr->timestamp.nsec > 1000000000) {
+	    pr->timestamp.nsec -= 1000000000;
+	    pr->timestamp.sec++;
 	}
     }
     else {
-	pr->timestamp.tv_sec = lcp->label.start.sec;
-	pr->timestamp.tv_usec = lcp->label.start.nsec / 1000;
-	if (pr->timestamp.tv_usec >= 1000)
-	    pr->timestamp.tv_usec -= 1000;
+	pr->timestamp = lcp->label.start;
+	if (pr->timestamp.nsec >= 1000000)
+	    pr->timestamp.nsec -= 1000000;
 	else {
-	    pr->timestamp.tv_usec = 1000000 - 1000 + pr->timestamp.tv_usec;;
-	    pr->timestamp.tv_sec--;
+	    pr->timestamp.nsec = 1000000000 - 1000000 + pr->timestamp.nsec;
+	    pr->timestamp.sec--;
 	}
     }
     *result = pr;
@@ -1201,7 +1265,7 @@ __pmLogGenerateMark_ctx(__pmContext *ctxp, int mode, pmResult **result)
 }
 
 int
-__pmLogGenerateMark(__pmLogCtl *lcp, int mode, pmResult **result)
+__pmLogGenerateMark(__pmLogCtl *lcp, int mode, __pmResult **result)
 {
     int		sts;
     __pmContext	*ctxp;
@@ -1230,6 +1294,71 @@ clearMarkDone(__pmContext *ctxp)
     }
 }
 
+/*
+ * Output a <mark> record ... last_stamp is the base timestamp and
+ * the timestamp of the <mark> record is last_stamp + inc (if inc
+ * is not NULL) else last_stamp.
+ *
+ * archive <mark> record is an array of __int32_t values
+ * V2			V3
+ * [0] len (header)	[0] len (header)
+ * [1] sec		[1]&[2] sec
+ * [2] nsec		[3] nsec
+ * [3] numpmid (0)	[4] numpmid (0)
+ * [4] len (trailer)	[5] len (trailer)
+ */
+int
+__pmLogWriteMark(__pmArchCtl *acp, const __pmTimestamp *last_stamp, const __pmTimestamp *inc)
+{
+    __int32_t		buf[6];		/* enough for V3 */
+    int			vers;
+    size_t		rlen;
+    int			k = 0;
+    int			sts;
+    __pmTimestamp	stamp;
+
+    stamp = *last_stamp;		/* struct assignment */
+    if (inc != NULL) {
+	/* optional increment */
+	__pmTimestampInc(&stamp, inc);
+    }
+
+    vers = __pmLogVersion(acp->ac_log);
+
+    if (vers >= PM_LOG_VERS03)
+	rlen = 6 * sizeof(__int32_t);
+    else
+	rlen = 5 * sizeof(__int32_t);
+
+    buf[k++] = htonl(rlen);	/* header len */
+
+    if (vers >= PM_LOG_VERS03) {
+	__pmPutTimestamp(&stamp, &buf[k]);
+	k += 3;
+    }
+    else {
+	__pmPutTimeval(&stamp, &buf[k]);
+	k += 2;
+    }
+
+    buf[k++] = 0;		/* numpmid */
+    buf[k] = buf[0];		/* trailer len */
+
+    sts = (int)__pmFwrite((__pmPDU *)buf, 1, rlen, acp->ac_mfp);
+    if (sts != rlen) {
+	if (pmDebugOptions.log) {
+	    char	errmsg[PM_MAXERRMSGLEN];
+	    fprintf(stderr, "__pmLogWriteMark: version %d write failed: returns %d expecting %zd: %s\n",
+		vers, sts, rlen, osstrerror_r(errmsg, sizeof(errmsg)));
+	}
+	sts = -oserror();
+    }
+    else
+	sts = 0;
+
+    return sts;
+}
+
 /*
  * read next forward or backward from the log
  *
@@ -1243,7 +1372,7 @@ clearMarkDone(__pmContext *ctxp)
  * can be carried down the call stack.
  */
 int
-__pmLogRead_ctx(__pmContext *ctxp, int mode, __pmFILE *peekf, pmResult **result, int option)
+__pmLogRead_ctx(__pmContext *ctxp, int mode, __pmFILE *peekf, __pmResult **result, int option)
 {
     __pmLogCtl	*lcp;
     __pmArchCtl	*acp;
@@ -1255,6 +1384,7 @@ __pmLogRead_ctx(__pmContext *ctxp, int mode, __pmFILE *peekf, pmResult **result,
     __pmPDU	*pb;
     __pmFILE	*f;
     int		n;
+    int		version;
     ctx_ctl_t	ctx_ctl = { NULL, 0 };
 
     sts = lock_ctx(ctxp, &ctx_ctl);
@@ -1263,6 +1393,7 @@ __pmLogRead_ctx(__pmContext *ctxp, int mode, __pmFILE *peekf, pmResult **result,
 
     lcp = ctxp->c_archctl->ac_log;
     acp = ctxp->c_archctl;
+    version = __pmLogLabelVersion(&lcp->label);
 
     /*
      * Strip any XTB data from mode, its not used here
@@ -1501,9 +1632,8 @@ again:
 	/* swab PDU buffer - done later in __pmDecodeResult */
 
 	if (pmDebugOptions.pdu && pmDebugOptions.desperate) {
-	    int	j;
+	    int		j, jend = PM_PDU_SIZE(header->len);
 	    char	*p;
-	    int	jend = PM_PDU_SIZE(header->len);
 
 	    /* clear the padding bytes, lest they contain garbage */
 	    p = (char *)pb + header->len;
@@ -1553,7 +1683,7 @@ again:
 	goto func_return;
     }
 
-    if (option == PMLOGREAD_TO_EOF && paranoidCheck(head, pb) == -1) {
+    if (option == PMLOGREAD_TO_EOF && paranoidCheck(head, version, pb) == -1) {
 	__pmUnpinPDUBuf(pb);
 	sts = PM_ERR_LOGREC;
 	goto func_return;
@@ -1569,12 +1699,9 @@ again:
 	head -= sizeof(head) + sizeof(trail);
 	if (sts >= 0) {
 	    double delta;
-	    __pmTimestamp stamp;
 	    fprintf(stderr, "@");
-	    pmPrintStamp(stderr, &(*result)->timestamp);
-	    stamp.sec = (__int64_t)(*result)->timestamp.tv_sec;
-	    stamp.nsec = (__int32_t)(*result)->timestamp.tv_usec * 1000;
-	    delta = __pmTimestampSub(&stamp, &lcp->label.start);
+	    __pmPrintTimestamp(stderr, &(*result)->timestamp);
+	    delta = __pmTimestampSub(&(*result)->timestamp, &lcp->label.start);
 	    fprintf(stderr, " (t=%.6f)", delta);
 	}
 	else {
@@ -1596,7 +1723,7 @@ again:
 
     if (pmDebugOptions.pdu) {
 	fprintf(stderr, "__pmLogRead timestamp=");
-	pmPrintStamp(stderr, &(*result)->timestamp);
+	__pmPrintTimestamp(stderr, &(*result)->timestamp);
 	fprintf(stderr, " " PRINTF_P_PFX "%p ... " PRINTF_P_PFX "%p", &pb[3], &pb[head/sizeof(__pmPDU)+3]);
 	fputc('\n', stderr);
 	dumpbuf(rlen, &pb[3]);		/* see above to explain "3" */
@@ -1614,7 +1741,7 @@ func_return:
 }
 
 int
-__pmLogRead(__pmArchCtl *acp, int mode, __pmFILE *peekf, pmResult **result, int option)
+__pmLogRead(__pmArchCtl *acp, int mode, __pmFILE *peekf, __pmResult **result, int option)
 {
     int		sts;
     __pmContext	*ctxp;
@@ -1654,7 +1781,7 @@ check_all_derived(int numpmid, pmID pmidlist[])
 }
 
 int
-__pmLogFetch(__pmContext *ctxp, int numpmid, pmID pmidlist[], pmResult **result)
+__pmLogFetch(__pmContext *ctxp, int numpmid, pmID pmidlist[], __pmResult **result)
 {
     int		i;
     int		j;
@@ -1663,12 +1790,7 @@ __pmLogFetch(__pmContext *ctxp, int numpmid, pmID pmidlist[], pmResult **result)
     int		sts = 0;
     int		found;
     double	tdiff;
-#if 0		// TODO when *result => __pmResult
     __pmResult	*newres;
-#else
-    pmResult	*newres;
-    __pmResult	*__newres;
-#endif
     pmDesc	desc;
     int		kval;
     __pmHashNode	*hp;
@@ -1722,18 +1844,17 @@ more:
 	    nskip = 0;
 	    while (__pmLogRead_ctx(ctxp, ctxp_mode == PM_MODE_FORW ? PM_MODE_BACK : PM_MODE_FORW, NULL, result, PMLOGREAD_NEXT) >= 0) {
 		nskip++;
-		tmp.sec = (*result)->timestamp.tv_sec;
-		tmp.nsec = (*result)->timestamp.tv_usec * 1000;
+		tmp = (*result)->timestamp;
 		tdiff = __pmTimestampSub(&tmp, &ctxp->c_origin);
 		if (ctxp_mode == PM_MODE_FORW && tdiff < 0) {
 		    /* too far ... next one forward is the one we need */
-		    pmFreeResult(*result);
+		    __pmFreeResult(*result);
 		    *result = NULL;
 		    break;
 		}
 		else if (ctxp_mode == PM_MODE_BACK && tdiff > 0) {
 		    /* too far ... next one back is the one we need */
-		    pmFreeResult(*result);
+		    __pmFreeResult(*result);
 		    *result = NULL;
 		    break;
 		}
@@ -1744,13 +1865,13 @@ more:
 		     * in the right direction to avoid reading it twice
 		     * (once above and once the next time through here)
 		     */
-		    pmFreeResult(*result);
+		    __pmFreeResult(*result);
 		    *result = NULL;
 		    break;
 		}
 		ctxp->c_archctl->ac_offset = __pmFtell(ctxp->c_archctl->ac_mfp);
 		ctxp->c_archctl->ac_vol = ctxp->c_archctl->ac_curvol;
-		pmFreeResult(*result);
+		__pmFreeResult(*result);
 		tmp.sec = -1;
 	    }
 	    ctxp->c_archctl->ac_serial = 1;
@@ -1774,13 +1895,12 @@ more:
 	}
 	if ((sts = __pmLogRead_ctx(ctxp, ctxp->c_mode, NULL, result, PMLOGREAD_NEXT)) < 0)
 	    break;
-	tmp.sec = (*result)->timestamp.tv_sec;
-	tmp.nsec = (*result)->timestamp.tv_usec * 1000;
+	tmp = (*result)->timestamp;
 	tdiff = __pmTimestampSub(&tmp, &ctxp->c_origin);
 	if ((tdiff < 0 && ctxp_mode == PM_MODE_FORW) ||
 	    (tdiff > 0 && ctxp_mode == PM_MODE_BACK)) {
 		nskip++;
-		pmFreeResult(*result);
+		__pmFreeResult(*result);
 		*result = NULL;
 		continue;
 	}
@@ -1789,7 +1909,7 @@ more:
 	    if (nskip) {
 		fprintf(stderr, "__pmLogFetch: ctx=%d skip %d to ",
 		    pmWhichContext(), nskip);
-		    pmPrintStamp(stderr, &(*result)->timestamp);
+		    __pmPrintTimestamp(stderr, &(*result)->timestamp);
 		    fputc('\n', stderr);
 		}
 #ifdef DESPERATE
@@ -1800,8 +1920,7 @@ more:
 	}
     }
     if (found) {
-	ctxp->c_origin.sec = (*result)->timestamp.tv_sec;
-	ctxp->c_origin.nsec = (*result)->timestamp.tv_usec * 1000;
+	ctxp->c_origin = (*result)->timestamp;
     }
 
     if (*result != NULL && (*result)->numpmid == 0) {
@@ -1811,7 +1930,7 @@ more:
 	 * otherwise keep searching
 	 */
 	if (numpmid != 0) {
-	    pmFreeResult(*result);
+	    __pmFreeResult(*result);
 	    goto more;
 	}
     }
@@ -1833,11 +1952,10 @@ more:
 	     *     pmFreeResult
 	     */
 
-	    if ((__newres = __pmAllocResult(numpmid)) == NULL) {
+	    if ((newres = __pmAllocResult(numpmid)) == NULL) {
 		pmNoMem("__pmLogFetch.newres", sizeof(__pmResult) + (numpmid -1)* sizeof(pmValueSet *), PM_FATAL_ERR);
 		/* NOTREACHED */
 	    }
-	    newres = __pmOffsetResult(__newres);
 	    newres->numpmid = numpmid;
 	    newres->timestamp = (*result)->timestamp;
 	    u = 0;
@@ -1883,9 +2001,9 @@ more:
 		 * not one of our pmids was in the log record, try
 		 * another log record ...
 		 */
-		pmFreeResult(*result);
+		__pmFreeResult(*result);
 		newres->numpmid = 0;		/* don't free vset's */
-		pmFreeResult(newres);
+		__pmFreeResult(newres);
 		goto more;
 	    }
 	    /*
@@ -1893,7 +2011,7 @@ more:
 	     * pdubuf or the pmid_ctl struct
 	     */
 	    (*result)->numpmid = 0;		/* don't free vset's */
-	    pmFreeResult(*result);
+	    __pmFreeResult(*result);
 	    *result = newres;
 	}
 	else
@@ -1921,7 +2039,7 @@ more:
 		char	errmsg[PM_MAXERRMSGLEN];
 		pmNotifyErr(LOG_WARNING, "__pmLogFetch: missing pmDesc for pmID %s: %s",
 			    pmIDStr_r(desc.pmid, strbuf, sizeof(strbuf)), pmErrStr_r(sts, errmsg, sizeof(errmsg)));
-		pmFreeResult(newres);
+		__pmFreeResult(newres);
 		break;
 	    }
 	    if (desc.indom == PM_INDOM_NULL)
@@ -2017,12 +2135,13 @@ __pmLogSetTime(__pmContext *ctxp)
     mode = ctxp->c_mode & __PM_MODE_MASK; /* strip XTB data */
 
     if (mode == PM_MODE_INTERP)
-	mode = ctxp->c_delta > 0 ? PM_MODE_FORW : PM_MODE_BACK;
+	mode = ctxp->c_direction > 0 ? PM_MODE_FORW : PM_MODE_BACK;
 
     if (pmDebugOptions.log) {
 	fprintf(stderr, "%s(%d) ", "__pmLogSetTime", pmWhichContext());
 	__pmPrintTimestamp(stderr, &ctxp->c_origin);
-	fprintf(stderr, " delta=%d", ctxp->c_delta);
+	fprintf(stderr, " delta=%s%lld.%09ld", ctxp->c_direction < 0 ? "-" : "",
+		(long long)ctxp->c_delta.sec, (long)ctxp->c_delta.nsec);
     }
 
     /*
@@ -2217,11 +2336,11 @@ __pmLogSetTime(__pmContext *ctxp)
 		 * back up one record ...
 		 * index points to the END of the record!
 		 */
-		pmResult	*result;
+		__pmResult	*result;
 		if (pmDebugOptions.log)
 		    fprintf(stderr, " back up ...\n");
 		if (__pmLogRead_ctx(ctxp, PM_MODE_BACK, NULL, &result, PMLOGREAD_NEXT) >= 0)
-		    pmFreeResult(result);
+		    __pmFreeResult(result);
 		if (pmDebugOptions.log)
 		    fprintf(stderr, "...");
 	    }
@@ -2280,17 +2399,20 @@ __pmLogSetTime(__pmContext *ctxp)
     return 0;
 }
 
-/* Read the label of the current archive. */
-int
-__pmGetArchiveLabel(__pmLogCtl *lcp, pmLogLabel *lp)
+/*
+ * Different routines implement the different API versions.
+ * We also have to copy the structure to hide differences
+ * between the internal and external structure versions.
+ */
+typedef int (*pmgetlabel_t)(__pmLogCtl *, void *);
+
+static int
+__pmGetOriginalArchiveLabel(__pmLogCtl *lcp, void *userdata)
 {
+    pmLogLabel		*lp = (pmLogLabel *)userdata;
     __pmLogLabel	*rlp = &lcp->label;
     size_t		bytes;
 
-    /*
-     * we have to copy the structure to hide the differences
-     * between the internal and external structure versions.
-     */
     lp->ll_magic = rlp->magic;
     lp->ll_pid = (pid_t)rlp->pid;
     lp->ll_start.tv_sec = rlp->start.sec;
@@ -2304,9 +2426,34 @@ __pmGetArchiveLabel(__pmLogCtl *lcp, pmLogLabel *lp)
     return 0;
 }
 
+static int
+__pmGetHighResArchiveLabel(__pmLogCtl *lcp, void *userdata)
+{
+    pmHighResLogLabel	*lp = (pmHighResLogLabel *)userdata;
+    __pmLogLabel	*rlp = &lcp->label;
+    size_t		bytes;
+
+    lp->magic = rlp->magic;
+    lp->pid = (pid_t)rlp->pid;
+    lp->start.tv_sec = rlp->start.sec;
+    lp->start.tv_nsec = rlp->start.nsec;
+    bytes = MINIMUM(strlen(rlp->hostname), PM_MAX_HOSTNAMELEN - 1);
+    memcpy(lp->hostname, rlp->hostname, bytes);
+    lp->hostname[bytes] = '\0';
+    bytes = MINIMUM(strlen(rlp->timezone), PM_MAX_TIMEZONELEN - 1);
+    memcpy(lp->timezone, rlp->timezone, bytes);
+    lp->timezone[bytes] = '\0';
+    bytes = rlp->zoneinfo == NULL ? 0 :
+	    MINIMUM(strlen(rlp->zoneinfo), PM_MAX_ZONEINFOLEN - 1);
+    if (bytes)
+	memcpy(lp->zoneinfo, rlp->zoneinfo, bytes);
+    lp->zoneinfo[bytes] = '\0';
+    return 0;
+}
+
 /* Read the label of the first archive in the context. */
-int
-pmGetArchiveLabel(pmLogLabel *lp)
+static int
+__pmGetArchiveLabel(pmgetlabel_t getlabel, void *userdata)
 {
     int		save_arch = 0;		/* pander to gcc */
     int		save_vol = 0;		/* pander to gcc */
@@ -2343,7 +2490,7 @@ pmGetArchiveLabel(pmLogLabel *lp)
     }
 
     /* Get the label. */
-    if ((sts = __pmGetArchiveLabel(lcp, lp)) < 0) {
+    if ((sts = getlabel(lcp, userdata)) < 0) {
 	PM_UNLOCK(ctxp->c_lock);
 	return sts;
     }
@@ -2365,6 +2512,20 @@ pmGetArchiveLabel(pmLogLabel *lp)
     return 0;
 }
 
+/* Pass back the label in the original label format. */
+int
+pmGetArchiveLabel(pmLogLabel *lp)
+{
+    return __pmGetArchiveLabel(__pmGetOriginalArchiveLabel, (void *)lp);
+}
+
+/* Pass back the label in the high resolution label format. */
+int
+pmGetHighResArchiveLabel(pmHighResLogLabel *lp)
+{
+    return __pmGetArchiveLabel(__pmGetHighResArchiveLabel, (void *)lp);
+}
+
 /*
  * Get the end time of the current archive.
  *
@@ -2380,23 +2541,24 @@ __pmGetArchiveEnd_ctx(__pmContext *ctxp, __pmTimestamp *tsp)
     struct stat	sbuf;
     __pmFILE	*f;
     long	save = 0;
-    pmResult	*rp = NULL;
-    pmResult	*nrp;
+    __pmResult	*rp = NULL;
+    __pmResult	*nrp;
     int		i;
     int		sts;
     int		found;
     int		head;
     long	offset;
     int		vol;
-    __pmoff32_t	logend;
-    __pmoff32_t	physend = 0;
+    int		vers = __pmLogVersion(lcp);
+    size_t	logend;
+    size_t	physend = 0;
 
     PM_ASSERT_IS_LOCKED(ctxp->c_lock);
 
     /*
      * default, when all else fails ...
      */
-    tsp->sec = INT_MAX;
+    tsp->sec = PM_MAX_TIME_T;
     tsp->nsec = 0;
 
     /*
@@ -2439,9 +2601,9 @@ __pmGetArchiveEnd_ctx(__pmContext *ctxp, __pmTimestamp *tsp)
 	if (sbuf.st_size <= __pmLogLabelSize(lcp))
 	    goto prior_vol;
 
-	physend = (__pmoff32_t)sbuf.st_size;
-	if (sizeof(off_t) > sizeof(__pmoff32_t)) {
-	    /* 64-bit off_t */
+	physend = sbuf.st_size;
+	if (sizeof(off_t) > sizeof(__pmoff32_t) && vers == PM_LOG_VERS02) {
+	    /* 64-bit off_t, version 2 archives */
 	    if (physend != sbuf.st_size) {
 		/* oops, 32-bit offset not the same */
 		pmNotifyErr(LOG_ERR, "pmGetArchiveEnd: PCP archive file"
@@ -2494,7 +2656,7 @@ __pmGetArchiveEnd_ctx(__pmContext *ctxp, __pmTimestamp *tsp)
 		/* this is badly damaged! */
 		if (pmDebugOptions.log) {
 		    fprintf(stderr, "pmGetArchiveEnd: "
-                            "Error reading record ending at posn=%d ti[%d]@",
+                            "Error reading record ending at posn=%zu ti[%d]@",
 			    logend, i);
 		    __pmPrintTimestamp(stderr, &lcp->ti[i].stamp);
 		    fputc('\n', stderr);
@@ -2521,7 +2683,7 @@ __pmGetArchiveEnd_ctx(__pmContext *ctxp, __pmTimestamp *tsp)
 	    /* this one is ok, remember it as it may be the last one */
 	    found = 1;
 	    if (rp != NULL)
-		pmFreeResult(rp);
+		__pmFreeResult(rp);
 	    rp = nrp;
 	}
 	if (found)
@@ -2547,11 +2709,9 @@ prior_vol:
 	__pmFclose(f);
 
     if (found) {
-	tsp->sec = rp->timestamp.tv_sec;
-	tsp->nsec = rp->timestamp.tv_usec * 1000;
+	*tsp = rp->timestamp;
 	if (vol == lcp->maxvol) {
-	    lcp->endtime.sec = rp->timestamp.tv_sec;
-	    lcp->endtime.nsec = rp->timestamp.tv_usec * 1000;
+	    lcp->endtime = rp->timestamp;
 	    lcp->physend = physend;
 	}
 	sts = 0;
@@ -2561,7 +2721,7 @@ prior_vol:
 	 * rp is not NULL from found==1 path _or_ from error break
 	 * after an initial paranoidLogRead() success
 	 */
-	pmFreeResult(rp);
+	__pmFreeResult(rp);
     }
 
     return sts;
@@ -2591,13 +2751,25 @@ __pmGetArchiveEnd(__pmArchCtl *acp, __pmTimestamp *tsp)
 int
 pmGetArchiveEnd(struct timeval *tp)
 {
-    int			sts;
     __pmTimestamp	stamp;
+    int			sts;
 
     sts = pmGetArchiveEnd_ctx(NULL, &stamp);
     tp->tv_sec = stamp.sec;
     tp->tv_usec = stamp.nsec / 1000;
+    return sts;
+}
 
+/* Get the high resolution end time of the final archive in the context. */
+int
+pmGetHighResArchiveEnd(struct timespec *ts)
+{
+    __pmTimestamp	stamp;
+    int			sts;
+
+    sts = pmGetArchiveEnd_ctx(NULL, &stamp);
+    ts->tv_sec = stamp.sec;
+    ts->tv_nsec = stamp.nsec;
     return sts;
 }
 
@@ -2715,7 +2887,7 @@ __pmLogChangeArchive(__pmContext *ctxp, int arch)
  * so that the current context can be carried down the call stack.
  */
 static int
-LogCheckForNextArchive(__pmContext *ctxp, int mode, pmResult **result)
+LogCheckForNextArchive(__pmContext *ctxp, int mode, __pmResult **result)
 {
     __pmArchCtl	*acp;
     int		sts = 0;
@@ -2744,7 +2916,7 @@ LogCheckForNextArchive(__pmContext *ctxp, int mode, pmResult **result)
 }
 
 int
-__pmLogCheckForNextArchive(__pmLogCtl *lcp, int mode, pmResult **result)
+__pmLogCheckForNextArchive(__pmLogCtl *lcp, int mode, __pmResult **result)
 {
     int		sts;
     __pmContext	*ctxp;
