@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2014,2021 Red Hat.
+ * Copyright (c) 2012-2014,2021-2022 Red Hat.
  * Copyright (c) 1995-2000 Silicon Graphics, Inc.  All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or modify it
@@ -62,8 +62,31 @@ typedef struct {
     __pmPDU		data[2];	/* zero or more (2 for alignment) */
 } highres_result_t;
 
-static void
-__pmGetResultSize(int pdutype, int numpmid, pmValueSet * const *vset,
+/*
+ * __pmResult record, Version 3
+ * The real on-disk format starts at "from" (which is really "len")
+ * but the libpcp routines fake this out to be a PDU format, which
+ * adds 2 __int32_t fields at the start
+ */
+typedef struct {
+    __int32_t	len;		/* PDU header -- repacked */
+    __int32_t	type;		/* ditto */
+    __int32_t	from;		/* ditto -- becomes len */
+    __int32_t	sec[2];		/* __pmTimestamp */
+    __int32_t	nsec;
+    __int32_t	numpmid;
+    __int32_t	data[1];	/* zero or more */
+} log_result_v3_t;
+
+/*
+ * This routine gets used for both PDUs and external log records, so
+ * "type" is one of
+ * - PDU_HIGHRES_RESULT - PDU
+ * - PDU_RESULT - PDU or V2 archive
+ * - PM_LOG_VERS03 - V3 archive
+ */
+void
+__pmGetResultSize(int type, int numpmid, pmValueSet * const *vset,
 		size_t *needp, size_t *vneedp)
 {
     size_t	need;	/* bytes for the PDU */
@@ -71,10 +94,16 @@ __pmGetResultSize(int pdutype, int numpmid, pmValueSet * const *vset,
     int		i, j;
    
     vneed = 0;
-    if (pdutype == PDU_HIGHRES_RESULT)
+    if (type == PDU_HIGHRES_RESULT)
 	need = sizeof(highres_result_t) - (sizeof(__pmPDU) * 2);
-    else
+    else if (type == PDU_RESULT)
 	need = sizeof(result_t) - sizeof(__pmPDU);
+    else if (type == PM_LOG_VERS03)
+	need = sizeof(log_result_v3_t) - sizeof(__int32_t);
+    else {
+	fprintf(stderr, "__pmGetResultSize: Botch: type=%d not allowed\n", type);
+	exit(1);
+    }
 
     /* now add space for each vlist_t (data field in result structures) */
     for (i = 0; i < numpmid; i++) {
@@ -153,15 +182,16 @@ __pmEncodeValueSet(__pmPDU *pdubuf, int numpmid, pmValueSet * const *vset,
 }
 
 int
-__pmEncodeResult(int targetfd, const pmResult *result, __pmPDU **pdu)
+__pmEncodeResult(const __pmLogCtl *lcp, const __pmResult *result, __pmPDU **pdu)
 {
     size_t	need, vneed;
     __pmPDU	*pdubuf;
-    result_t	*pp;
     int		type = PDU_RESULT;
 
-    (void)targetfd;
-    __pmGetResultSize(type, result->numpmid, result->vset, &need, &vneed);
+    if (lcp != NULL && __pmLogVersion(lcp) == PM_LOG_VERS03)
+	__pmGetResultSize(PM_LOG_VERS03, result->numpmid, result->vset, &need, &vneed);
+    else
+	__pmGetResultSize(type, result->numpmid, result->vset, &need, &vneed);
 
     /*
      * Need to reserve additional space for trailer (an int) in case the
@@ -169,15 +199,28 @@ __pmEncodeResult(int targetfd, const pmResult *result, __pmPDU **pdu)
      */
     if ((pdubuf = __pmFindPDUBuf((int)(need + vneed + sizeof(int)))) == NULL)
 	return -oserror();
-    pp = (result_t *)pdubuf;
-    pp->hdr.len = (int)(need + vneed);
-    pp->hdr.type = type;
-    pp->timestamp.tv_sec = htonl((__int32_t)(result->timestamp.tv_sec));
-    pp->timestamp.tv_usec = htonl((__int32_t)(result->timestamp.tv_usec));
-    pp->numpmid = htonl(result->numpmid);
+    if (lcp != NULL && __pmLogVersion(lcp) == PM_LOG_VERS03) {
+	log_result_v3_t	*lrp;
+	lrp = (log_result_v3_t *)pdubuf;
+	lrp->len = (int)(need + vneed);
+	lrp->from = 0;			/* not used for log records */
+	lrp->type = PDU_RESULT;		/* not used for log records */
+	__pmPutTimestamp(&result->timestamp, &lrp->sec[0]);
+	lrp->numpmid = htonl(result->numpmid);
+	__pmEncodeValueSet(pdubuf, result->numpmid, result->vset,
+			(vlist_t *)lrp->data, pdubuf + need/sizeof(__pmPDU));
+    }
+    else {
+	result_t	*pp;
+	pp = (result_t *)pdubuf;
+	pp->hdr.len = (int)(need + vneed);
+	pp->hdr.type = type;
+	__pmPutTimeval(&result->timestamp, (__int32_t *)&pp->timestamp);
+	pp->numpmid = htonl(result->numpmid);
 
-    __pmEncodeValueSet(pdubuf, result->numpmid, result->vset,
+	__pmEncodeValueSet(pdubuf, result->numpmid, result->vset,
 			(vlist_t *)pp->data, pdubuf + need/sizeof(__pmPDU));
+    }
 
     /* Note PDU remains pinned ... see thread-safe comments above */
     *pdu = pdubuf;
@@ -185,7 +228,7 @@ __pmEncodeResult(int targetfd, const pmResult *result, __pmPDU **pdu)
 }
 
 int
-__pmEncodeHighResResult(const pmHighResResult *result, __pmPDU **pdu)
+__pmEncodeHighResResult(const __pmResult *result, __pmPDU **pdu)
 {
     size_t		need, vneed;
     __pmPDU		*pdubuf;
@@ -204,8 +247,8 @@ __pmEncodeHighResResult(const pmHighResResult *result, __pmPDU **pdu)
     pp->hdr.len = (int)(need + vneed);
     pp->hdr.type = type;
     pp->numpmid = htonl(result->numpmid);
-    pp->timestamp.tv_sec = (__int64_t)(result->timestamp.tv_sec);
-    pp->timestamp.tv_nsec = (__int64_t)(result->timestamp.tv_nsec);
+    pp->timestamp.tv_sec = result->timestamp.sec;
+    pp->timestamp.tv_nsec = result->timestamp.nsec;
     __htonll((char *)&pp->timestamp.tv_sec);
     __htonll((char *)&pp->timestamp.tv_nsec);
 
@@ -221,18 +264,21 @@ __pmEncodeHighResResult(const pmHighResResult *result, __pmPDU **pdu)
  * Internal variant of __pmSendResult() with current context.
  */
 int
-__pmSendResult_ctx(__pmContext *ctxp, int fd, int from, const pmResult *result)
+__pmSendResult_ctx(__pmContext *ctxp, int fd, int from, const __pmResult *result)
 {
     int		sts;
     __pmPDU	*pdubuf = NULL;
     result_t	*pp;
+    __pmLogCtl	*lcp = NULL;
 
     if (ctxp != NULL)
 	PM_ASSERT_IS_LOCKED(ctxp->c_lock);
 
     if (pmDebugOptions.pdu)
-	__pmDumpResult_ctx(ctxp, stderr, result);
-    if ((sts = __pmEncodeResult(fd, result, &pdubuf)) < 0)
+	__pmPrintResult_ctx(ctxp, stderr, result);
+    if (ctxp != NULL && ctxp->c_type == PM_CONTEXT_ARCHIVE)
+	lcp = ctxp->c_archctl->ac_log;
+    if ((sts = __pmEncodeResult(lcp, result, &pdubuf)) < 0)
 	return sts;
     pp = (result_t *)pdubuf;
     pp->hdr.from = from;
@@ -242,7 +288,7 @@ __pmSendResult_ctx(__pmContext *ctxp, int fd, int from, const pmResult *result)
 }
 
 int
-__pmSendResult(int fd, int from, const pmResult *result)
+__pmSendResult(int fd, int from, const __pmResult *result)
 {
     return __pmSendResult_ctx(NULL, fd, from, result);
 }
@@ -251,7 +297,7 @@ __pmSendResult(int fd, int from, const pmResult *result)
  * Internal variant of __pmSendHighResResult() with current context.
  */
 int
-__pmSendHighResResult_ctx(__pmContext *ctxp, int fd, int from, const pmHighResResult *result)
+__pmSendHighResResult_ctx(__pmContext *ctxp, int fd, int from, const __pmResult *result)
 {
     __pmPDU		*pdubuf = NULL;
     highres_result_t	*pp;
@@ -261,7 +307,7 @@ __pmSendHighResResult_ctx(__pmContext *ctxp, int fd, int from, const pmHighResRe
 	PM_ASSERT_IS_LOCKED(ctxp->c_lock);
 
     if (pmDebugOptions.pdu)
-	__pmDumpHighResResult_ctx(ctxp, stderr, result);
+	__pmPrintResult_ctx(ctxp, stderr, result);
     if ((sts = __pmEncodeHighResResult(result, &pdubuf)) < 0)
 	return sts;
     pp = (highres_result_t *)pdubuf;
@@ -272,15 +318,15 @@ __pmSendHighResResult_ctx(__pmContext *ctxp, int fd, int from, const pmHighResRe
 }
 
 int
-__pmSendHighResResult(int fd, int from, const pmHighResResult *result)
+__pmSendHighResResult(int fd, int from, const __pmResult *result)
 {
     return __pmSendHighResResult_ctx(NULL, fd, from, result);
 }
 
 #if defined(HAVE_64BIT_PTR)
-static int
+int
 __pmDecodeValueSet(__pmPDU *pdubuf, int pdulen, __pmPDU *data, char *pduend,
-		int numpmid, int preamble, int unaligned, pmValueSet *vset[])
+		int numpmid, int preamble, int unaligned, pmValueSet **vset)
 {
     char	*newbuf;
     int		valfmt;
@@ -387,6 +433,7 @@ __pmDecodeValueSet(__pmPDU *pdubuf, int pdulen, __pmPDU *data, char *pduend,
 					    "__pmDecodeValueSet", i, j);
 			return PM_ERR_IPC;
 		    }
+
 		    __ntohpmValueBlock(pduvbp);
 		    if (pduvbp->vlen < PM_VAL_HDR_SIZE ||
 			pduvbp->vlen > pdulen) {
@@ -561,7 +608,7 @@ __pmDecodeValueSet(__pmPDU *pdubuf, int pdulen, __pmPDU *data, char *pduend,
 
 #elif defined(HAVE_32BIT_PTR)
 
-static int
+int
 __pmDecodeValueSet(__pmPDU *pdubuf, int pdulen, __pmPDU *data, char *pduend,
 		int numpmid, int preamble, int unaligned, pmValueSet *vset[])
 {
@@ -705,75 +752,89 @@ __pmDecodeValueSet(__pmPDU *pdubuf, int pdulen, __pmPDU *data, char *pduend,
 #endif
 
 /*
- * Internal variant of __pmDecodeResult() with current context.
+ * Internal variant of __pmDecodeResult() with current context and
+ * internal result structure format.
  *
  * Enter here with pdubuf already pinned ... result may point into
  * _another_ pdu buffer that is pinned on exit
  */
 int
-__pmDecodeResult_ctx(__pmContext *ctxp, __pmPDU *pdubuf, pmResult **result)
+__pmDecodeResult_ctx(__pmContext *ctxp, __pmPDU *pdubuf, __pmResult **result)
 {
-    int		sts;
-    int		numpmid;	/* number of metrics */
-    char	*pduend;	/* end pointer for incoming buffer */
-    size_t	bytes, nopad;
-    result_t	*pp;
-#if 0	// TODO when pr,*result -> __pmResult
-    __pmResult	*pr;
-#else
-    __pmResult	*__pr;
-    pmResult	*pr;
-#endif
+    int			sts;
+    int			numpmid;	/* number of metrics */
+    int			len = pdubuf[0];
+    __pmPDU		*vset;
+    char		*pduend;	/* end pointer for incoming buffer */
+    size_t		bytes, nopad;
+    int			maxnumpmid;
+    __pmResult		*pr;
+    __pmTimestamp	stamp;
 
     if (ctxp != NULL)
 	PM_ASSERT_IS_LOCKED(ctxp->c_lock);
 
-    pp = (result_t *)pdubuf;
-    pduend = (char *)pdubuf + pp->hdr.len;
-    bytes = sizeof(result_t) - sizeof(__pmPDU);
+    if (ctxp != NULL && ctxp->c_type == PM_CONTEXT_ARCHIVE && __pmLogVersion(ctxp->c_archctl->ac_log) == PM_LOG_VERS03) {
+	/*
+	 * V3 archive
+	 */
+	log_result_v3_t	*lrp = (log_result_v3_t *)pdubuf;
+	pduend = (char *)pdubuf + len;
+	bytes = sizeof(log_result_v3_t) - sizeof(__int32_t);
+	nopad = bytes;
+	numpmid = ntohl(lrp->numpmid);
+	__pmLoadTimestamp((__int32_t *)&lrp->sec[0], &stamp);
+	vset = (__pmPDU *)lrp->data;
+    }
+    else {
+	/*
+	 * over the wire PDU or V2 archive
+	 */
+	result_t	*pp = (result_t *)pdubuf;
+	pduend = (char *)pdubuf + len;
+	bytes = sizeof(result_t) - sizeof(__pmPDU);
+	nopad = sizeof(pp->hdr) + sizeof(pp->timestamp) + sizeof(pp->numpmid);
+	numpmid = ntohl(pp->numpmid);
+	__pmLoadTimeval((__int32_t *)&pp->timestamp, &stamp);
+	vset = pp->data;
+    }
+
     if (pduend - (char *)pdubuf < bytes) {
 	if (pmDebugOptions.pdu && pmDebugOptions.desperate)
 	    fprintf(stderr, "%s: Bad: len=%d smaller than min %d\n",
-			    "__pmDecodeResult", pp->hdr.len, (int)bytes);
+			    "__pmDecodeResult", len, (int)bytes);
 	return PM_ERR_IPC;
     }
 
-    numpmid = ntohl(pp->numpmid);
-    if (numpmid < 0 || numpmid > pp->hdr.len) {
+    if (numpmid < 0 || numpmid > len) {
 	if (pmDebugOptions.pdu && pmDebugOptions.desperate)
 	    fprintf(stderr, "%s: Bad: numpmid=%d negative or not smaller "
 			    "than PDU len %d\n", "__pmDecodeResult",
-			    numpmid, pp->hdr.len);
+			    numpmid, len);
 	return PM_ERR_IPC;
     }
-    bytes = (INT_MAX - sizeof(pmResult)) / sizeof(pmValueSet *);
-    if (numpmid >= bytes) {
+    maxnumpmid = (INT_MAX - sizeof(pmResult)) / sizeof(pmValueSet *);
+    if (numpmid >= maxnumpmid) {
 	if (pmDebugOptions.pdu && pmDebugOptions.desperate)
 	    fprintf(stderr, "%s: Bad: numpmid=%d larger than max %ld\n",
-			    "__pmDecodeResult", numpmid, (long)bytes);
+			    "__pmDecodeResult", numpmid, (long)maxnumpmid);
 	return PM_ERR_IPC;
     }
 
-    if ((__pr = (__pmResult *)__pmAllocResult(numpmid)) == NULL)
+    if ((pr = __pmAllocResult(numpmid)) == NULL)
 	return -oserror();
-
-    pr = __pmOffsetResult(__pr);
     pr->numpmid = numpmid;
-    pr->timestamp.tv_sec = ntohl(pp->timestamp.tv_sec);
-    pr->timestamp.tv_usec = ntohl(pp->timestamp.tv_usec);
+    pr->timestamp = stamp;		/* struct asssignment */
 
-    bytes = sizeof(result_t) - sizeof(__pmPDU);
-    nopad = sizeof(pp->hdr) + sizeof(pp->timestamp) + sizeof(pp->numpmid);
-
-    if ((sts = __pmDecodeValueSet(pdubuf, pp->hdr.len, pp->data, pduend,
+    if ((sts = __pmDecodeValueSet(pdubuf, len, vset, pduend,
 				  numpmid, bytes, nopad, pr->vset)) < 0) {
 	pr->numpmid = 0;	/* force no pmValueSet's to free */
-	pmFreeResult(pr);
+	__pmFreeResult(pr);
 	return sts;
     }
 
     if (pmDebugOptions.pdu)
-	__pmDumpResult_ctx(ctxp, stderr, pr);
+	__pmPrintResult_ctx(ctxp, stderr, pr);
 
     /*
      * Note we return with the input buffer (pdubuf) still pinned and
@@ -785,7 +846,7 @@ __pmDecodeResult_ctx(__pmContext *ctxp, __pmPDU *pdubuf, pmResult **result)
 }
 
 int
-__pmDecodeResult(__pmPDU *pdubuf, pmResult **result)
+__pmDecodeResult(__pmPDU *pdubuf, __pmResult **result)
 {
     return __pmDecodeResult_ctx(NULL, pdubuf, result);
 }
@@ -797,13 +858,13 @@ __pmDecodeResult(__pmPDU *pdubuf, pmResult **result)
  * _another_ pdu buffer that is pinned on exit
  */
 int
-__pmDecodeHighResResult_ctx(__pmContext *ctxp, __pmPDU *pdubuf, pmHighResResult **result)
+__pmDecodeHighResResult_ctx(__pmContext *ctxp, __pmPDU *pdubuf, __pmResult **result)
 {
     int			sts;
     int			numpmid;	/* number of metrics */
     char		*pduend;	/* end pointer for incoming buffer */
     size_t		bytes, nopad;
-    pmHighResResult	*pr;
+    __pmResult		*pr;
     highres_result_t	*pp;
 
     if (ctxp != NULL)
@@ -835,27 +896,27 @@ __pmDecodeHighResResult_ctx(__pmContext *ctxp, __pmPDU *pdubuf, pmHighResResult
 	return PM_ERR_IPC;
     }
 
-    if ((pr = (pmHighResResult *)malloc(sizeof(pmHighResResult) +
-			 	(numpmid - 1) * sizeof(pmValueSet *))) == NULL)
+    if ((pr = __pmAllocResult(numpmid)) == NULL)
 	return -oserror();
 
     pr->numpmid = numpmid;
     __ntohll((char *)&pp->timestamp.tv_sec);
-    pr->timestamp.tv_sec = pp->timestamp.tv_sec;
+    pr->timestamp.sec = pp->timestamp.tv_sec;
     __ntohll((char *)&pp->timestamp.tv_nsec);
-    pr->timestamp.tv_nsec = pp->timestamp.tv_nsec;
+    pr->timestamp.nsec = pp->timestamp.tv_nsec;
 
     bytes = sizeof(highres_result_t) - (sizeof(__pmPDU) * 2);
     nopad = sizeof(pp->hdr) + sizeof(pp->numpmid) + sizeof(pp->timestamp);
 
     if ((sts = __pmDecodeValueSet(pdubuf, pp->hdr.len, pp->data, pduend,
 				  numpmid, bytes, nopad, pr->vset)) < 0) {
-	free(pr);
+	pr->numpmid = 0;	/* force no pmValueSet's to free */
+	__pmFreeResult(pr);
 	return sts;
     }
 
     if (pmDebugOptions.pdu)
-	__pmDumpHighResResult_ctx(ctxp, stderr, pr);
+	__pmPrintResult_ctx(ctxp, stderr, pr);
 
     /*
      * Note we return with the input buffer (pdubuf) still pinned and
@@ -867,7 +928,7 @@ __pmDecodeHighResResult_ctx(__pmContext *ctxp, __pmPDU *pdubuf, pmHighResResult
 }
 
 int
-__pmDecodeHighResResult(__pmPDU *pdubuf, pmHighResResult **result)
+__pmDecodeHighResResult(__pmPDU *pdubuf, __pmResult **result)
 {
     return __pmDecodeHighResResult_ctx(NULL, pdubuf, result);
 }
