@@ -1,11 +1,12 @@
 /*
+ * Copyright (c) 2021 Red Hat.
  * Copyright (c) 1995-2003,2004 Silicon Graphics, Inc.  All Rights Reserved.
- * 
+ *
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
  * by the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
@@ -26,6 +27,7 @@
 
 #include "pmapi.h"
 #include "libpcp.h"
+#include "sha256.h"
 #include "internal.h"
 
 static char	*envtz;			/* buffer in env */
@@ -276,6 +278,194 @@ __pmSquashTZ(char *tzbuffer)
 #endif
 }
 
+#define ZONEINFO "/zoneinfo/"
+#define LOCALTIME "/etc/localtime"
+#define ZONESHARED "/usr/share/zoneinfo"
+
+static int
+tzfilehash(int fd, unsigned char *hash)
+{
+    SHA256_CTX		ctx;
+    ssize_t		bytes;
+    char		buf[BUFSIZ];
+
+    sha256_init(&ctx);
+    while ((bytes = read(fd, buf, sizeof(buf))) > 0)
+	sha256_update(&ctx, (unsigned char *)buf, bytes);
+    if (bytes < 0)
+	return -1;
+    sha256_final(&ctx, hash);
+    return 0;
+}
+
+static off_t
+tzfilesize(int fd)
+{
+    struct stat		sbuf;
+
+    if (fstat(fd, &sbuf) == 0 && S_ISREG(sbuf.st_mode))
+	return sbuf.st_size;
+    if (S_ISDIR(sbuf.st_mode))
+	return -1;
+    return 0;	/* ignored */
+}
+
+static void
+zoneinfo_scan(char *base, char *tzpath, size_t tzpathlen, off_t tzfilelen,
+			unsigned char *tzhash, int *found)
+{
+    DIR			*dir;
+    struct dirent	*dp;
+    size_t		bytes;
+    off_t		length;
+    unsigned char	hash[SHA256_BLOCK_SIZE];
+    char		path[MAXPATHLEN];
+    char		*endp;
+    int			fd, endi;
+
+    if ((dir = opendir(base)) == NULL)
+        return;
+    endi = pmsprintf(path, sizeof(path) - 1, "%s", base);
+    endp = &path[endi];
+    *endp++ = pmPathSeparator();
+
+    while ((dp = readdir(dir)) != NULL) {
+	if (dp->d_name[0] == '.')
+	    continue;
+	bytes = (endp - path) + strlen(dp->d_name);
+	if (bytes >= sizeof(path)-1)
+	    continue;
+	strcpy(endp, dp->d_name);
+	if ((fd = open(path, O_RDONLY)) < 0)
+	    continue;
+	if ((length = tzfilesize(fd)) < 0) {	/* S_ISDIR */
+	    zoneinfo_scan(path, tzpath, tzpathlen, tzfilelen, tzhash, found);
+	    goto next;
+	}
+	if (length == 0)	/* ignore stat errors or an empty file */
+	    goto next;
+	if (length != tzfilelen)	/* match LOCALTIME */
+	    goto next;
+
+	/* At this point we have a candidate regular, right-sized file */
+	if (tzfilehash(fd, hash) < 0)
+	    goto next;
+	if (memcmp(tzhash, hash, SHA256_BLOCK_SIZE) != 0)
+	    goto next;
+
+	/*
+	 * Found a match - keep it if it is the shortest yet as this will
+	 * favour Australia/Melbourne over posix/Australia/Melbourne, for
+	 * example.
+	 */
+	if (*found == 0 || strlen(tzpath) > bytes)
+	    pmsprintf(tzpath, tzpathlen, "%s", path);
+	*found = 1;
+next:
+	close(fd);
+    }
+    closedir(dir);
+}
+
+/*
+ * Descend /usr/share/zoneinfo looking for a file with the same size
+ * and then compare file contents ... if SHA256 signature is equal,
+ * consider it a match.
+ *
+ * If more than one file matches, pick the first but emit a warning.
+ */
+static char *
+zoneinfo_fallback(char *path, size_t pathlen)
+{
+    unsigned char	hash[SHA256_BLOCK_SIZE];
+    off_t		len;
+    int			fd, found = 0;
+
+    if ((fd = open(LOCALTIME, O_RDONLY)) < 0) {
+	if (pmDebugOptions.timecontrol)
+	    fprintf(stderr, "%s: cannot open %s: %s\n",
+			    "__pmZoneinfo", LOCALTIME, pmErrStr(-oserror()));
+	return NULL;
+    }
+    if ((len = tzfilesize(fd)) <= 0) {
+	close(fd);
+	return NULL;
+    }
+    if (tzfilehash(fd, hash) < 0) {
+	close(fd);
+	return NULL;
+    }
+    close(fd);
+
+    /* recurse system path, looking for matches on tzlen and tzhash */
+    zoneinfo_scan(ZONESHARED, path+1, pathlen-1, len, hash, &found);
+    if (!found)
+	return NULL;
+    path[0] = ':';
+    return path;
+}
+
+/*
+ * Get the local timezone tzfile identification
+ *
+ * We'd like this to return something like ":Australia/Melbourne"
+ * that can be used as a $TZ setting.
+ *
+ * If /etc/localtime is a symbolic link, get the pathname it points
+ * to and strip anything up to (and including) the string "/zoneinfo/".
+ * Else, fallback to attempting to match the file contents for all TZ
+ * files below the /usr/share/zoneinfo directory recursively.
+ *
+ * Return NULL on failure.
+ */
+char *
+__pmZoneinfo(void)
+{
+    ssize_t	sts;
+    char	*buf;
+    char	*tmp_buf;
+    char	*p;
+    char	*q;
+
+    /* +1 for : +1 for NULL */
+    buf = (char *)calloc(1, MAXPATHLEN+2);
+    if (buf == NULL)
+	return NULL;
+
+    sts = readlink(LOCALTIME, &buf[1], MAXPATHLEN);
+    if (sts < 0) {
+	/* Exists but not a symlink - try recursive file content matching. */
+	if (oserror() == ENOENT ||
+	    zoneinfo_fallback(buf+1, MAXPATHLEN+1) == NULL) {
+	    free(buf);
+	    return NULL;
+	}
+    } else {
+	buf[sts+1] = '\0';
+    }
+
+    /* try to find prefix .../zoneinfo/... */
+    p = strstr(buf+1, ZONEINFO);
+    if (p != NULL) {
+	/* found it! */
+	q = &p[strlen(ZONEINFO)-1];
+	tmp_buf = strdup(q);
+	if (tmp_buf != NULL) {
+	    free(buf);
+	    buf = tmp_buf;
+	}
+    }
+    else {
+	/* no prefix, truncate ... nice to have, not necessary */
+	tmp_buf = realloc(buf, sts+2);
+	if (tmp_buf != NULL)
+	    buf = tmp_buf;
+    }
+    buf[0] = ':';
+
+    return buf;
+}
+
 /*
  * __pmTimezone: work out local timezone
  */
@@ -445,29 +635,43 @@ pmNewContextZone(void)
 	return PM_ERR_NOCONTEXT;
 
     if (ctxp->c_type == PM_CONTEXT_ARCHIVE) {
-	sts = pmNewZone(ctxp->c_archctl->ac_log->l_label.ill_tz);
+	sts = pmNewZone(ctxp->c_archctl->ac_log->label.timezone);
 	PM_UNLOCK(ctxp->c_lock);
     }
     else if (ctxp->c_type == PM_CONTEXT_LOCAL) {
 	char	tzbuf[PM_TZ_MAXLEN];
+	char	*tz, *info;
+
 	/* from env, not PMCD */
 	PM_UNLOCK(ctxp->c_lock);
-	__pmTimezone_r(tzbuf, sizeof(tzbuf));
-	sts = pmNewZone(tzbuf);
+	if ((info = tz = __pmZoneinfo()) == NULL)
+	    tz = __pmTimezone_r(tzbuf, sizeof(tzbuf));
+	sts = pmNewZone(tz);
+	if (info != NULL)
+	    free(info);
     }
     else {
 	/* assume PM_CONTEXT_HOST */
-	const char	*name = "pmcd.timezone";
-	pmID		pmid;
+	const char	*names[] = { "pmcd.timezone", "pmcd.zoneinfo" };
+	pmID		pmids[2];
 	pmResult	*rp;
+	pmValueSet	*tzp, *zip;
 
 	PM_UNLOCK(ctxp->c_lock);
-	if ((sts = pmLookupName(1, &name, &pmid)) < 0)
-	    return sts;
-	if ((sts = pmFetch(1, &pmid, &rp)) >= 0) {
-	    if (rp->vset[0]->numval == 1 && 
-		(rp->vset[0]->valfmt == PM_VAL_DPTR || rp->vset[0]->valfmt == PM_VAL_SPTR))
-		sts = pmNewZone((char *)rp->vset[0]->vlist[0].value.pval->vbuf);
+	if ((sts = pmLookupName(2, names, pmids)) < 0) {
+	    /* as long as one of the metrics is in the PMNS we're ok */
+	    if (pmids[0] == PM_ID_NULL && pmids[1] == PM_ID_NULL)
+		return sts;
+	}
+	if ((sts = pmFetch(2, pmids, &rp)) >= 0) {
+	    tzp = rp->vset[0];
+	    zip = rp->vset[1];
+	    if (zip->numval == 1 &&
+		(zip->valfmt == PM_VAL_DPTR || zip->valfmt == PM_VAL_SPTR))
+		sts = pmNewZone((char *)zip->vlist[0].value.pval->vbuf);
+	    else if (tzp->numval == 1 &&
+		(tzp->valfmt == PM_VAL_DPTR || tzp->valfmt == PM_VAL_SPTR))
+		sts = pmNewZone((char *)tzp->vlist[0].value.pval->vbuf);
 	    else
 		sts = PM_ERR_VALUE;
 	    pmFreeResult(rp);
