@@ -1,7 +1,7 @@
 /*
  * General Utility Routines
  *
- * Copyright (c) 2012-2018,2021 Red Hat.
+ * Copyright (c) 2012-2018,2021-2022 Red Hat.
  * Copyright (c) 2009 Aconex.  All Rights Reserved.
  * Copyright (c) 1995-2002,2004 Silicon Graphics, Inc.  All Rights Reserved.
  *
@@ -833,17 +833,14 @@ void
 __pmPrintResult_ctx(__pmContext *ctxp, FILE *f, const __pmResult *resp)
 {
     int		i;
-    struct timespec ts;
 
     if (ctxp != NULL)
 	PM_ASSERT_IS_LOCKED(ctxp->c_lock);
 
     save_debug();
-    fprintf(f, "__pmResult dump from " PRINTF_P_PFX "%p timestamp: %" FMT_INT64 ".%06d ",
+    fprintf(f, "__pmResult dump from " PRINTF_P_PFX "%p timestamp: %" FMT_INT64 ".%09d ",
 	resp, resp->timestamp.sec, resp->timestamp.nsec);
-    ts.tv_sec = resp->timestamp.sec;
-    ts.tv_nsec = resp->timestamp.nsec;
-    pmPrintHighResStamp(f, &ts);
+    __pmPrintTimestamp(f, &resp->timestamp);
     fprintf(f, " numpmid: %d\n", resp->numpmid);
     for (i = 0; i < resp->numpmid; i++)
 	dump_valueset(ctxp, f, resp->vset[i]);
@@ -866,8 +863,10 @@ __pmDumpHighResResult_ctx(__pmContext *ctxp, FILE *f, const pmHighResResult *hre
 	PM_ASSERT_IS_LOCKED(ctxp->c_lock);
 
     save_debug();
-    fprintf(f, "pmHighResResult dump from " PRINTF_P_PFX "%p timestamp: %d.%09d ",
-	hresp, (int)hresp->timestamp.tv_sec, (int)hresp->timestamp.tv_nsec);
+    fprintf(f, "%s dump from " PRINTF_P_PFX "%p timestamp: %lld.%09lld ",
+	    "pmHighResResult", hresp,
+	    (long long)hresp->timestamp.tv_sec,
+	    (long long)hresp->timestamp.tv_nsec);
     pmPrintHighResStamp(f, &hresp->timestamp);
     fprintf(f, " numpmid: %d\n", hresp->numpmid);
     for (i = 0; i < hresp->numpmid; i++)
@@ -879,7 +878,6 @@ void
 __pmDumpHighResResult(FILE *f, const pmHighResResult *hresp)
 {
     __pmDumpHighResResult_ctx(NULL, f, hresp);
-
 }
 
 static void
@@ -1233,6 +1231,19 @@ pmtimespecNow(struct timespec *ts)
     return __pmGetTimespec(ts);
 }
 
+int
+__pmGetTimestamp(__pmTimestamp *timestamp)
+{
+    struct timespec ts;
+    int sts;
+
+    if ((sts = pmtimespecNow(&ts)) < 0)
+	return sts;
+    timestamp->sec = ts.tv_sec;
+    timestamp->nsec = ts.tv_nsec;
+    return sts;
+}
+
 /*
  * a : b for pmTimeval ... <0 for a<b, ==0 for a==b, >0 for a>b
  */
@@ -1418,6 +1429,10 @@ pmTypeStr(int type)
     return "???";
 }
 
+/*
+ * thread-safe version of __pmLogMetaTypeStr() ... buflen
+ * should be at least 17
+ */
 char *
 pmTypeStr_r(int type, char *buf, int buflen)
 {
@@ -1425,6 +1440,33 @@ pmTypeStr_r(int type, char *buf, int buflen)
     return buf;
 }
 
+/*
+ * Return the name of metadata record type
+ */
+const char *
+__pmLogMetaTypeStr(int type)
+{
+    static char		*typename[] = {
+	"???", "DESC", "INDOM_V2", "LABEL_V2", "TEXT", "INDOM",
+	"INDOM_DELTA", "LABEL"
+    };
+
+    if (type >= 0 && type < sizeof(typename) / sizeof(typename[0]))
+	return typename[type];
+    return "???";
+}
+
+/*
+ * thread-safe version of __pmLogMetaTypeStr() ... buflen
+ * should be at least 15
+ */
+char *
+__pmLogMetaTypeStr_r(int type, char *buf, int buflen)
+{
+    pmsprintf(buf, buflen, "%s", __pmLogMetaTypeStr(type));
+    return buf;
+}
+
 /*
  * descriptor
  */
@@ -1806,7 +1848,7 @@ PM_FAULT_POINT("libpcp/" __FILE__ ":2", PM_FAULT_ALLOC);
 	msgbuf_tmp = realloc(msgbuf, msgbuflen);
 	if (msgbuf_tmp == NULL) {
 	    int		msgbuflen_tmp = msgbuflen;
-	    msgbuf[msgbuflen - MSGCHUNK] = '\0';
+	    msgbuf[msgbuflen - MSGCHUNK - 1] = '\0';
 	    fprintf(stderr, "%s", msgbuf);
 	    vfprintf(stderr, fmt, arg);
 	    fputc('\n', stderr);
@@ -1950,7 +1992,7 @@ __pmSetClientId(const char *id)
     const char		*name = "pmcd.client.whoami";
     pmID		pmid;
     int			sts;
-    pmResult		store;
+    __pmResult		store;
     pmValueSet		pmvs;
     pmValueBlock	*pmvb;
     char        	host[MAXHOSTNAMELEN];
@@ -2181,7 +2223,7 @@ int
 getmachineid(char *buffer, size_t length)
 {
     FILE *fp = fopen("/etc/machine-id", "r");
-    char machine[MAXMACHINEIDLEN];
+    char machine[MAXMACHINEIDLEN+1];
 
     if (fp) {
 	int i = fscanf(fp, "%" STR(MAXMACHINEIDLEN) "s", machine);
@@ -2827,8 +2869,6 @@ __pmShutdown(void)
 
     if ((sts = __pmShutdownLocal()) < 0 && !code)
 	code = sts;
-    if ((sts = __pmShutdownCertificates()) < 0 && !code)
-	code = sts;
     if ((sts = __pmShutdownSecureSockets()) < 0 && !code)
 	code = sts;
     return code;
@@ -2851,13 +2891,25 @@ __pmMemoryUnmap(void *addr, size_t sz)
 }
 #endif /* !IS_MINGW */
 
+static char	*text_start = NULL;
+
+void
+__pmDumpStackInit(void *addr)
+{
+    /*
+     * Inititialization call ... caller knows where their text
+     * segment starts
+     */
+    text_start = (char *)addr;
+}
+
 #if HAVE_TRACE_BACK_STACK
 #include <libexc.h>
 #define MAX_DEPTH 30	/* max callback procedure depth */
 #define MAX_SIZE 48	/* max function name length */
 
 void
-__pmDumpStack(FILE *f)
+__pmDumpStack(void)
 {
     __uint64_t	call_addr[MAX_DEPTH];
     char	*call_fn[MAX_DEPTH];
@@ -2865,47 +2917,123 @@ __pmDumpStack(FILE *f)
     int		res;
     int		i;
 
+    fprintf(stderr, "Procedure call traceback ...\n");
     for (i = 0; i < MAX_DEPTH; i++)
 	call_fn[i] = names[i];
     res = trace_back_stack(MAX_DEPTH, call_addr, call_fn, MAX_DEPTH, MAX_SIZE);
     for (i = 1; i < res; i++) {
 	if (sizeof(void *) == sizeof(long long))
-	    fprintf(f, "  0x%016llx [%s]\n", call_addr[i], call_fn[i]);
+	    fprintf(stderr, "  0x%016llx [%s]\n", call_addr[i], call_fn[i]);
 	else
-	    fprintf(f, "  0x%08lx [%s]\n", (__uint32_t)call_addr[i], call_fn[i]);
+	    fprintf(stderr, "  0x%08lx [%s]\n", (__uint32_t)call_addr[i], call_fn[i]);
     }
 }
 
 #elif HAVE_BACKTRACE
 #include <execinfo.h>
-#define MAX_DEPTH 30	/* max callback procedure depth */
 
 void
-__pmDumpStack(FILE *f)
+__pmDumpStack(void)
 {
-    int		nframe;
-    void	*buf[MAX_DEPTH];
-    char	**symbols;
-    int		i;
+#define MAX_TRACE_DEPTH 32	/* max callback procedure depth */
+#define MAX_SYMBOL_LENGTH 128	/* max length of a function name */
+    void	*backaddr[MAX_TRACE_DEPTH] = { 0 };
+    int		nsymbols;
+#ifdef HAVE___ETEXT
+    extern char	__etext;
+    char	*text_end = &__etext;
+#else
+#ifdef HAVE__ETEXT
+    extern char	_etext;
+    char	*text_end = &_etext;
+#else
+#ifdef HAVE_ETEXT
+    extern char	etext;
+    char	*text_end = &etext;
+#else
+    char	*text_end = NULL;
+#endif
+#endif
+#endif
 
-    nframe = backtrace(buf, MAX_DEPTH);
-    if (nframe < 1) {
-	fprintf(f, "backtrace -> %d frames?\n", nframe);
-	return;
+    fprintf(stderr, "Procedure call traceback ...\n");
+    if (text_start != NULL)
+	fprintf(stderr, "executable text segment: " PRINTF_P_PFX "%p ... " PRINTF_P_PFX "%p\n", text_start, text_end);
+
+    nsymbols = backtrace(backaddr, MAX_TRACE_DEPTH);
+    if (nsymbols > 0) {
+	int	fd;
+	char	**symbols = NULL;
+	int	i;
+
+	/*
+	 * Possible name collision here to simplify the code ... we unlink()
+	 * as soon as it is created so the risk is small.
+	 */
+	fd = open("/tmp/dumpstack", O_CREAT|O_RDWR, 0644);
+	if (fd < 0) {
+	    fprintf(stderr, "Failed to create \"/tmp/dumpstack\", falling back to backtrace_symbols()\n");
+	    symbols = backtrace_symbols(backaddr, nsymbols);
+	}
+	else {
+	    /*
+	     * Preferred path to avoid calling malloc() in backtrace_symbols()
+	     * in case that's the real cause of a problem that got us here.
+	     */
+	    unlink("/tmp/dumpstack");
+	    backtrace_symbols_fd(backaddr, nsymbols, fd);
+	    lseek(fd, (off_t)0, SEEK_SET);
+	}
+
+	for (i = 0; i < nsymbols; i++) {
+	    if (fd >= 0) {
+		char	c;
+		fprintf(stderr, "  ");
+		for ( ; ; ) {
+		    int		sts;
+		    sts = read(fd, &c, 1);
+		    if (sts < 0) {
+			fprintf(stderr, "Botch: read() returns %d\n", sts);
+			break;
+		    }
+		    else if (sts == 0)
+			break;
+		    else if (c == '\n')
+			break;
+		    fputc(c, stderr);
+		}
+	    }
+	    else if (symbols != NULL)
+		fprintf(stderr, "  %s", symbols[i]);
+	    else
+		fprintf(stderr, "  %p ??unknown??", backaddr[i]);
+	    if (text_start != NULL) {
+		/*
+		 * report address offset from the base of the text segment
+		 * ... this matches addresses from nm(1), but more importantly
+		 * is the address that is needed for addr2line(1)
+		 */
+		if (text_start  <= (char *)backaddr[i] && 
+		    (text_end == NULL || (char *)backaddr[i] <= text_end))
+		    fprintf(stderr, " (0x%llx)", (long long)((char *)backaddr[i]-text_start));
+	    }
+	    fputc('\n', stderr);
+	}
+	if (fd >= 0)
+	    close(fd);
+	else if (symbols != NULL)
+	    free(symbols);
     }
-    symbols = backtrace_symbols(buf, nframe);
-    if (symbols == NULL) {
-	fprintf(f, "backtrace_symbols failed!\n");
-	return;
+    else {
+	fprintf(stderr, "backtrace() returns %d, nothing to report\n", nsymbols);
     }
-    for (i = 1; i < nframe; i++)
-	fprintf(f, "  " PRINTF_P_PFX "%p [%s]\n", buf[i], symbols[i]);
+    return;
 }
 #else	/* no known mechanism, provide a stub (called unconditionally) */
 void
-__pmDumpStack(FILE *f)
+__pmDumpStack(void)
 {
-    fprintf(f, "[No backtrace support available]\n");
+    fprintf(stderr, "[No backtrace support available]\n");
 }
 #endif /* HAVE_BACKTRACE */
 
@@ -2974,3 +3102,63 @@ pmInDom_build(unsigned int domain, unsigned int serial)
     memcpy(&indom, &indom_int, sizeof(indom));
     return indom;
 }
+
+/*
+ * Convert archive feature bits into a string ... result is variable
+ * length and not static, so result is malloc'd and 
+ * __pmLogFeaturesStr_r() is not required.
+ *
+ * Return NULL on alloc error ... caller should test before calling
+ * free().
+ */
+char *
+__pmLogFeaturesStr(__uint32_t features)
+{
+    char	*ans = (char *)malloc(1);
+    int		len = 1;
+    __uint32_t	check = features;
+    int		pos;
+    int		unknown = 0;
+    char	buf[8];		/* enough for bit_NN */
+
+    if (ans == NULL) {
+	pmNoMem("__pmLogFeaturesStr: malloc", 1, PM_RECOV_ERR);
+	return NULL;
+    }
+    ans[0] = '\0';
+
+    for (pos = 31; pos >= 0; pos--) {
+	if (check & 0x80000000) {
+	    char	*ans_tmp;
+	    char	*append;
+	    switch (pos) {
+		case 31:	/* QA */
+			append = "QA";
+			break;
+
+		default:
+			append = buf;
+			snprintf(buf, 8, "bit_%02d", pos);
+			break;
+	    }
+	    len += strlen(append) + 1;
+	    if (unknown > 0)
+		len++;
+	    ans_tmp = (char *)realloc(ans, len);
+	    if (ans_tmp == NULL) {
+		pmNoMem("__pmLogFeaturesStr: realloc", len, PM_RECOV_ERR);
+		/* best we can do ... */
+		free(ans);
+		return NULL;
+	    }
+	    ans = ans_tmp;
+	    if (unknown > 0)
+		strcat(ans, " ");
+	    strcat(ans, append);
+	    unknown++;
+	}
+	check = check << 1;
+    }
+
+    return(ans);
+}
