@@ -238,7 +238,7 @@ __pmdaOpenUnix(char *sockname, int *infd, int *outfd)
     }
     memset(&myaddr, 0, sizeof(myaddr));
     myaddr.sun_family = AF_UNIX;
-    strcpy(myaddr.sun_path, sockname);
+    pmstrncpy(myaddr.sun_path, sizeof(myaddr.sun_path), sockname);
     len = (int)offsetof(struct sockaddr_un, sun_path) + (int)strlen(myaddr.sun_path);
     sts = bind(sfd, (struct sockaddr*) &myaddr, len);
     if (sts < 0) {
@@ -728,7 +728,7 @@ pmdaInit(pmdaInterface *dispatch, pmdaIndom *indoms, int nindoms,
 		if (i == pmda->e_nindoms) {
 		    char	strbuf[20];
 		    pmNotifyErr(LOG_CRIT, 
-				 "pmdaInit: PMDA %s: Undefined instance domain serial (%d) specified in metric %s(%d)\n",
+				 "pmdaInit: PMDA %s: Undefined instance domain serial (%d) specified for metric %s (e_metrics[%d])\n",
 				 pmda->e_name, mindomp->serial, 
 				 pmIDStr_r(pmda->e_metrics[m].m_desc.pmid, strbuf, sizeof(strbuf)), m);
 		    dispatch->status = PM_ERR_GENERIC;
@@ -833,6 +833,7 @@ __pmdaSetupPDU(int infd, int outfd, int flags, const char *agentname)
 	pmNotifyErr(LOG_CRIT, "__pmdaSetupPDU: PMDA %s: version exchange failure, got PDU type %s expecting PDU_CREDS)",
 	    agentname,
 	    __pmPDUTypeStr_r(sts, strbuf, sizeof(strbuf)));
+	__pmDumpPDUTrace(stderr);
     }
 
     if (pinpdu > 0)
