@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017-2021 Red Hat.
+ * Copyright (c) 2017-2022 Red Hat.
  *
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -110,6 +110,7 @@ redis_param_raw(sds cmd, sds param)
 }
 
 extern void redisGlobalsInit(struct dict *);
+extern void redisGlobalsClose(void);
 
 extern void redis_series_source(redisSlots *, void *);
 extern void redis_series_mark(redisSlots *, sds, int, void *);
@@ -125,7 +126,8 @@ typedef struct seriesGetContext {
 
     context_t		context;
     unsigned long long	count;		/* number of samples processed */
-    pmResult		*result;	/* currently active sample data */
+    pmHighResResult	*result;	/* currently active sample data */
+    int			loaded;		/* end of archive data reached */
     int			error;		/* PMAPI error code from fetch */
 
     redisDoneCallBack	done;
@@ -152,6 +154,9 @@ typedef struct seriesLoadBaton {
     const char		**metrics;	/* metric specification strings */
     dict		*errors;	/* PMIDs where errors observed */
     dict		*wanted;	/* allowed metrics list PMIDs */
+    sds			*exclude_patterns;	/* list of exclude metric patterns (e.g. proc.*) */
+    unsigned int	exclude_npatterns;	/* number of exclude metric patterns */
+    dict		*exclude_pmids;		/* dict of excluded pmIDs (pmID: NULL) */
 
     int			error;
     void		*arg;
