@@ -200,7 +200,7 @@ atopsar(int argc, char *argv[])
 			   case 'H':		/* repeat headers          */
 				repeathead = 23;	/* define default  */
 #if defined(TIOCGWINSZ)
-				if (isatty(1))
+				if (isatty(fileno(stdout)))
 				{
 					struct winsize wsz;
 
@@ -311,7 +311,7 @@ atopsar(int argc, char *argv[])
 	*/
 	if (usecolors == 't')
 	{
-		if (! isatty(1) )
+		if (! isatty(fileno(stdout)) )
 			usecolors = 0;
 	}
 
@@ -578,8 +578,9 @@ reportraw(double timenow, double numsecs,
 	double			timed;
 	unsigned int		rv;
 	static unsigned int	curline, headline, sampsum,
-				totalsec, totalexit, lastnpres,
+				totalexit, lastnpres,
 				lastntrun, lastntslpi, lastntslpu, lastnzomb;
+	static time_t		totalsec;
 	static double		lasttime;
 	static struct sstat	totsyst;
 
@@ -1028,7 +1029,7 @@ do_atopsarflags(char *name, char *val)
 		   case 'H':		/* repeat headers          */
 			repeathead = 23;	/* define default  */
 #if defined(TIOCGWINSZ)
-			if (isatty(1))
+			if (isatty(fileno(stdout)))
 			{
 				struct winsize wsz;
 
@@ -1262,10 +1263,6 @@ gpuline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
 	return nlines;
 }
 
- /*
- ** other processor statistics
- */
-
 /*
 ** other processor statistics
 */
@@ -1349,9 +1346,9 @@ memline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
 	unsigned int	mbadness, sbadness;
 
 	if (membadness)
-		mbadness = ((ss->mem.physmem  - ss->mem.freemem -
-	                     ss->mem.cachemem - ss->mem.buffermem)
-	                               * 100.0 / ss->mem.physmem) 
+		mbadness = ((ss->mem.physmem  - ss->mem.freemem 
+	                     - ss->mem.cachemem - ss->mem.buffermem
+	                     + ss->mem.shmem) * 100.0 / ss->mem.physmem) 
 	                               * 100   / membadness;
 	else
 		mbadness = 0;
@@ -1386,7 +1383,7 @@ memline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
 static void
 swaphead(int osvers, int osrel, int ossub)
 {
-	printf("pagescan/s   swapin/s  swapout/s      "
+	printf("pagescan/s  swapin/s swapout/s oomkill"
 	       "  commitspc  commitlim   _swap_");
 }
 
@@ -1417,10 +1414,11 @@ swapline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
 
 	preprint(badness);
 
-	printf("%10.2lf  %9.2lf  %9.2lf       %9lluM %9lluM",
+	printf("%10.2lf %9.2lf %9.2lf %7lld %9lluM %9lluM",
 		(double)ss->mem.pgscans / deltasec,
 		(double)ss->mem.swins   / deltasec,
 		(double)ss->mem.swouts  / deltasec,
+		        ss->mem.oomkills,
 		        ss->mem.committed / 1024,
 		        ss->mem.commitlim / 1024);
 
@@ -1435,8 +1433,7 @@ swapline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
 static void
 psihead(int osvers, int osrel, int ossub)
 {
-	printf("cs_10_60_300   ms_10_60_300 mf_10_60_300   "
-	       "is_10_60_300 if_10_60_300");
+	printf("cpusome    memsome  memfull    iosome  iofull");
 }
 
 static int
@@ -1561,8 +1558,10 @@ gendskline(struct sstat *ss, char *tstamp, char selector)
 		char	*pn;
 		int	len;
 
-		if ( (iotot = dp->nread + dp->nwrite) == 0 &&
- 		     !firstcall && !allresources             )
+		iotot = dp->nread + dp->nwrite +
+		             (dp->ndisc != -1 ? dp->ndisc : 0);
+
+		if (iotot == 0 && !firstcall && !allresources)
 			continue;	/* no activity on this disk */
 
 		/*
@@ -1790,12 +1789,12 @@ ibline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
 			continue;
 
 		/*
-		** convert byte-transfers to bit-transfers     (*       8)
-		** convert bit-transfers  to megabit-transfers (/ 1000000)
+		** convert byte-transfers to bit-transfers     (*          8)
+		** convert bit-transfers  to gigabit-transfers (/ 1000000000)
 		** per second
 		*/
-		ival = ss->ifb.ifb[i].rcvb*ss->ifb.ifb[i].lanes/125000/deltasec;
-		oval = ss->ifb.ifb[i].sndb*ss->ifb.ifb[i].lanes/125000/deltasec;
+		ival = ss->ifb.ifb[i].rcvb*ss->ifb.ifb[i].lanes/125000000/deltasec;
+		oval = ss->ifb.ifb[i].sndb*ss->ifb.ifb[i].lanes/125000000/deltasec;
 
 		/*
 		** calculate busy-percentage for port
@@ -2368,8 +2367,8 @@ httpline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
 static void
 topchead(int osvers, int osrel, int ossub)
 {
-	printf("    pid command  cpu%% |     pid command  cpu%% | "
-	       "    pid command  cpu%%_top3_");
+	printf("  pid command  cpu%% |   pid command  cpu%% | "
+	       "  pid command  cpu%%_top3_");
 }
 
 static int
@@ -2402,21 +2401,21 @@ topcline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
 		availcpu = 1;	/* avoid divide-by-zero */
 
 	if (nactproc >= 1 && (ps[0])->cpu.stime + (ps[0])->cpu.utime > 0)
-	    printf("%7d %-8.8s %3.0lf%% | ",
+	    printf("%5d %-8.8s %3.0lf%% | ",
 	      (ps[0])->gen.pid, (ps[0])->gen.name,
 	      (double)((ps[0])->cpu.stime + (ps[0])->cpu.utime)*100.0/availcpu);
         else
 	    printf("%19s | ", " ");
 
 	if (nactproc >= 2 && (ps[1])->cpu.stime + (ps[1])->cpu.utime > 0)
-	    printf("%7d %-8.8s %3.0lf%% | ",
+	    printf("%5d %-8.8s %3.0lf%% | ",
 	      (ps[1])->gen.pid, (ps[1])->gen.name,
 	      (double)((ps[1])->cpu.stime + (ps[1])->cpu.utime)*100.0/availcpu);
         else
 	    printf("%19s | ", " ");
 
 	if (nactproc >= 3 && (ps[2])->cpu.stime + (ps[2])->cpu.utime > 0)
-	    printf("%7d %-8.8s %3.0lf%%\n",
+	    printf("%5d %-8.8s %3.0lf%%\n",
 	      (ps[2])->gen.pid, (ps[2])->gen.name,
 	      (double)((ps[2])->cpu.stime + (ps[2])->cpu.utime)*100.0/availcpu);
         else
@@ -2432,8 +2431,8 @@ topcline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
 static void
 topmhead(int osvers, int osrel, int ossub)
 {
-	printf("    pid command  mem%% |     pid command  mem%% | "
-	       "    pid command  mem%%_top3_");
+	printf("  pid command  mem%% |   pid command  mem%% | "
+	       "  pid command  mem%%_top3_");
 }
 
 static int
@@ -2458,21 +2457,21 @@ topmline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
 	availmem  = ss->mem.physmem;
 
         if (nactproc >= 1)
-	    printf("%7d %-8.8s %3.0lf%% | ",
+	    printf("%5d %-8.8s %3.0lf%% | ",
 	      (ps[0])->gen.pid, (ps[0])->gen.name,
 	      (double)(ps[0])->mem.rmem * 100.0 / availmem);
         else
 	    printf("%19s | ", " ");
 
         if (nactproc >= 2)
-	    printf("%7d %-8.8s %3.0lf%% | ",
+	    printf("%5d %-8.8s %3.0lf%% | ",
 	      (ps[1])->gen.pid, (ps[1])->gen.name,
 	      (double)(ps[1])->mem.rmem * 100.0 / availmem);
         else
 	    printf("%19s | ", " ");
 
         if (nactproc >= 3)
-	    printf("%7d %-8.8s %3.0lf%%\n",
+	    printf("%5d %-8.8s %3.0lf%%\n",
 	      (ps[2])->gen.pid, (ps[2])->gen.name,
 	      (double)(ps[2])->mem.rmem * 100.0 / availmem);
         else
@@ -2488,8 +2487,8 @@ topmline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
 static void
 topdhead(int osvers, int osrel, int ossub)
 {
-	printf("    pid command  dsk%% |     pid command  dsk%% | "
-	       "    pid command  dsk%%_top3_");
+	printf("  pid command  dsk%% |   pid command  dsk%% | "
+	       "  pid command  dsk%%_top3_");
 }
 
 static int
@@ -2530,21 +2529,21 @@ topdline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
 	qsort(ps, nactproc, sizeof(struct tstat *), compdsk);
 
         if (nactproc >= 1 && (ps[0])->dsk.rio + (ps[0])->dsk.wio > 0)
-	    printf("%7d %-8.8s %3.0lf%% | ",
+	    printf("%5d %-8.8s %3.0lf%% | ",
 	      (ps[0])->gen.pid, (ps[0])->gen.name,
 	      (double)((ps[0])->dsk.rio+(ps[0])->dsk.wio) *100.0/availdsk);
         else
 	    printf("%19s | ", " ");
 
         if (nactproc >= 2 && (ps[1])->dsk.rio + (ps[1])->dsk.wio > 0)
-	    printf("%7d %-8.8s %3.0lf%% | ",
+	    printf("%5d %-8.8s %3.0lf%% | ",
 	      (ps[1])->gen.pid, (ps[1])->gen.name,
 	      (double)((ps[1])->dsk.rio+(ps[1])->dsk.wio) *100.0/availdsk);
         else
 	    printf("%19s | ", " ");
 
         if (nactproc >= 3 && (ps[2])->dsk.rio + (ps[2])->dsk.wio > 0)
-	    printf("%7d %-8.8s %3.0lf%%\n",
+	    printf("%5d %-8.8s %3.0lf%%\n",
 	      (ps[2])->gen.pid, (ps[2])->gen.name,
 	      (double)((ps[2])->dsk.rio+(ps[2])->dsk.wio) *100.0/availdsk);
         else
@@ -2560,8 +2559,8 @@ topdline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
 static void
 topnhead(int osvers, int osrel, int ossub)
 {
-	printf("    pid command  net%% |     pid command  net%% | "
-	       "    pid command  net%%_top3_");
+	printf("  pid command  net%% |   pid command  net%% | "
+	       "  pid command  net%%_top3_");
 }
 
 static int
@@ -2609,7 +2608,7 @@ topnline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
 		           (ps[0])->net.udpssz + (ps[0])->net.udprsz;
 
 		if (totbytes > 0)
-			printf("%7d %-8.8s %3.0lf%% | ",
+			printf("%5d %-8.8s %3.0lf%% | ",
 				(ps[0])->gen.pid, (ps[0])->gen.name,
 				(double)totbytes * 100.0 / availnet);
         	else
@@ -2624,7 +2623,7 @@ topnline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
 		           (ps[1])->net.udpssz + (ps[1])->net.udprsz;
 
 		if (totbytes > 0)
-			printf("%7d %-8.8s %3.0lf%% | ",
+			printf("%5d %-8.8s %3.0lf%% | ",
 				(ps[1])->gen.pid, (ps[1])->gen.name,
 				(double)totbytes * 100.0 / availnet);
         	else
@@ -2639,7 +2638,7 @@ topnline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
 		           (ps[2])->net.udpssz + (ps[2])->net.udprsz;
 
 		if (totbytes > 0)
-			printf("%7d %-8.8s %3.0lf%%\n",
+			printf("%5d %-8.8s %3.0lf%%\n",
 				(ps[2])->gen.pid, (ps[2])->gen.name,
 				(double)totbytes * 100.0 / availnet);
         	else
@@ -2718,7 +2717,7 @@ struct pridef pridef[] =
    {0,  "n",  'U',  udpv6head,	udpv6line,  	"udp  v6",                },
    {0,  "n",  't',  tcphead,	tcpline,  	"tcp        (general)",   },
    {0,  "n",  'T',  TCPhead,	TCPline,  	"tcp        (errors)",    },
-   {0,  "n",  'h',  httphead,	httpline,  	"HTTP activity",          },
+   {0,  "n",  'o',  httphead,	httpline,  	"HTTP activity",          },
    {0,  "",   'O',  topchead,	topcline,  	"top-3 processes cpu",    },
    {0,  "",   'G',  topmhead,	topmline,  	"top-3 processes memory", },
    {0,  "",   'D',  topdhead,	topdline,  	"top-3 processes disk",   },
