@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2015-2019 Red Hat.
+** Copyright (C) 2015-2022 Red Hat.
 ** Copyright (C) 2000-2012 Gerlof Langeveld.
 **
 ** This program is free software; you can redistribute it and/or modify it
@@ -76,58 +76,70 @@ sstat_alloc(const char *purpose)
 void
 sstat_reset(struct sstat *sstat)
 {
-	void		*cpu, *gpu, *intf, *ifb, *mdd, *lvm, *dsk, *nfs;
-	unsigned int	nrcpu, nrgpus, nrintf, nrports, nrdsk, nrlvm, nrmdd, nrnfs;
+	void		*cpu, *gpu, *intf, *ifb, *memnuma, *cpunuma;
+	void	       	*dsk, *lvm, *mdd, *nfs;
+	unsigned int	nrcpu, nrgpus, nrintf, nrports, nrmemnuma, nrcpunuma;
+	unsigned int	nrdsk, nrlvm, nrmdd, nrnfs;
 
 	cpu = sstat->cpu.cpu;
+	gpu = sstat->gpu.gpu;
 	intf = sstat->intf.intf;
 	ifb = sstat->ifb.ifb;
 	dsk = sstat->dsk.dsk;
 	lvm = sstat->dsk.lvm;
 	mdd = sstat->dsk.mdd;
 	nfs = sstat->nfs.nfsmounts.nfsmnt;
-	gpu = sstat->gpu.gpu;
+	memnuma = sstat->memnuma.numa;
+	cpunuma = sstat->cpunuma.numa;
 
 	nrcpu = sstat->cpu.nrcpu;
 	nrintf = sstat->intf.nrintf;
 	nrports = sstat->ifb.nrports;
+	nrgpus = sstat->gpu.nrgpus;
 	nrdsk = sstat->dsk.ndsk;
 	nrlvm = sstat->dsk.nlvm;
 	nrmdd = sstat->dsk.nmdd;
 	nrnfs = sstat->nfs.nfsmounts.nrmounts;
-	nrgpus = sstat->gpu.nrgpus;
+	nrmemnuma = sstat->memnuma.nrnuma;
+	nrcpunuma = sstat->cpunuma.nrnuma;
 
 	/* clear fixed portion now that pointers/sized are safe */
 	memset(sstat, 0, sizeof(struct sstat));
 
 	/* clear the dynamically-sized buffers and restore ptrs */
 	memset(cpu, 0, sizeof(struct percpu) * nrcpu);
+	memset(gpu, 0, sizeof(struct pergpu) * nrgpus);
 	memset(intf, 0, sizeof(struct perintf) * nrintf);
 	memset(ifb, 0, sizeof(struct perifb) * nrports);
 	memset(dsk, 0, sizeof(struct perdsk) * nrdsk);
 	memset(lvm, 0, sizeof(struct perdsk) * nrlvm);
 	memset(mdd, 0, sizeof(struct perdsk) * nrmdd);
 	memset(nfs, 0, sizeof(struct pernfsmount) * nrnfs);
-	memset(gpu, 0, sizeof(struct pergpu) * nrgpus);
+	memset(memnuma, 0, sizeof(struct mempernuma) * nrmemnuma);
+	memset(cpunuma, 0, sizeof(struct cpupernuma) * nrcpunuma);
 
 	/* stitch the main sstat buffer back together once more */
 	sstat->cpu.cpu = cpu;
+	sstat->gpu.gpu = gpu;
 	sstat->intf.intf = intf;
 	sstat->ifb.ifb = ifb;
 	sstat->dsk.dsk = dsk;
 	sstat->dsk.lvm = lvm;
 	sstat->dsk.mdd = mdd;
 	sstat->nfs.nfsmounts.nfsmnt = nfs;
-	sstat->gpu.gpu = gpu;
+	sstat->memnuma.numa = memnuma;
+	sstat->cpunuma.numa = cpunuma;
 
 	sstat->cpu.nrcpu = nrcpu;
+	sstat->gpu.nrgpus = nrgpus;
 	sstat->intf.nrintf = nrintf;
 	sstat->ifb.nrports = nrports;
 	sstat->dsk.ndsk = nrdsk;
 	sstat->dsk.nlvm = nrlvm;
 	sstat->dsk.nmdd = nrmdd;
 	sstat->nfs.nfsmounts.nrmounts = nrnfs;
-	sstat->gpu.nrgpus = nrgpus;
+	sstat->memnuma.nrnuma = nrmemnuma;
+	sstat->cpunuma.nrnuma = nrcpunuma;
 }
 
 static void
@@ -145,9 +157,15 @@ update_processor(struct percpu *cpu, int id, pmResult *result, pmDesc *descs, in
 	cpu->steal = extract_count_t_inst(result, descs, PERCPU_STEAL, id, offset);
 	cpu->guest = extract_count_t_inst(result, descs, PERCPU_GUEST, id, offset);
 
-	memset(&cpu->freqcnt, 0, sizeof(cpu->freqcnt));
-	cpu->freqcnt.cnt = extract_count_t_inst(result, descs, PERCPU_FREQCNT_CNT, id, offset);
+	cpu->freqcnt.cnt = extract_count_t_inst(result, descs, PERCPU_FREQ_CNT, id, offset);
+	if (cpu->freqcnt.cnt == 0)
+		cpu->freqcnt.cnt = extract_count_t_inst(result, descs, PERCPU_FREQ_CLK, id, offset);
+	cpu->freqcnt.ticks = extract_count_t_inst(result, descs, PERCPU_FREQ_HIT, id, offset);
+	cpu->freqcnt.maxfreq = extract_count_t_inst(result, descs, PERCPU_FREQ_MAX, id, offset);
+
 	cpu->instr = extract_count_t_inst(result, descs, PERCPU_PERF_INSTR, id, offset);
+	if (cpu->instr == 0)	/* try ix86arch INSTRUCTION_RETIRED fallback */
+		cpu->instr = extract_count_t_inst(result, descs, PERCPU_PERF_INST1, id, offset);
 	cpu->cycle = extract_count_t_inst(result, descs, PERCPU_PERF_CYCLE, id, offset);
 }
 
@@ -199,6 +217,44 @@ update_ibport(struct perifb *ib, int id, char *name, pmResult *rp, pmDesc *dp, i
 	ib->sndp = extract_count_t_inst(rp, dp, PERIFB_PORT_OUTPKT, id, offset);
 }
 
+static void
+update_memnuma(struct mempernuma *nmp, int id, char *name, pmResult *rp, pmDesc *dp, int offset)
+{
+	nmp->numanr = id;
+
+	nmp->frag = extract_count_t_inst(rp, dp, PERNODE_MEM_EXTFRAG, id, offset);
+	nmp->totmem = extract_count_t_inst(rp, dp, PERNODE_MEM_TOTAL, id, offset);
+	nmp->freemem = extract_count_t_inst(rp, dp, PERNODE_MEM_FREE, id, offset);
+	nmp->filepage = extract_count_t_inst(rp, dp, PERNODE_MEM_FILE, id, offset);
+	nmp->dirtymem = extract_count_t_inst(rp, dp, PERNODE_MEM_DIRTY, id, offset);
+	nmp->slabmem = extract_count_t_inst(rp, dp, PERNODE_MEM_SLAB, id, offset);
+	nmp->slabreclaim = extract_count_t_inst(rp, dp, PERNODE_MEM_SRECLAIM, id, offset);
+	nmp->active = extract_count_t_inst(rp, dp, PERNODE_MEM_ACTIVE, id, offset);
+	nmp->inactive = extract_count_t_inst(rp, dp, PERNODE_MEM_INACTIVE, id, offset);
+	nmp->shmem = extract_count_t_inst(rp, dp, PERNODE_MEM_SHMEM, id, offset);
+	nmp->tothp = extract_count_t_inst(rp, dp, PERNODE_MEM_HUGEPAGES, id, offset);
+}
+
+static void
+update_cpunuma(struct cpupernuma *ncp, int id, char *name, pmResult *rp, pmDesc *dp, int offset)
+{
+	/* iterate over cpu_node map and count CPUs in this NUMA node */
+	ncp->numanr = id;
+	ncp->nrcpu = extract_integer_instmap_count(rp, dp, MAP_CPU_NODE, id);
+	if (ncp->nrcpu < 1)
+	    ncp->nrcpu = 1;
+
+	ncp->stime = extract_count_t_inst(rp, dp, PERNODE_CPU_SYS, id, offset);
+	ncp->utime = extract_count_t_inst(rp, dp, PERNODE_CPU_USER, id, offset);
+	ncp->ntime = extract_count_t_inst(rp, dp, PERNODE_CPU_NICE, id, offset);
+	ncp->itime = extract_count_t_inst(rp, dp, PERNODE_CPU_IDLE, id, offset);
+	ncp->wtime = extract_count_t_inst(rp, dp, PERNODE_CPU_IOWAIT, id, offset);
+	ncp->Itime = extract_count_t_inst(rp, dp, PERNODE_CPU_IRQ_HARD, id, offset);
+	ncp->Stime = extract_count_t_inst(rp, dp, PERNODE_CPU_IRQ_SOFT, id, offset);
+	ncp->steal = extract_count_t_inst(rp, dp, PERNODE_CPU_STEAL, id, offset);
+	ncp->guest = extract_count_t_inst(rp, dp, PERNODE_CPU_GUEST, id, offset);
+}
+
 static void
 update_disk(struct perdsk *dsk, int id, char *name, pmResult *rp, pmDesc *dp, int offset)
 {
@@ -211,6 +267,9 @@ update_disk(struct perdsk *dsk, int id, char *name, pmResult *rp, pmDesc *dp, in
 	dsk->nwsect = extract_count_t_inst(rp, dp, PERDISK_NWSECT, id, offset);
 	dsk->io_ms = extract_count_t_inst(rp, dp, PERDISK_IO_MS, id, offset);
 	dsk->avque = extract_count_t_inst(rp, dp, PERDISK_AVEQ, id, offset);
+	dsk->ndisc = extract_count_t_inst(rp, dp, PERDISK_NDISC, id, offset);
+	dsk->ndsect = extract_count_t_inst(rp, dp, PERDISK_NDSECT, id, offset);
+	dsk->inflight = extract_count_t_inst(rp, dp, PERDISK_INFLIGHT, id, offset);
 }
 
 static void
@@ -225,6 +284,9 @@ update_lvm(struct perdsk *dsk, int id, char *name, pmResult *rp, pmDesc *dp, int
 	dsk->nwsect = extract_count_t_inst(rp, dp, PERDM_NWSECT, id, offset);
 	dsk->io_ms = 0;
 	dsk->avque = 0;
+	dsk->ndisc = extract_count_t_inst(rp, dp, PERDM_NDISC, id, offset);
+	dsk->ndsect = extract_count_t_inst(rp, dp, PERDM_NDSECT, id, offset);
+	dsk->inflight = extract_count_t_inst(rp, dp, PERDM_INFLIGHT, id, offset);
 }
 
 static void
@@ -239,6 +301,9 @@ update_mdd(struct perdsk *dsk, int id, char *name, pmResult *rp, pmDesc *dp, int
 	dsk->nwsect = extract_count_t_inst(rp, dp, PERMD_NWSECT, id, offset);
 	dsk->io_ms = 0;
 	dsk->avque = 0;
+	dsk->ndisc = extract_count_t_inst(rp, dp, PERMD_NDISC, id, offset);
+	dsk->ndsect = extract_count_t_inst(rp, dp, PERMD_NDSECT, id, offset);
+	dsk->inflight = extract_count_t_inst(rp, dp, PERMD_INFLIGHT, id, offset);
 }
 
 static void
@@ -261,28 +326,57 @@ update_mnt(struct pernfsmount *mp, int id, char *name, pmResult *rp, pmDesc *dp,
 	mp->pagesmwrite = extract_count_t_inst(rp, dp, PERNFS_WRPAGES, id, offset);
 }
 
+static void
+update_gpu(struct pergpu *gpu, int id, char *name, pmResult *rp, pmDesc *dp, int offset)
+{
+	gpu->gpunr = id;
+
+	extract_string_inst(rp, dp, GPU_CARDNAME, &gpu->type[0],
+				sizeof(gpu->type)-1, id, offset);
+	gpu->type[sizeof(gpu->type)-1] = '\0';
+
+	extract_string_inst(rp, dp, GPU_BUSID, &gpu->busid[0],
+				sizeof(gpu->busid)-1, id, offset);
+	gpu->busid[sizeof(gpu->busid)-1] = '\0';
+
+	gpu->samples = extract_count_t_inst(rp, dp, GPU_SAMPLES, id, offset);
+	gpu->nrprocs = extract_integer_inst(rp, dp, GPU_NGPUPROCS, id, offset);
+	gpu->gpupercnow = extract_integer_inst(rp, dp, GPU_GPUBUSY, id, offset);
+	gpu->mempercnow = extract_integer_inst(rp, dp, GPU_MEMBUSY, id, offset);
+	gpu->memtotnow = extract_count_t_inst(rp, dp, GPU_MEMTOTAL, id, offset);
+	gpu->memtotnow /= 1024;	/* convert to KiB */
+	gpu->memusenow = extract_count_t_inst(rp, dp, GPU_MEMUSED, id, offset);
+	gpu->memusenow /= 1024;	/* convert to KiB */
+	gpu->gpuperccum = extract_count_t_inst(rp, dp, GPU_GPUBUSY_ACCUM, id, offset);
+	gpu->memperccum = extract_count_t_inst(rp, dp, GPU_MEMBUSY_ACCUM, id, offset);
+	gpu->memusecum = extract_count_t_inst(rp, dp, GPU_MEMUSED_ACCUM, id, offset);
+	gpu->memusecum /= 1024;	/* convert to KiB */
+}
+
+static pmID	pmids[SYST_NMETRICS];
+static pmDesc	descs[SYST_NMETRICS];
+
+void
+setup_photosyst(void)
+{
+	setup_metrics(systmetrics, pmids, descs, SYST_NMETRICS);
+}
+
 char
 photosyst(struct sstat *si)
 {
-	static int	setup;
 	count_t		count;
 	unsigned int	nrcpu, nrdisk, nrintf, nrports;
 	unsigned int	onrcpu, onrdisk, onrintf, onrports;
-	unsigned int	nrlvm, nrmdd, nrnfs;
-	unsigned int	onrlvm, onrmdd, onrnfs;
-	static pmID	pmids[SYST_NMETRICS];
-	static pmDesc	descs[SYST_NMETRICS];
+	unsigned int	nrmemnuma, nrcpunuma;
+	unsigned int	nrlvm, nrmdd, nrnfs, nrgpus;
+	unsigned int	onrlvm, onrmdd, onrnfs, onrgpus;
+	unsigned int	onrmemnuma, onrcpunuma;
 	pmResult	*result;
 	size_t		size;
 	char		**insts;
 	int		*ids, i;
 
-	if (!setup)
-	{
-		setup_metrics(systmetrics, pmids, descs, SYST_NMETRICS);
-		setup = 1;
-	}
-
 	if (fetch_metrics("system", SYST_NMETRICS, pmids, &result) < 0)
 		return MRESET;
 
@@ -293,6 +387,9 @@ photosyst(struct sstat *si)
 	onrlvm  = si->dsk.nlvm;
 	onrmdd  = si->dsk.nmdd;
 	onrnfs  = si->nfs.nfsmounts.nrmounts;
+	onrgpus = si->gpu.nrgpus;
+	onrmemnuma = si->memnuma.nrnuma;
+	onrcpunuma = si->cpunuma.nrnuma;
 
 	sstat_reset(si);
 	si->stamp = result->timestamp;
@@ -306,7 +403,6 @@ photosyst(struct sstat *si)
 	si->cpu.csw = extract_count_t(result, descs, CPU_CSW);
 	si->cpu.devint = extract_count_t(result, descs, CPU_DEVINT);
 	si->cpu.nprocs = extract_count_t(result, descs, CPU_NPROCS);
-	si->cpu.btime  = extract_count_t(result, descs, CPU_BTIME);
 	si->cpu.all.utime = extract_count_t(result, descs, CPU_UTIME);
 	si->cpu.all.ntime = extract_count_t(result, descs, CPU_NTIME);
 	si->cpu.all.stime = extract_count_t(result, descs, CPU_STIME);
@@ -328,7 +424,7 @@ photosyst(struct sstat *si)
 	{
 		size = nrcpu * sizeof(struct percpu);
 		si->cpu.cpu = (struct percpu *)realloc(si->cpu.cpu, size);
-		ptrverify(si->cpu.cpu, "photosyst cpus [%ld]", (long)size);
+		ptrverify(si->cpu.cpu, "photosyst cpus [%zu]", size);
 	}
 	for (i=0; i < nrcpu; i++)
 	{
@@ -368,9 +464,16 @@ photosyst(struct sstat *si)
 	count += extract_count_t(result, descs, MEM_STEAL_MOVABLE);
 	count += extract_count_t(result, descs, MEM_STEAL_NORMAL);
 	si->mem.pgsteal = count;
+	si->mem.oomkills = extract_count_t(result, descs, MEM_OOM_KILL);
+	si->mem.pgins = extract_count_t(result, descs, MEM_PGPGIN);
+	si->mem.pgouts = extract_count_t(result, descs, MEM_PGPGOUT);
 	si->mem.allocstall = extract_count_t(result, descs, MEM_ALLOCSTALL);
+	si->mem.compactstall = extract_count_t(result, descs, MEM_COMPACTSTALL);
+	si->mem.pgmigrate = extract_count_t(result, descs, MEM_PGMIGRATE);
+	si->mem.numamigrate = extract_count_t(result, descs, MEM_NUMAMIGRATE);
 
 	/* /proc/meminfo */
+	si->mem.swapcached = extract_count_t(result, descs, MEM_SWAPCACHED);
 	si->mem.cachemem = extract_count_t(result, descs, MEM_CACHEMEM);
 	si->mem.cachedrt = extract_count_t(result, descs, MEM_CACHEDRT);
 	si->mem.physmem = extract_count_t(result, descs, MEM_PHYSMEM);
@@ -383,10 +486,18 @@ photosyst(struct sstat *si)
 	si->mem.slabreclaim = extract_count_t(result, descs, MEM_SLABRECLAIM);
 	si->mem.committed = extract_count_t(result, descs, MEM_COMMITTED);
 	si->mem.commitlim = extract_count_t(result, descs, MEM_COMMITLIM);
+	si->mem.pagetables = extract_count_t(result, descs, MEM_PAGETABLES);
 	si->mem.tothugepage = extract_count_t(result, descs, MEM_TOTHUGEPAGE);
 	si->mem.freehugepage = extract_count_t(result, descs, MEM_FREEHUGEPAGE);
 	si->mem.hugepagesz = extract_count_t(result, descs, HUGEPAGESZ);
 
+	/* /sys/kernel/debug/vmmemctl or /proc/vmmemctl */
+	si->mem.vmwballoon = (count_t) -1; /* TODO */
+
+	/* /sys/kernel/mm/ksm */
+	si->mem.ksmshared = extract_count_t(result, descs, MEM_KSMSHARED);
+	si->mem.ksmsharing = extract_count_t(result, descs, MEM_KSMSHARING);
+
 	/* shmctl(2) */
 	si->mem.shmrss = extract_count_t(result, descs, MEM_SHMRSS) * 1024;
 	si->mem.shmswp = extract_count_t(result, descs, MEM_SHMSWP) * 1024;
@@ -405,7 +516,7 @@ photosyst(struct sstat *si)
 	{
 		size = (nrintf + 1) * sizeof(struct perintf);
 		si->intf.intf = (struct perintf *)realloc(si->intf.intf, size);
-		ptrverify(si->intf.intf, "photosyst intf [%d]\n", (long)size);
+		ptrverify(si->intf.intf, "photosyst intf [%zu]\n", size);
 	}
 	for (i=0; i < nrintf; i++)
 	{
@@ -555,7 +666,7 @@ photosyst(struct sstat *si)
 	{
 		size = (nrdisk + 1) * sizeof(struct perdsk);
 		si->dsk.dsk = (struct perdsk *)realloc(si->dsk.dsk, size);
-		ptrverify(si->dsk.dsk, "photosyst disk [%ld]\n", (long)size);
+		ptrverify(si->dsk.dsk, "photosyst disk [%zu]\n", size);
 	}
 	for (i=0; i < nrdisk; i++)
 	{
@@ -577,7 +688,7 @@ photosyst(struct sstat *si)
 	{
 		size = (nrlvm + 1) * sizeof(struct perdsk);
 		si->dsk.lvm = (struct perdsk *)realloc(si->dsk.lvm, size);
-		ptrverify(si->dsk.lvm, "photosyst lvm [%ld]\n", (long)size);
+		ptrverify(si->dsk.lvm, "photosyst lvm [%zu]\n", size);
 	}
 	for (i=0; i < nrlvm; i++)
 	{
@@ -599,7 +710,7 @@ photosyst(struct sstat *si)
 	{
 		size = (nrmdd + 1) * sizeof(struct perdsk);
 		si->dsk.mdd = (struct perdsk *)realloc(si->dsk.mdd, size);
-		ptrverify(si->dsk.mdd, "photosyst md [%ld]\n", (long)size);
+		ptrverify(si->dsk.mdd, "photosyst md [%zu]\n", size);
 	}
 
 	for (i=0; i < nrmdd; i++)
@@ -655,7 +766,7 @@ photosyst(struct sstat *si)
 	{
 		size = (nrnfs + 1) * sizeof(struct pernfsmount);
 		si->nfs.nfsmounts.nfsmnt = (struct pernfsmount *)realloc(si->nfs.nfsmounts.nfsmnt, size);
-		ptrverify(si->nfs.nfsmounts.nfsmnt, "photosyst nfs [%ld]\n", (long)size);
+		ptrverify(si->nfs.nfsmounts.nfsmnt, "photosyst nfs [%zu]\n", size);
 	}
 
 	for (i=0; i < nrnfs; i++)
@@ -695,6 +806,68 @@ photosyst(struct sstat *si)
 	si->psi.iofull.avg300 = extract_count_t_inst(result, descs, PSI_IOFULL_AVG, 300, 2);
 	si->psi.iofull.total = extract_count_t(result, descs, PSI_IOFULL_TOTAL);
 
+	/* /proc/spl/kstat/zfs/arcstats */
+	si->mem.zfsarcsize = extract_count_t(result, descs, ZFS_ARCSIZE);
+
+	/* /sys/kernel/debug/zswap */
+	si->mem.zswtotpool = extract_count_t(result, descs, ZSWAP_TOTALSIZE);
+	si->mem.zswstored = extract_count_t(result, descs, ZSWAP_STOREDMEM);
+
+	/* NUMA memory metrics */
+	insts = NULL; /* silence coverity */
+	ids = NULL;
+	nrmemnuma = get_instances("memnuma", PERNODE_MEM_TOTAL, descs, &ids, &insts);
+	if (nrmemnuma > onrmemnuma)
+	{
+		size = (nrmemnuma + 1) * sizeof(struct mempernuma);
+		si->memnuma.numa = (struct mempernuma *)realloc(si->memnuma.numa, size);
+		ptrverify(si->memnuma.numa, "photosyst memnuma [%zu]\n", size);
+	}
+
+	for (i=0; i < nrmemnuma; i++)
+	{
+		if (pmDebugOptions.appl0)
+			fprintf(stderr, "%s: updating NUMA node %d mem: %s\n",
+				pmGetProgname(), ids[i], insts[i]);
+		update_memnuma(&si->memnuma.numa[i], ids[i], insts[i], result, descs, i);
+	}
+	si->memnuma.nrnuma = nrmemnuma;
+	free(insts);
+	free(ids);
+
+	/* NUMA CPU metrics */
+	insts = NULL; /* silence coverity */
+	ids = NULL;
+	nrcpunuma = get_instances("cpunuma", PERNODE_CPU_USER, descs, &ids, &insts);
+	if (nrcpunuma > onrcpunuma)
+	{
+		size = (nrcpunuma + 1) * sizeof(struct cpupernuma);
+		si->cpunuma.numa = (struct cpupernuma *)realloc(si->cpunuma.numa, size);
+		ptrverify(si->cpunuma.numa, "photosyst cpunuma [%zu]\n", size);
+	}
+
+	for (i=0; i < nrcpunuma; i++)
+	{
+		if (pmDebugOptions.appl0)
+			fprintf(stderr, "%s: updating NUMA node %d cpu: %s\n",
+				pmGetProgname(), ids[i], insts[i]);
+		update_cpunuma(&si->cpunuma.numa[i], ids[i], insts[i], result, descs, i);
+	}
+	si->cpunuma.nrnuma = nrcpunuma;
+	free(insts);
+	free(ids);
+
+	/* IP version 4: TCP & UDP memory allocations - /proc/net/sockstat */
+	si->mem.tcpsock = extract_count_t(result, descs, NET_SOCKSTAT_TCPMEM);
+	si->mem.udpsock = extract_count_t(result, descs, NET_SOCKSTAT_UDPMEM);
+
+	/* TODO - LLC statistics */
+#if 0
+	si->llc.perllc[n].occupancy;
+	si->llc.perllc[n].mbm_local;
+	si->llc.perllc[n].mbm_total;
+#endif
+
 	/* Infiniband statistics */
 	insts = NULL; /* silence coverity */
 	ids = NULL;
@@ -703,7 +876,7 @@ photosyst(struct sstat *si)
 	{
 		size = (nrports + 1) * sizeof(struct perifb);
 		si->ifb.ifb = (struct perifb *)realloc(si->ifb.ifb, size);
-		ptrverify(si->ifb.ifb, "photosyst ifb [%d]\n", (long)size);
+		ptrverify(si->ifb.ifb, "photosyst ifb [%zu]\n", size);
 	}
 
 	for (i=0; i < nrports; i++)
@@ -725,6 +898,29 @@ photosyst(struct sstat *si)
 	si->www.bworkers  = extract_integer(result, descs, WWW_BWORKERS);
 	si->www.iworkers  = extract_integer(result, descs, WWW_IWORKERS);
 
+	/* GPU statistics */
+	insts = NULL;
+	ids = NULL;
+	nrgpus = get_instances("gpus", GPU_GPUBUSY, descs, &ids, &insts);
+	if (nrgpus > onrgpus)
+	{
+		size = (nrgpus + 1) * sizeof(struct pergpu);
+		si->gpu.gpu = (struct pergpu *)realloc(si->gpu.gpu, size);
+		ptrverify(si->gpu.gpu, "photosyst gpus [%zu]\n", size);
+	}
+
+	for (i=0; i < nrgpus; i++)
+	{
+		if (pmDebugOptions.appl0)
+			fprintf(stderr, "%s: updating GPU %d: %s\n",
+				pmGetProgname(), ids[i], insts[i]);
+		update_gpu(&si->gpu.gpu[i], ids[i], insts[i], result, descs, i);
+	}
+	si->gpu.gpu[nrgpus].type[0] = '\0'; 
+	si->gpu.nrgpus = nrgpus;
+	free(insts);
+	free(ids);
+
 	pmFreeResult(result);
 	return '\0';
 }
