@@ -7,7 +7,7 @@
 ** Include-file describing system-level counters maintained.
 **
 ** Copyright (C) 1996-2014 Gerlof Langeveld
-** Copyright (C) 2015 Red Hat.
+** Copyright (C) 2015,2021 Red Hat.
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -29,7 +29,6 @@
 #define	MAXIBNAME	12
 
 /************************************************************************/
-
 struct	memstat {
 	count_t	physmem;	// number of physical pages
 	count_t	freemem;	// number of free     pages
@@ -46,6 +45,8 @@ struct	memstat {
 	count_t	allocstall;	// try to free pages forced
 	count_t	swouts;		// number of pages swapped out
 	count_t	swins;		// number of pages swapped in
+	count_t	tcpsock;	// number of pages allocated by TCP sockets
+	count_t	udpsock;	// number of pages allocated by UDP sockets
 
 	count_t	commitlim;	// commit limit in pages
 	count_t	committed;	// number of reserved pages
@@ -61,6 +62,62 @@ struct	memstat {
 	count_t	hugepagesz;	// huge page size (bytes)
 
 	count_t	vmwballoon;	// vmware claimed balloon pages
+	count_t	zfsarcsize;	// zfsonlinux ARC size (pages)
+	count_t swapcached;	// swap cache (pages)
+	count_t	ksmsharing;	// saved i.e. deduped memory (pages)
+	count_t	ksmshared;	// current size shared pages (pages)
+	count_t	zswstored;	// zswap stored pages (pages)
+	count_t	zswtotpool;	// total pool size (pages)
+	count_t	oomkills;	// number of oom killings
+	count_t	compactstall;	// counter for process stalls
+	count_t	pgmigrate;	// counter for migrated successfully (pages)
+	count_t	numamigrate;	// counter for numa migrated (pages)
+	count_t	pgouts;		// total number of pages written to block device
+	count_t	pgins;		// total number of pages read from block device
+	count_t	pagetables;	// page tables of processes (pages)
+};
+
+/************************************************************************/
+
+struct	mempernuma {
+	int	numanr;
+	float	frag;		// fragmentation level for this numa
+	count_t	totmem;		// number of physical pages for this numa
+	count_t	freemem;	// number of free     pages for this numa
+	count_t	filepage;	// number of file     pages for this numa
+	count_t	dirtymem;	// number of cache    pages (dirty) for this numa
+	count_t	slabmem;	// number of slab     pages for this numa
+	count_t	slabreclaim;	// reclaimable slab (pages) for this numa
+
+	count_t	active;		// number of pages used more recently for this numa
+	count_t	inactive;	// number of pages less recently used for this numa
+
+	count_t	shmem;		// tot shmem incl. tmpfs (pages) for this numa
+	count_t	tothp;		// total huge pages (huge pages) for this numa
+};
+
+struct	memnuma {
+	count_t           nrnuma;		/* the counts of numa		*/
+	struct mempernuma *numa;
+};
+
+struct	cpupernuma {
+	int	numanr;
+	count_t	nrcpu;		// number of cpu's
+	count_t	stime;		// accumulate system  time in clock ticks for per numa
+	count_t	utime;		// accumulate user    time in clock ticks for per numa
+	count_t	ntime;		// accumulate nice    time in clock ticks for per numa
+	count_t	itime;		// accumulate idle    time in clock ticks for per numa
+	count_t	wtime;		// accumulate iowait  time in clock ticks for per numa
+	count_t	Itime;		// accumulate irq     time in clock ticks for per numa
+	count_t	Stime;		// accumulate softirq time in clock ticks for per numa
+	count_t	steal;		// accumulate steal   time in clock ticks for per numa
+	count_t	guest;		// accumulate guest   time in clock ticks for per numa
+};
+
+struct	cpunuma {
+	count_t           nrnuma;		/* the counts of numa		*/
+	struct cpupernuma *numa;
 };
 
 /************************************************************************/
@@ -83,7 +140,7 @@ struct freqcnt {
         count_t maxfreq;/* frequency in MHz                    */
         count_t cnt;    /* number of clock ticks times state   */
         count_t ticks;  /* number of total clock ticks         */
-                        /* if zero, cnt is actul freq          */
+                        /* if zero, cnt is actual freq         */
 };
 
 struct percpu {
@@ -107,7 +164,6 @@ struct	cpustat {
 	count_t	devint;	/* number of device interrupts 		*/
 	count_t	csw;	/* number of context switches		*/
 	count_t	nprocs;	/* number of processes started          */
-	count_t	btime;	/* boot time                            */
 	float	lavg1;	/* load average last    minute          */
 	float	lavg5;	/* load average last  5 minutes         */
 	float	lavg15;	/* load average last 15 minutes         */
@@ -120,12 +176,15 @@ struct	cpustat {
 
 struct	perdsk {
         char	name[MAXDKNAM];	/* empty string for last        */
-        count_t	nread;	/* number of read  transfers            */
-        count_t	nrsect;	/* number of sectors read               */
-        count_t	nwrite;	/* number of write transfers            */
-        count_t	nwsect;	/* number of sectors written            */
-        count_t	io_ms;	/* number of millisecs spent for I/O    */
-        count_t	avque;	/* average queue length                 */
+        count_t	nread;		/* number of read  transfers            */
+        count_t	nrsect;		/* number of sectors read               */
+        count_t	nwrite;		/* number of write transfers            */
+        count_t	nwsect;		/* number of sectors written            */
+        count_t	io_ms;		/* number of millisecs spent for I/O    */
+        count_t	avque;		/* average queue length                 */
+        count_t	ndisc;		/* number of discards (-1 = unavailable)*/
+        count_t	ndsect;		/* number of sectors discarded          */
+        count_t	inflight;	/* number of inflight I/O               */
 };
 
 struct dskstat {
@@ -163,7 +222,7 @@ struct	perintf {
 	count_t	speed;	/* interface speed in megabits/second	*/
 	count_t	speedp;	/* previous interface speed 		*/
 	char	duplex;	/* full duplex (boolean) 		*/
-	char 	type;	/* interface type ('e'/'w'/'?')  	*/
+	char 	type;	/* interface type ('e'/'w'/'v'/'?')  	*/
 };
 
 struct intfstat {
@@ -175,8 +234,8 @@ struct intfstat {
 
 struct  pernfsmount {
 	char 	mountdev[MAXMNTNAME];		/* mountdevice 		*/
-	count_t	age;			/* number of seconds mounted	*/
-
+        count_t	age;			/* number of seconds mounted	*/
+	
 	count_t	bytesread;		/* via normal reads		*/
 	count_t	byteswrite;		/* via normal writes		*/
 	count_t	bytesdread;		/* via direct reads		*/
@@ -220,20 +279,20 @@ struct nfsstat {
 	} client;
 
 	struct {
-		int             	nrmounts;
+        	int             	nrmounts;
 		struct pernfsmount	*nfsmnt;
 	} nfsmounts;
 };
 
 /************************************************************************/
-struct psi {
+struct	psi {
 	float	avg10;		// average pressure last 10 seconds
 	float	avg60;		// average pressure last 60 seconds
 	float	avg300;		// average pressure last 300 seconds
 	count_t	total;		// total number of milliseconds
 };
 
-struct pressure {
+struct	pressure {
 	char	   present;	/* pressure stats supported?	*/
 	char       future[3];
 	struct psi cpusome;	/* pressure stall info 'some'   */
@@ -318,6 +377,20 @@ struct ifbstat {
 	int		nrports;	// total number of IB ports
 	struct perifb   *ifb;
 };
+
+/************************************************************************/
+struct perllc {
+	unsigned char	id;
+	float		occupancy;
+	count_t		mbm_local;
+	count_t		mbm_total;
+};
+
+struct llcstat {
+	unsigned char	nrllcs;	        // total number of LLC
+	struct perllc   *perllc;
+};
+
 /************************************************************************/
 
 struct	sstat {
@@ -327,12 +400,15 @@ struct	sstat {
 	struct memstat	mem;
 	struct netstat	net;
 	struct intfstat	intf;
+	struct memnuma	memnuma;
+	struct cpunuma	cpunuma;
 	struct dskstat  dsk;
-	struct nfsstat	nfs;
-	struct contstat	cfs;
+	struct nfsstat  nfs;
+	struct contstat cfs;
 	struct pressure	psi;
-	struct gpustat	gpu;
-	struct ifbstat	ifb;
+	struct gpustat 	gpu;
+	struct ifbstat 	ifb;
+	struct llcstat  llc;
 
 	struct wwwstat	www;
 };
@@ -340,6 +416,7 @@ struct	sstat {
 /*
 ** prototypes
 */
+void	setup_photosyst(void);
 char	photosyst (struct sstat *);
 void	deviatsyst(struct sstat *, struct sstat *, struct sstat *, double);
 void	totalsyst (char,           struct sstat *, struct sstat *);
