@@ -8,7 +8,7 @@
 ** figures.
 **
 ** Copyright (C) 2000-2010 Gerlof Langeveld
-** Copyright (C) 2015-2020 Red Hat.
+** Copyright (C) 2015-2021 Red Hat.
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -29,7 +29,9 @@
 #endif
 #include <pwd.h>
 #include <grp.h>
+#include <ctype.h>
 #include <stdarg.h>
+#include <locale.h>
 
 #include "atop.h"
 #include "photoproc.h"
@@ -38,13 +40,14 @@
 #include "showlinux.h"
 
 static struct pselection procsel = {"", {USERSTUB, }, {0,},
-                                    "", 0, { 0, },  "", 0, { 0, } };
+                                    "", 0, { 0, },  "", 0, { 0, }, "", "" };
 static struct sselection syssel;
 
 static void	showhelp(int);
 static int	paused;     	/* boolean: currently in pause-mode     */
 static int	fixedhead;	/* boolean: fixate header-lines         */
 static int	sysnosort;	/* boolean: suppress sort of resources  */
+static int	threadsort;	/* boolean: sort threads per process    */
 static int	avgval;		/* boolean: average values i.s.o. total */
 static int	suppressexit;	/* boolean: suppress exited processes   */
 
@@ -60,6 +63,8 @@ static int	maxintlines = 999;  /* maximum interface lines          */
 static int	maxifblines = 999;  /* maximum infinibnd lines          */
 static int	maxnfslines = 999;  /* maximum nfs mount lines          */
 static int	maxcontlines = 999; /* maximum container lines          */
+static int	maxnumalines = 999; /* maximum numa      lines          */
+static int	maxllclines = 999;  /* maximum llc       lines          */
 
 static short	colorinfo   = COLOR_GREEN;
 static short	coloralmost = COLOR_CYAN;
@@ -111,8 +116,8 @@ generic_samp(double sampletime, double nsecs,
 	char		*lastsortp, curorder, autoorder;
 	char		buf[34];
 	struct timeval	timeval;
-	struct syscap	syscap;
 	struct passwd 	*pwd;
+	struct syscap	syscap;
 
 	/*
 	** curlist points to the active list of tstat-pointers that
@@ -220,6 +225,10 @@ generic_samp(double sampletime, double nsecs,
 			qsort(sstat->intf.intf, sstat->intf.nrintf,
 		  	       sizeof sstat->intf.intf[0], intfcompar);
 
+		if (sstat->ifb.nrports > 1 && maxifblines > 0)
+			qsort(sstat->ifb.ifb, sstat->ifb.nrports,
+		  	       sizeof sstat->ifb.ifb[0], ifbcompar);
+
 		if (sstat->nfs.nfsmounts.nrmounts > 1 && maxnfslines > 0)
 			qsort(sstat->nfs.nfsmounts.nfsmnt,
 		              sstat->nfs.nfsmounts.nrmounts,
@@ -229,6 +238,18 @@ generic_samp(double sampletime, double nsecs,
 		if (sstat->cfs.nrcontainer > 1 && maxcontlines > 0)
 			qsort(sstat->cfs.cont, sstat->cfs.nrcontainer,
 		  	       sizeof sstat->cfs.cont[0], contcompar);
+
+		if (sstat->memnuma.nrnuma > 1 && maxnumalines > 0)
+			qsort(sstat->memnuma.numa, sstat->memnuma.nrnuma,
+			       sizeof sstat->memnuma.numa[0], memnumacompar);
+
+		if (sstat->cpunuma.nrnuma > 1 && maxnumalines > 0)
+			qsort(sstat->cpunuma.numa, sstat->cpunuma.nrnuma,
+			       sizeof sstat->cpunuma.numa[0], cpunumacompar);
+
+		if (sstat->llc.nrllcs > 1 && maxllclines > 0)
+			qsort(sstat->llc.perllc, sstat->llc.nrllcs,
+				sizeof sstat->llc.perllc[0], llccompar);
 	}
 
 	/*
@@ -239,6 +260,7 @@ generic_samp(double sampletime, double nsecs,
 	while (1)
 	{
 		curline = 1;
+		genline[0] = '\0';
 
 	        /*
        	 	** prepare screen or file output for new sample
@@ -259,27 +281,30 @@ generic_samp(double sampletime, double nsecs,
 
                 int seclen	= val2elapstr(nsecs, buf, sizeof(buf)-1);
                 int lenavail 	= (screen ? COLS : linelen) -
-						49 - seclen - nodenamelen;
+						52 - seclen - nodenamelen;
                 int len1	= lenavail / 3;
                 int len2	= lenavail - len1 - len1; 
 
-		printg("ATOP - %s%*s%s  %s%*s%c%c%c%c%c%c%c%c%c%c%c%c%c%c%*s%s"
-		       " elapsed", 
+		printg("ATOP - %s%*s%s  %s%*s%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%"
+		       "*s%s elapsed", 
 			sysname.nodename, len1, "", 
 			format1, format2, len1, "",
 			threadview                    ? MTHREAD    : '-',
+			threadsort                    ? MTHRSORT   : '-',
 			fixedhead  		      ? MSYSFIXED  : '-',
 			sysnosort  		      ? MSYSNOSORT : '-',
 			deviatonly 		      ? '-'        : MALLPROC,
 			usecolors  		      ? '-'        : MCOLORS,
 			avgval     		      ? MAVGVAL    : '-',
 			calcpss     		      ? MCALCPSS   : '-',
+			getwchan     		      ? MGETWCHAN  : '-',
 			suppressexit 		      ? MSUPEXITS  : '-',
 			procsel.userid[0] != USERSTUB ? MSELUSER   : '-',
 			procsel.prognamesz	      ? MSELPROC   : '-',
 			procsel.container[0]	      ? MSELCONT   : '-',
 			procsel.pid[0] != 0	      ? MSELPID    : '-',
 			procsel.argnamesz	      ? MSELARG    : '-',
+			procsel.states[0]	      ? MSELSTATE  : '-',
 			syssel.lvmnamesz +
 			syssel.dsknamesz +
 			syssel.itfnamesz	      ? MSELSYS    : '-',
@@ -315,12 +340,11 @@ generic_samp(double sampletime, double nsecs,
 			autoorder = showorder;
 
 		curline = prisyst(sstat, curline, nsecs, avgval,
-					fixedhead, &syssel, &autoorder,
-					maxcpulines, maxgpulines,
-					maxdsklines, maxmddlines,
-		                        maxlvmlines, maxintlines,
-					maxifblines, maxnfslines,
-					maxcontlines);
+		                  fixedhead, &syssel, &autoorder,
+		                  maxcpulines, maxgpulines, maxdsklines,
+				  maxmddlines, maxlvmlines,
+		                  maxintlines, maxifblines, maxnfslines,
+		                  maxcontlines, maxnumalines, maxllclines);
 
 		/*
  		** if system-wide statistics do not fit,
@@ -336,13 +360,15 @@ generic_samp(double sampletime, double nsecs,
 			move(curline, 0);
 
 			limitedlines();
-
+			
 			curline = prisyst(sstat, curline, nsecs, avgval,
 					fixedhead,  &syssel, &autoorder,
-					maxcpulines, maxdsklines, maxmddlines,
-					maxmddlines, maxlvmlines,
-		                        maxintlines, maxifblines, maxnfslines,
-		                        maxcontlines);
+					maxcpulines, maxgpulines,
+					maxdsklines, maxmddlines,
+		                        maxlvmlines, maxintlines,
+					maxifblines, maxnfslines,
+		                        maxcontlines, maxnumalines,
+					maxllclines);
 
 			/*
  			** if system-wide statistics still do not fit,
@@ -394,29 +420,73 @@ generic_samp(double sampletime, double nsecs,
 		{
 			if (flag&RRBOOT)
 			{
+				char *initmsg = "*** System and Process Activity since Boot ***";
+				char *viewmsg;
+
+				if (rawreadflag)
+				{
+					viewmsg   = "Rawfile view";
+				}
+				else
+				{
+ 					uid_t ruid, euid, suid;
+
+					getresuid(&ruid, &euid, &suid);
+
+					if (suid == 0)
+					{
+						viewmsg   = "Unrestricted view (privileged)";
+					}
+					else
+					{
+						viewmsg   = "Restricted view (unprivileged)";
+					}
+				}
+
 				if (screen)
 				{
 					if (usecolors)
 						attron(COLOR_PAIR(COLORINFO));
 
 					attron(A_BLINK);
-
-					printg("%*s", (COLS-45)/2, " ");
-				}
-				else
-				{
-					printg("                   ");
 				}
 
-       				printg("*** system and process activity "
-				       "since boot ***");
+       				printg(initmsg);
 
 				if (screen)
 				{
 					if (usecolors)
 						attroff(COLOR_PAIR(COLORINFO));
 					attroff(A_BLINK);
+
+					printg("%*s", (int)(
+						COLS - strlen(initmsg) - strlen(viewmsg)),
+						" ");
 				}
+				else
+				{
+					printg("%*s", (int)(
+						80 - strlen(initmsg) - strlen(viewmsg)),
+						" ");
+				}
+
+				if (screen)
+				{
+					if (usecolors)
+						attron(COLOR_PAIR(COLORALMOST));
+
+					attron(A_BLINK);
+				}
+
+       				printg(viewmsg);
+
+				if (screen)
+				{
+					if (usecolors)
+						attroff(COLOR_PAIR(COLORALMOST));
+					attroff(A_BLINK);
+				}
+
 			}
 		}
 
@@ -577,6 +647,7 @@ generic_samp(double sampletime, double nsecs,
 			if ( procsel.userid[0] == USERSTUB &&
 			    !procsel.prognamesz            &&
 			    !procsel.container[0]          &&
+			    !procsel.states[0]             &&
 			    !procsel.argnamesz             &&
 			    !procsel.pid[0]                &&
 			    !suppressexit                    )
@@ -687,24 +758,30 @@ generic_samp(double sampletime, double nsecs,
 				else
 					j = ntotal;
 
+				/*
+ 				** zip process list with thread list
+				*/
 				if (zipagain)
 				{
 					struct tstat *tall = devtstat->taskall;
 					struct tstat *pcur;
+					long int     n;
 
 					for (i=j=0; i < ncurlist; i++)
 					{
 					    pcur = curlist[i];
 
-					    tsklist[j++] = pcur;
+					    tsklist[j++] = pcur; // take process
+
+					    n = j; // start index of threads
 
 					    for (t = pcur - tall + 1;
-					         t < devtstat->ntaskall &&
-						 pcur->gen.tgid		&&
-					         pcur->gen.tgid == 
-					            (tall+t)->gen.tgid;
+					         t < devtstat->ntaskall && pcur->gen.tgid;
 					         t++)
 					    {
+						if (pcur->gen.tgid != (tall+t)->gen.tgid)
+							continue;
+
 						if (deviatonly &&
 							showtype  != MPROCMEM &&
 						        showorder != MSORTMEM   )
@@ -717,6 +794,14 @@ generic_samp(double sampletime, double nsecs,
 						else
 							tsklist[j++] = tall+t;
 					    }
+
+				            if (threadsort && j-n > 0 &&
+							curorder != MSORTMEM)
+					    {
+						qsort(&tsklist[n], j-n,
+				                  sizeof(struct tstat *),
+				                  procsort[(int)curorder&0x1f]);
+					    }
 					}
 
 					zipagain = 0;
@@ -739,7 +824,8 @@ generic_samp(double sampletime, double nsecs,
 
 			priphead(firstproc/plistsz+1, (ncurlist-1)/plistsz+1,
 			       		&showtype, &curorder,
-					showorder == MSORTAUTO ? 1 : 0);
+					showorder == MSORTAUTO ? 1 : 0,
+					sstat->cpu.nrcpu);
 
 			if (screen)
 			{
@@ -798,7 +884,7 @@ generic_samp(double sampletime, double nsecs,
 				if (tsklist)   free(tsklist);
 				if (sellist)   free(sellist);
 
-				return '\0';	
+				return lastchar;	
 
 			   /*
 			   ** stop it
@@ -1085,6 +1171,21 @@ generic_samp(double sampletime, double nsecs,
 				firstproc = 0;
 				break;
 
+			   /*
+			   ** cgroup v2 info per process
+			   */
+			   case MPROCCGR:
+				if ( !(supportflags & CGROUPV2) )
+				{
+					statmsg = "No cgroup v2 figures "
+					          "available; request ignored!";
+					break;
+				}
+
+				showtype  = MPROCCGR;
+				firstproc = 0;
+				break;
+
 			   /*
 			   ** own defined output per process
 			   */
@@ -1415,15 +1516,7 @@ generic_samp(double sampletime, double nsecs,
 					break;
 
 				   case 12:	// container id
-					/*
-					 * Don't even ask ... we want to
-					 * scan the input buffer and are only
-					 * interested in a possible error ...
-					 * the (void)(...+1) babble is the
-					 * only way to silence new "smart" C
-					 * compilers!
-					 */
-					(void)(strtol(procsel.container, &p, 16)+1);
+					(void)strtol(procsel.container, &p, 16);
 
 					if (*p)
 					{
@@ -1508,6 +1601,81 @@ generic_samp(double sampletime, double nsecs,
 				firstproc = 0;
 				break;
 
+			    /*
+			    ** focus on specific process/thread state
+			    */
+			   case MSELSTATE:
+				alarm(0);       /* stop the clock */
+				echo();
+
+				move(statline, 0);
+				clrtoeol();
+
+				/* Linux fs/proc/array.c - task_state_array */
+				printw("Comma-separated process/thread states "
+				       "(R|S|D|I|T|t|X|Z|P): ");
+
+				memset(procsel.states, 0, sizeof procsel.states);
+
+				scanw("%15s\n", genline);
+
+				char *sp = strtok(genline, ",");
+
+				while (sp && *sp)
+				{
+					if (isspace(*sp))
+					{
+						sp++;
+						continue;
+					}
+
+					if (strlen(sp) > 1)
+					{
+						statmsg = "Invalid state!";
+						memset(procsel.states, 0,
+							sizeof procsel.states);
+						break;
+					}
+
+					int needed = 0;
+
+					switch (*sp)
+					{
+						case 'R': /* running */
+						case 'S': /* sleeping */
+						case 'D': /* disk sleep */
+						case 'I': /* idle */
+						case 'T': /* stopped */
+						case 't': /* tracing stop */
+						case 'X': /* dead */
+						case 'Z': /* zombie */
+						case 'P': /* parked */
+							if (!strchr(procsel.states, *sp))
+								needed = 1;
+							break;
+						default:
+							statmsg = "Invalid state!";
+							memset(procsel.states,
+								0, sizeof procsel.states);
+							beep();
+							break;
+					}
+
+					if (needed)
+					    procsel.states[strlen(procsel.states)] = *sp;
+
+					sp = strtok(NULL, ",");
+				}
+
+				noecho();
+
+				move(statline, 0);
+
+				if ((interval.tv_sec || interval.tv_usec) && !paused && !rawreadflag)
+					setalarm2(3, 0);  /* set short timer */
+
+				firstproc = 0;
+				break;
 
 			   /*
 			   ** focus on specific command line arguments
@@ -1741,7 +1909,7 @@ generic_samp(double sampletime, double nsecs,
 
 			   /*
 			   ** per-thread view wanted with sorting on
-			   ** process level or thread level
+			   ** process level
 			   */
 			   case MTHREAD:
 				if (threadview)
@@ -1758,6 +1926,24 @@ generic_samp(double sampletime, double nsecs,
 				}
 				break;
 
+			   /*
+			   ** sorting on thread level as well (threadview)
+			   */
+			   case MTHRSORT:
+				if (threadsort)
+				{
+					threadsort = 0;
+					statmsg    = "Thread sorting disabled for thread view";
+					firstproc  = 0;
+				}
+				else
+				{
+					threadsort = 1;
+					statmsg    = "Thread sorting enabled for thread view";
+					firstproc  = 0;
+				}
+				break;
+
 			   /*
 			   ** per-process PSS calculation wanted 
 			   */
@@ -1774,6 +1960,22 @@ generic_samp(double sampletime, double nsecs,
 				}
 				break;
 
+			   /*
+			   ** per-thread WCHAN definition 
+			   */
+			   case MGETWCHAN:
+				if (getwchan)
+				{
+					getwchan   = 0;
+					statmsg    = "WCHAN gathering disabled";
+				}
+				else
+				{
+					getwchan   = 1;
+					statmsg    = "WCHAN gathering enabled";
+				}
+				break;
+
 			   /*
 			   ** suppression of exited processes in output
 			   */
@@ -1879,6 +2081,16 @@ generic_samp(double sampletime, double nsecs,
 				            "statistics (now %d): ",
 					    maxcontlines, statline);
 
+				maxnumalines =
+				  getnumval("Maximum lines for numa "
+				            "statistics (now %d): ",
+					    maxnumalines, statline);
+
+				maxllclines =
+				  getnumval("Maximum lines for LLC "
+				            "statistics (now %d): ",
+					    maxllclines, statline);
+
 				if ((interval.tv_sec || interval.tv_usec) && !paused && !rawreadflag)
 					setalarm2(3, 0);  /* set short timer */
 
@@ -2203,6 +2415,7 @@ accumulate(struct tstat *curproc, struct tstat *curstat)
 		curstat->mem.vdata  += curproc->mem.vdata;
 		curstat->mem.vstack += curproc->mem.vstack;
 		curstat->mem.vswap  += curproc->mem.vswap;
+		curstat->mem.vlock  += curproc->mem.vlock;
 		curstat->mem.rgrow  += curproc->mem.rgrow;
 		curstat->mem.vgrow  += curproc->mem.vgrow;
 
@@ -2327,6 +2540,15 @@ procsuppress(struct tstat *curstat, struct pselection *sel)
 		}
 	}
 
+	/*
+	** check if only processes in specific states should be shown 
+	*/
+	if (sel->states[0])
+	{
+		if (strchr(sel->states, curstat->gen.state) == NULL)
+			return 1;
+	}
+
 	return 0;
 }
 
@@ -2360,6 +2582,12 @@ limitedlines(void)
 
 	if (maxcontlines == 999)	// default?
 		maxcontlines = 1;
+
+	if (maxnumalines == 999)	// default?
+		maxnumalines = 0;
+
+	if (maxllclines  == 999)	// default?
+		maxllclines = 0;
 }
 
 /*
@@ -2504,7 +2732,7 @@ generic_init(void)
 			{
 				fprintf(stderr, "BCC PMDA not active or "
 					        "'netproc' module not enabled; "
-					        "request ignored!");
+					        "request ignored!\n");
 				sleep(3);
 				break;
 			}
@@ -2521,6 +2749,18 @@ generic_init(void)
 			showtype  = MPROCARG;
 			break;
 
+		   case MPROCCGR:
+			if ( !(supportflags & CGROUPV2) )
+			{
+				fprintf(stderr, "No cgroup v2 details "
+				          "available; request ignored!\n");
+				sleep(3);
+				break;
+			}
+
+			showtype  = MPROCCGR;
+			break;
+
 		   case MPROCOWN:
 			showtype  = MPROCOWN;
 			break;
@@ -2565,6 +2805,13 @@ generic_init(void)
 				threadview = 1;
 			break;
 
+		   case MTHRSORT:
+			if (threadsort)
+				threadsort = 0;
+			else
+				threadsort = 1;
+			break;
+
 		   case MCALCPSS:
 			if (calcpss)
 				calcpss = 0;
@@ -2572,6 +2819,13 @@ generic_init(void)
 				calcpss = 1;
 			break;
 
+		   case MGETWCHAN:
+			if (getwchan)
+				getwchan = 0;
+			else
+				getwchan = 1;
+			break;
+
 		   case MSUPEXITS:
 			if (suppressexit)
 				suppressexit = 0;
@@ -2604,7 +2858,7 @@ generic_init(void)
        	** check if STDOUT is related to a tty or
        	** something else (file, pipe)
        	*/
-       	if ( isatty(1) )
+       	if ( isatty(fileno(stdout)) )
                	screen = 1;
        	else
              	screen = 0;
@@ -2615,9 +2869,22 @@ generic_init(void)
        	*/
        	if (screen)
 	{
+		/*
+		** if stdin is not connected to a tty (might be redirected
+		** to pipe or file), close it and duplicate stdout (tty)
+		** to stdin
+		*/
+       		if ( !isatty(fileno(stdin)) )
+		{
+			(void) dup2(fileno(stdout), fileno(stdin));
+		}
+
 		/*
 		** initialize screen-handling via curses
 		*/
+		setlocale(LC_ALL, "");
+		setlocale(LC_NUMERIC, "C");
+
 		initscr();
 		cbreak();
 		noecho();
@@ -2666,11 +2933,13 @@ static struct helptext {
 	{"\t'%c'  - memory details\n",				MPROCMEM},
 	{"\t'%c'  - disk details\n",				MPROCDSK},
 	{"\t'%c'  - network details\n",				MPROCNET},
-	{"\t'%c'  - GPU details\n",				MPROCGPU},
+	{"\t'%c'  - cgroups v2 details\n",			MPROCCGR},
 	{"\t'%c'  - scheduling and thread-group info\n",	MPROCSCH},
+	{"\t'%c'  - GPU details\n",				MPROCGPU},
 	{"\t'%c'  - various info (ppid, user/group, date/time, status, "
 	 "exitcode)\n",	MPROCVAR},
 	{"\t'%c'  - full command line per process\n",		MPROCARG},
+	{"\t'%c'  - cgroup v2 info per process\n",		MPROCCGR},
 	{"\t'%c'  - use own output line definition\n",		MPROCOWN},
 	{"\n",							' '},
 	{"Sort list of processes in order of:\n",		' '},
@@ -2692,10 +2961,11 @@ static struct helptext {
 	                              "(regular expression)\n", MSELUSER},
 	{"\t'%c'  - focus on specific program name        "
 	                              "(regular expression)\n", MSELPROC},
-	{"\t'%c'  - focus on specific contained id (CID)\n",    MSELCONT},
+	{"\t'%c'  - focus on specific container id (CID)\n",    MSELCONT},
 	{"\t'%c'  - focus on specific command line string "
 	                              "(regular expression)\n", MSELARG},
 	{"\t'%c'  - focus on specific process id (PID)\n",      MSELPID},
+	{"\t'%c'  - focus on specific process/thread state(s)\n", MSELSTATE},
 	{"\n",							' '},
 	{"System resource selections (keys shown in header line):\n",' '},
 	{"\t'%c'  - focus on specific system resources    "
@@ -2713,8 +2983,10 @@ static struct helptext {
 									' '},
 	{"\n",							' '},
 	{"Presentation (keys shown in header line):\n",  	' '},
-	{"\t'%c'  - show individual threads                        (toggle)\n",
+	{"\t'%c'  - show threads within process (thread view)      (toggle)\n",
 		 						MTHREAD},
+	{"\t'%c'  - sort threads (when combined with thread view)  (toggle)\n",
+		 						MTHRSORT},
 	{"\t'%c'  - show all processes (default: active processes) (toggle)\n",
 								MALLPROC},
 	{"\t'%c'  - show fixed number of header lines              (toggle)\n",
@@ -2729,6 +3001,8 @@ static struct helptext {
 								MAVGVAL},
 	{"\t'%c'  - calculate proportional set size (PSIZE)        (toggle)\n",
 								MCALCPSS},
+	{"\t'%c'  - determine WCHAN per thread                     (toggle)\n",
+								MGETWCHAN},
 	{"\n",							' '},
 	{"Raw file viewing:\n",					' '},
 	{"\t'%c'  - show next     sample in raw file\n",	MSAMPNEXT},
@@ -2859,7 +3133,8 @@ generic_usage(void)
 			MSUPEXITS);
 	printf("\t  -%c  show limited number of lines for certain resources\n",
 			MSYSLIMIT);
-	printf("\t  -%c  show individual threads\n", MTHREAD);
+	printf("\t  -%c  show threads within process\n", MTHREAD);
+	printf("\t  -%c  sort threads (when combined with '%c')\n", MTHRSORT, MTHREAD);
 	printf("\t  -%c  show average-per-second i.s.o. total values\n\n",
 			MAVGVAL);
 	printf("\t  -%c  no colors in case of high occupation\n",
@@ -2878,6 +3153,8 @@ generic_usage(void)
 	                 "date/time)\n", MPROCVAR);
 	printf("\t  -%c  show command line per process\n",
 			MPROCARG);
+	printf("\t  -%c  show cgroup v2 info per process\n",
+			MPROCCGR);
 	printf("\t  -%c  show own defined process-info\n",
 			MPROCOWN);
 	printf("\t  -%c  show cumulated process-info per user\n",
@@ -3042,6 +3319,17 @@ do_maxcont(char *name, char *val)
 	maxcontlines = get_posval(name, val);
 }
 
+void
+do_maxnuma(char *name, char *val)
+{
+	maxnumalines = get_posval(name, val);
+}
+
+void
+do_maxllc(char *name, char *val)
+{
+	maxllclines = get_posval(name, val);
+}
 
 struct colmap {
 	char 	*colname;
@@ -3176,6 +3464,10 @@ do_flags(char *name, char *val)
 			showtype  = MPROCARG;
 			break;
 
+		   case MPROCCGR:
+			showtype  = MPROCCGR;
+			break;
+
 		   case MPROCOWN:
 			showtype  = MPROCOWN;
 			break;
@@ -3212,6 +3504,10 @@ do_flags(char *name, char *val)
 			threadview = 1;
 			break;
 
+		   case MTHRSORT:
+			threadsort = 1;
+			break;
+
 		   case MCOLORS:
 			usecolors = 0;
 			break;
@@ -3220,6 +3516,10 @@ do_flags(char *name, char *val)
 			calcpss = 1;
 			break;
 
+		   case MGETWCHAN:
+			getwchan = 1;
+			break;
+
 		   case MSUPEXITS:
 			suppressexit = 1;
 			break;
