@@ -108,7 +108,10 @@ class MetricRepository:
         if instance is not None:
             return dict(map(lambda x: (x[0].inst, x[2]), self.group[metric].netValues))
         else:
-            return self.group[metric].netValues[0][2]
+            if self.group[metric].netValues == []:
+                return None
+            else:
+                return self.group[metric].netValues[0][2]
 
     def __fetch_previous_values(self,metric,instance):
         if instance is not None:
@@ -140,9 +143,11 @@ class CoreCpuUtil:
         c_time = self.metric_repository.current_value(metric, self.instance)
         if p_time is not None and c_time is not None:
             value = (100*(c_time - p_time))/(1000*self.delta_time)
-            if self.instance is None:
+            if self.instance is None and self.total_cpus() is not None:
                 return float("%.2f"%(value/self.total_cpus()))
             else:
+                if self.total_cpus() is None:
+                    return None
                 return float("%.2f"%(value))
 
         else:
@@ -154,9 +159,11 @@ class CoreCpuUtil:
         c_time = self.metric_repository.current_value(metric, self.instance)
         if p_time is not None and c_time is not None:
             value = (100*(c_time - p_time))/(1000*self.delta_time)
-            if self.instance is None:
+            if self.instance is None and self.total_cpus() is not None:
                 return float("%.2f"%(value/self.total_cpus()))
             else:
+                if self.total_cpus() is None:
+                    return None
                 return float("%.2f"%(value))
         else:
             return None
@@ -167,9 +174,11 @@ class CoreCpuUtil:
         c_time = self.metric_repository.current_value(metric, self.instance)
         if p_time is not None and c_time is not None:
             value = (100*(c_time - p_time))/(1000*self.delta_time)
-            if self.instance is None:
+            if self.instance is None and self.total_cpus() is not None:
                 return float("%.2f"%(value/self.total_cpus()))
             else:
+                if self.total_cpus() is None:
+                    return None
                 return float("%.2f"%(value))
         else:
             return None
@@ -180,9 +189,11 @@ class CoreCpuUtil:
         c_time = self.metric_repository.current_value(metric, self.instance)
         if p_time is not None and c_time is not None:
             value = (100*(c_time - p_time))/(1000*self.delta_time)
-            if self.instance is None:
+            if self.instance is None and self.total_cpus() is not None:
                 return float("%.2f"%(value/self.total_cpus()))
             else:
+                if self.total_cpus() is None:
+                    return None
                 return float("%.2f"%(value))
         else:
             return None
@@ -193,9 +204,11 @@ class CoreCpuUtil:
         c_time = self.metric_repository.current_value(metric, self.instance)
         if p_time is not None and c_time is not None:
             value = (100*(c_time - p_time))/(1000*self.delta_time)
-            if self.instance is None:
+            if self.instance is None and self.total_cpus() is not None:
                 return float("%.2f"%(value/self.total_cpus()))
             else:
+                if self.total_cpus() is None:
+                    return None
                 return float("%.2f"%(value))
         else:
             return None
@@ -206,9 +219,11 @@ class CoreCpuUtil:
         c_time = self.metric_repository.current_value(metric, self.instance)
         if p_time is not None and c_time is not None:
             value = (100*(c_time - p_time))/(1000*self.delta_time)
-            if self.instance is None:
+            if self.instance is None and self.total_cpus() is not None:
                 return float("%.2f"%(value/self.total_cpus()))
             else:
+                if self.total_cpus() is None:
+                    return None
                 return float("%.2f"%(value))
         else:
             return None
@@ -219,9 +234,11 @@ class CoreCpuUtil:
         c_time = self.metric_repository.current_value(metric, self.instance)
         if p_time is not None and c_time is not None:
             value = (100*(c_time - p_time))/(1000*self.delta_time)
-            if self.instance is None:
+            if self.instance is None and self.total_cpus() is not None:
                 return float("%.2f"%(value/self.total_cpus()))
             else:
+                if self.total_cpus() is None:
+                    return None
                 return float("%.2f"%(value))
         else:
             return None
@@ -232,9 +249,11 @@ class CoreCpuUtil:
         c_time = self.metric_repository.current_value(metric, self.instance)
         if p_time is not None and c_time is not None:
             value = (100*(c_time - p_time))/(1000*self.delta_time)
-            if self.instance is None:
+            if self.instance is None and self.total_cpus() is not None:
                 return float("%.2f"%(value/self.total_cpus()))
             else:
+                if self.total_cpus() is None:
+                    return None
                 return float("%.2f"%(value))
         else:
             return None
@@ -245,9 +264,11 @@ class CoreCpuUtil:
         c_time = self.metric_repository.current_value(metric, self.instance)
         if p_time is not None and c_time is not None:
             value = (100*(c_time - p_time))/(1000*self.delta_time)
-            if self.instance is None:
+            if self.instance is None and self.total_cpus() is not None:
                 return float("%.2f"%(value/self.total_cpus()))
             else:
+                if self.total_cpus() is None:
+                    return None
                 return float("%.2f"%(value))
         else:
             return None
@@ -258,9 +279,11 @@ class CoreCpuUtil:
         c_time = self.metric_repository.current_value(metric, self.instance)
         if p_time is not None and c_time is not None:
             value = (100*(c_time - p_time))/(1000*self.delta_time)
-            if self.instance is None:
+            if self.instance is None and self.total_cpus() is not None:
                 return float("%.2f"%(value/self.total_cpus()))
             else:
+                if self.total_cpus() is None:
+                    return None
                 return float("%.2f"%(value))
         else:
             return None
@@ -413,7 +436,7 @@ class CpuUtilReporter:
         if self.header_print:
             self.printer("\n%-10s\t%-3s\t%-5s\t%-6s\t%-5s\t%-8s\t%-5s\t%-6s\t%-7s\t%-7s\t%-6s\t%-6s"%("Timestamp","CPU","%usr","%nice","%sys","%iowait","%irq","%soft","%steal","%guest","%nice","%idle"))
             self.header_print = False
-        if self.mpstat_options.cpu_list == "ALL" or self.mpstat_options.cpu_list == "ON":
+        if self.mpstat_options.cpu_list in ("ALL", "ON"):
             self.header_print = True
         if not isinstance(self.mpstat_options.cpu_list, list):
             cpu_util = cpu_utils.get_totalcpu_util()
@@ -555,16 +578,21 @@ class DisplayOptions:
         return self.mpstatoptions.no_options or (self.mpstatoptions.cpu_filter and not self.mpstatoptions.interrupts_filter)
 
     def display_total_cpu_usage(self):
-        return self.mpstatoptions.interrupts_filter and (self.mpstatoptions.interrupt_type == "SUM" or self.mpstatoptions.interrupt_type == "ALL")
+        return self.mpstatoptions.interrupts_filter and self.mpstatoptions.interrupt_type in ("SUM", "ALL")
 
     def display_hard_interrupt_usage(self):
-        return self.mpstatoptions.interrupts_filter and (self.mpstatoptions.interrupt_type == "CPU" or self.mpstatoptions.interrupt_type == "ALL")
+        return self.mpstatoptions.interrupts_filter and self.mpstatoptions.interrupt_type in ("CPU", "ALL")
 
     def display_soft_interrupt_usage(self):
-        return self.mpstatoptions.interrupts_filter and (self.mpstatoptions.interrupt_type == "SCPU" or self.mpstatoptions.interrupt_type == "ALL")
+        return self.mpstatoptions.interrupts_filter and self.mpstatoptions.interrupt_type in ("SCPU", "ALL")
 
 class MpstatReport(pmcc.MetricGroupPrinter):
     Machine_info_count = 0
+    ncpu = 1
+    machine = ''
+    release = ''
+    sysname = ''
+    nodename = ''
 
     def __init__(self, cpu_util_reporter, total_interrupt_usage_reporter, soft_interrupt_usage_reporter, hard_interrupt_usage_reporter):
         self.cpu_util_reporter = cpu_util_reporter
@@ -578,6 +606,7 @@ class MpstatReport(pmcc.MetricGroupPrinter):
         return s + u / 1000000.0
 
     def print_machine_info(self,group, context):
+        self.get_summary_metrics(group)
         timestamp = context.pmLocaltime(group.timestamp.tv_sec)
         # Please check strftime(3) for different formatting options.
         # Also check TZ and LC_TIME environment variables for more information
@@ -585,21 +614,45 @@ class MpstatReport(pmcc.MetricGroupPrinter):
         time_string = time.strftime("%x", timestamp.struct_time())
 
         header_string = ''
-        header_string += group['kernel.uname.sysname'].netValues[0][2] + '  '
-        header_string += group['kernel.uname.release'].netValues[0][2] + '  '
-        header_string += '(' + group['kernel.uname.nodename'].netValues[0][2] + ')  '
+        header_string += self.sysname + '  '
+        header_string += self.release + '  '
+        header_string += '(' + self.nodename + ')  '
         header_string += time_string + '  '
-        header_string += group['kernel.uname.machine'].netValues[0][2] + '  '
-        no_cpu = self.get_ncpu(group)
-        print("%s  (%s CPU)" % (header_string,no_cpu))
+        header_string += self.machine + '    '
+        header_string += '(' + str(self.ncpu) + ' CPU)'
+        print(header_string)
 
-    def get_ncpu(self,group):
-        return group['hinv.ncpu'].netValues[0][2]
+    def get_summary_metrics(self,group):
+        # extract metrics safely which may have 'logged-once' semantics;
+        # we checked earlier so know the metrics exist once at least, so
+        # fallback to previous observed value if not in latest sample -
+        # always try though in case a later sample finds updated values.
+        try:
+            self.ncpu = group['hinv.ncpu'].netValues[0][2]
+        except IndexError:
+            pass
+        try:
+            self.sysname = group['kernel.uname.sysname'].netValues[0][2]
+        except IndexError:
+            pass
+        try:
+            self.machine = group['kernel.uname.machine'].netValues[0][2]
+        except IndexError:
+            pass
+        try:
+            self.release = group['kernel.uname.release'].netValues[0][2]
+        except IndexError:
+            pass
+        try:
+            self.nodename = group['kernel.uname.nodename'].netValues[0][2]
+        except IndexError:
+            pass
 
     def report(self,manager):
         group = manager['mpstat']
         if group['kernel.all.cpu.user'].netPrevValues is None:
             # need two fetches to report rate converted counter metrics
+            self.get_summary_metrics(group)
             return
 
         if self.Machine_info_count == 0:
