@@ -1,6 +1,6 @@
 #!/usr/bin/env pmpython
 #
-# Copyright (C) 2015-2020 Marko Myllynen <myllynen@redhat.com>
+# Copyright (C) 2015-2021 Marko Myllynen <myllynen@redhat.com>
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the
@@ -65,7 +65,17 @@ class PCP2XLSX(object):
                      'type_prefer', 'precision_force',
                      'names_change',
                      'speclocal', 'instances', 'ignore_incompat', 'ignore_unknown',
-                     'omit_flat')
+                     'omit_flat', 'include_labels')
+
+        # Ignored for pmrep(1) compatibility
+        self.keys_ignore = (
+                     'timestamp', 'unitinfo', 'colxrow', 'separate_header', 'fixed_header',
+                     'delay', 'width', 'delimiter', 'extcsv', 'width_force',
+                     'extheader', 'repeat_header', 'interpol',
+                     'dynamic_header', 'overall_rank', 'overall_rank_alt', 'sort_metric',
+                     'limit_filter', 'limit_filter_force',
+                     'live_filter', 'rank', 'invert_filter', 'predicate',
+                     'instinfo', 'include_texts')
 
         # The order of preference for options (as present):
         # 1 - command line options
@@ -90,6 +100,7 @@ class PCP2XLSX(object):
         self.names_change = 0 # ignore
         self.instances = []
         self.omit_flat = 0
+        self.include_labels = 0
         self.precision = 3 # .3f
         self.precision_force = None
         self.timefmt = TIMEFMT
@@ -123,7 +134,7 @@ class PCP2XLSX(object):
         self.pmfg_ts = None
 
         # Read configuration and prepare to connect
-        self.config = self.pmconfig.set_config_file(DEFAULT_CONFIG)
+        self.config = self.pmconfig.set_config_path(DEFAULT_CONFIG)
         self.pmconfig.read_options()
         self.pmconfig.read_cmd_line()
         self.pmconfig.prepare_metrics()
@@ -134,7 +145,7 @@ class PCP2XLSX(object):
         opts = pmapi.pmOptions()
         opts.pmSetOptionCallback(self.option)
         opts.pmSetOverrideCallback(self.option_override)
-        opts.pmSetShortOptions("a:h:LK:c:Ce:D:V?HGA:S:T:O:s:t:rRIi:4:5vP:0:q:b:y:Q:B:Y:F:f:Z:z")
+        opts.pmSetShortOptions("a:h:LK:c:Ce:D:V?HGA:S:T:O:s:t:rRIi:4:5vmP:0:q:b:y:Q:B:Y:F:f:Z:z")
         opts.pmSetShortUsage("[option...] metricspec [...]")
 
         opts.pmSetLongOptionHeader("General options")
@@ -171,6 +182,7 @@ class PCP2XLSX(object):
         opts.pmSetLongOption("names-change", 1, "4", "ACTION", "ignore/abort on PMNS change (default: ignore)")
         opts.pmSetLongOption("instances", 1, "i", "STR", "instances to report (default: all current)")
         opts.pmSetLongOption("omit-flat", 0, "v", "", "omit single-valued metrics")
+        opts.pmSetLongOption("include-labels", 0, "m", "", "include metric label info")
         opts.pmSetLongOption("precision", 1, "P", "N", "prefer N digits after decimal separator (default: 3)")
         opts.pmSetLongOption("precision-force", 1, "0", "N", "force N digits after decimal separator")
         opts.pmSetLongOption("timestamp-format", 1, "f", "STR", "xlsx timestamp format")
@@ -236,6 +248,8 @@ class PCP2XLSX(object):
             self.instances = self.instances + self.pmconfig.parse_instances(optarg)
         elif opt == 'v':
             self.omit_flat = 1
+        elif opt == 'm':
+            self.include_labels = 1
         elif opt == 'P':
             self.precision = optarg
         elif opt == '0':
@@ -447,6 +461,16 @@ class PCP2XLSX(object):
                         self.ws.column_dimensions[chr(col)].width = l + PADDING
             self.row += 1
 
+            # Labels, static
+            if self.include_labels:
+                col = COLUMNA
+                for i, metric in enumerate(self.metrics):
+                    for j in range(len(self.pmconfig.insts[i][0])):
+                        labels = self.pmconfig.get_labels_str(metric, self.pmconfig.insts[i][0][j], False)
+                        col += 1
+                        write_cell(col, self.row, labels, True)
+                self.row += 1
+
             # Units, static
             col = COLUMNA
             for i, metric in enumerate(self.metrics):
