@@ -3,7 +3,7 @@
 #
 # Copyright (c) 1995-2001,2003 Silicon Graphics, Inc.  All Rights Reserved.
 # Portions Copyright (c) 2008 Aconex.  All Rights Reserved.
-# Portions Copyright (c) 2013-2016,2018 Red Hat.
+# Portions Copyright (c) 2013-2016,2018,2020 Red Hat.
 # 
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the
@@ -60,10 +60,10 @@ _setup_localhost()
 	# on our path
 	#
 	ping --help >$tmp/hlp 2>&1
-	if grep '.-c count' $tmp/hlp >/dev/null 2>&1
+	if grep '.-c <*count>*' $tmp/hlp >/dev/null 2>&1 || grep '.-c <count>' $tmp/hlp >/dev/null 2>&1
 	then
 	    __opt='-c 1 localhost'
-	elif grep '.-n count' $tmp/hlp >/dev/null 2>&1
+	elif grep '.-n <*count>*' $tmp/hlp >/dev/null 2>&1
 	then
 	    __opt='-n 1 localhost'
 	elif grep 'host .*packetsize .*count' $tmp/hlp >/dev/null 2>&1
@@ -266,36 +266,33 @@ END					{ exit status }'
     done
 }
 
+__filter_journalctl_pmcd()
+{
+    grep " pmcd\\[" | sed -e 's/\.\.*done$//'
+}
+
+__journalctl_pmcd()
+{
+    if `which journalctl >/dev/null 2>&1`
+    then
+	journalctl -n 10 -u pmcd 2>/dev/null | __filter_journalctl_pmcd
+    elif `which systemd-journalctl >/dev/null 2>&1`
+    then
+	systemd-journalctl -q -n 100 2>/dev/null | __filter_journalctl_pmcd
+    fi
+}
+
 __pmda_restart_pmcd()
 {
     if [ -n "$PCP_SYSTEMDUNIT_DIR" -a -f $PCP_SYSTEMDUNIT_DIR/pmcd.service -a "$PCPQA_SYSTEMD" != no ]
     then
 	# smells like systemctl is the go ...
 	#
-	if `which journalctl >/dev/null 2>&1`
-	then
-	    journalctl -n 10 -u pmcd \
-	    | grep " pmcd\\[" \
-	    | sed -e 's/\.\.*done$//' >$tmp.journal.pre
-	elif `which systemd-journalctl >/dev/null 2>&1`
-	then
-	    systemd-journalctl -q -n 100 \
-	    | grep " pmcd\\[" \
-	    | sed -e 's/\.\.*done$//' >$tmp.journal.pre
-	fi
+	__journalctl_pmcd > $tmp.journal.pre
 	systemctl restart pmcd.service
         sleep 2
-	if `which journalctl >/dev/null 2>&1`
-	then
-	    journalctl -n 10 -u pmcd \
-	    | grep " pmcd\\["  \
-	    | sed -e 's/\.\.*done$//' >$tmp.journal.post
-	elif `which systemd-journalctl >/dev/null 2>&1`
-	then
-	    systemd-journalctl -q -n 100 \
-	    | grep " pmcd\\[" \
-	    | sed -e 's/\.\.*done$//' >$tmp.journal.post
-	fi
+	__journalctl_pmcd > $tmp.journal.post
+
 	# diff the pre and post journal lines to find those most recently
 	# added:
 	#	- lines beginning with a digit are ed(1) commands, ignore
@@ -411,7 +408,7 @@ __check_root()
     else
 	if [ "X$ROOT" != X -a "X$ROOT" != X/ ]
 	then
-	    echo "Install: \$ROOT was set to \"$ROOT\""
+	    echo "$prog: \$ROOT was set to \"$ROOT\""
 	    echo "          Use -R rootdir to install somewhere other than /"
 	    status=1
 	    exit
@@ -433,7 +430,7 @@ __check_domain()
     then
 	__infile=domain.h.python
     else
-	echo "Install: cannot find domain.h (or similar) to determine the Performance Metrics Domain"
+	echo "$prog: cannot find domain.h (or similar) to determine the Performance Metrics Domain"
 	status=1
 	exit
     fi
@@ -460,7 +457,7 @@ __check_domain()
     fi
     if [ "X$domain" = X ]
     then
-	echo "Install: cannot determine the Performance Metrics Domain from $__infile"
+	echo "$prog: cannot determine the Performance Metrics Domain from $__infile"
 	status=1
 	exit
     fi
@@ -798,8 +795,22 @@ _setup()
 	    perl -e 'use PCP::PMDA' >$tmp/out 2>&1
 	    if test $? -eq 0
 	    then
-		eval PCP_PERL_DOMAIN=1 perl "$perl_name" > "$perl_dom"
-		eval PCP_PERL_PMNS=1 perl "$perl_name" > "$perl_pmns"
+		if eval PCP_PERL_DOMAIN=1 perl "$perl_name" >"$perl_dom"
+		then
+		    :
+		else
+		    echo "Arrgh! failed to create $perl_dom from $perl_name"
+		    status=1
+		    exit
+		fi
+		if eval PCP_PERL_PMNS=1 perl "$perl_name" >"$perl_pmns"
+		then
+		    :
+		else
+		    echo "Arrgh! failed to create $perl_pmns from $perl_name"
+		    status=1
+		    exit
+		fi
 	    elif $dso_opt || $daemon_opt
 	    then
 		:	# we have an alternative, so continue on
@@ -837,15 +848,35 @@ _setup()
 	    python_pmns="${pmda_dir}/pmns.python"
 	    python_dom="${pmda_dir}/domain.h.python"
 	    $python -c 'from pcp import pmda' 2>/dev/null
-	    if test $? -eq 0
+	    __module=$?
+	    $python -c "import ast; ast.parse(open('$python_name').read())" 2>$tmp/err >/dev/null
+	    __syntax=$?
+	    if test $__module -eq 0 -a $__syntax -eq 0
 	    then
-		eval PCP_PYTHON_DOMAIN=1 $python "$python_name" > "$python_dom"
-		eval PCP_PYTHON_PMNS=1 $python "$python_name" > "$python_pmns"
+		if eval PCP_PYTHON_DOMAIN=1 $python "$python_name" >"$python_dom"
+		then
+		    :
+		else
+		    echo "Arrgh! failed to create $python_dom from $python_name"
+		    status=1
+		    exit
+		fi
+		if eval PCP_PYTHON_PMNS=1 $python "$python_name" >"$python_pmns"
+		then
+		    :
+		else
+		    echo "Arrgh! failed to create $python_pmns from $python_name"
+		    status=1
+		    exit
+		fi
 	    elif $dso_opt || $daemon_opt
 	    then
 		:	# we have an alternative, so continue on
 	    else
-		echo 'Python pcp.pmda module is not installed, install it and try again'
+		test $__module -ne 0 && \
+		    echo 'Python pcp.pmda module is not installed, install it and try again'
+		test $__syntax -ne 0 && \
+		    echo 'Python script '$python_name' has syntax errors.' && cat $tmp/err
 		status=1
 		exit
 	    fi
@@ -1037,6 +1068,14 @@ _install()
     then
 	__choose_ipc $pmda_dir
 	__args="-d $domain $__args"
+	# Optionally use $PCP_DEBUG from the environment to set -D options
+	# in pmcd.conf for command line args
+	#
+	if [ -n "$PCP_DEBUG" ]
+	then
+	    __args="-D$PCP_DEBUG $__args"
+	fi
+
     elif [ "$pmda_type" = perl ]
     then
 	type="pipe	$ipc_prot		perl $perl_name"
@@ -1082,17 +1121,8 @@ _install()
     #
     if [ -f $help_source ]
     then
-	case $pmda_interface
-	in
-	    1)
-		    help_version=1
-		    ;;
-	    *)	# PMDA_INTERFACE_2 or later
-		    help_version=2
-		    ;;
-	esac
 	sed -e "/^@ $SYMDOM\./s/$SYMDOM\./$domain./" <$help_source \
-	| newhelp -n root -v $help_version -o $help_source
+	| newhelp -n root -o $help_source
     fi
 
     if [ "X$pmda_type" = Xperl -o "X$pmda_type" = Xpython ]
@@ -1369,7 +1399,7 @@ _remove()
 	for __n in $pmns_name
 	do
 	    $PCP_ECHO_PROG $PCP_ECHO_N "Check $__n metrics have gone away ... ""$PCP_ECHO_C"
-	    if pminfo -n $NAMESPACE -f $__n >$tmp/base 2>&1
+	    if PCP_DERIVED_CONFIG= pminfo -n $NAMESPACE -v -b 1000 $__n >$tmp/base 2>&1
 	    then
 		echo "Arrgh, something has gone wrong!"
 		cat $tmp/base
@@ -1504,8 +1534,6 @@ pmns_source=pmns
 pmns_dupok=false
 #	Source for the helptext
 help_source=help
-#	Assume libpcp_pmda.so.3
-pmda_interface=3
 #	Full pathname to directory where PMDA is to be found ...
 #	exectable and/or DSO, domain.h, pmns, control files, etc.
 pmda_dir="`pwd`"
