@@ -1,11 +1,11 @@
 /*
- * Copyright (c) 2016-2020 Red Hat.
+ * Copyright (c) 2016-2023 Red Hat.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation; either version 2 of the License, or (at your
  * option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
@@ -70,8 +70,8 @@ enum {
 	PROC_MEMORY_PROCS,	/* 46 memory group processes -> proc PMDA */
 	PROC_NET_CLS_GROUPS,	/* 47 network class fication control groups -> proc PMDA */
 	PROC_NET_CLS_PROCS,	/* 48 network classification group processes -> proc PMDA */
-	CLUSTER_INTERRUPT_LINES,/* 49 /proc/interrupts percpu interrupts */
-	CLUSTER_INTERRUPT_OTHER,/* 50 /proc/interrupts percpu interrupts */
+	CLUSTER_INTERRUPT_LINES,/* deprecated: do not re-use 49 */
+	CLUSTER_INTERRUPT_OTHER,/* deprecated: do not re-use 50 */
 	PROC_PID_FD,		/* 51 /proc/<pid>/fd -> proc PMDA */
 	CLUSTER_LV,		/* deprecated: do not re-use 52 */
 	CLUSTER_NET_NETSTAT,    /* 53 /proc/net/netstat */
@@ -112,6 +112,9 @@ enum {
 	CLUSTER_ZRAM_MM_STAT,	/* 88 /sys/block/zram[0-9]/mm_stat metrics */
 	CLUSTER_ZRAM_BD_STAT,	/* 89 /sys/block/zram[0-9]/bd_stat metrics */
 	CLUSTER_NET_ALL,	/* 90 /proc/net/dev aggregate metrics */
+	CLUSTER_FCHOST,		/* 91 /sys/class/fc_host metrics */
+	CLUSTER_WWID,		/* 92 multipath aggregated stats */
+	CLUSTER_PRESSURE_IRQ,	/* 93 /proc/pressure/irq metrics */
 
 	NUM_CLUSTERS		/* one more than highest numbered cluster */
 };
@@ -127,6 +130,7 @@ enum {
 	REFRESH_NET_LINKUP,
 	REFRESH_NET_RUNNING,
 	REFRESH_NET_WIRELESS,
+	REFRESH_NET_VIRTUAL,
 
 	REFRESH_NETADDR_INET,
 	REFRESH_NETADDR_IPV6,
@@ -135,6 +139,9 @@ enum {
 	REFRESH_PROC_DISKSTATS,
 	REFRESH_PROC_PARTITIONS,
 
+	REFRESH_SYSFS_KERNEL_UEVENTSEQ,
+	REFRESH_SYSFS_KERNEL_EXTFRAG,
+
 	NUM_REFRESHES		/* one more than highest refresh index */
 };
 
@@ -146,7 +153,7 @@ enum {
 	DISK_INDOM,		/* 1 - disks */
 	LOADAVG_INDOM,		/* 2 - 1, 5, 15 minute load averages */
 	NET_DEV_INDOM,		/* 3 - network interfaces */
-	INTERRUPTS_INDOM,	/* 4 - interrupt lines */
+	INTERRUPT_INDOM,	/* 4 - interrupt lines */
 	FILESYS_INDOM,		/* 5 - mounted bdev filesystems */
 	SWAPDEV_INDOM,		/* 6 - swap devices */
 	NFS_INDOM,		/* 7 - nfs operations */
@@ -168,8 +175,8 @@ enum {
 	ICMPMSG_INDOM,          /* 23 - icmp message types */
 	DM_INDOM,		/* 24 - device mapper devices */
 	MD_INDOM,		/* 25 - multi-device devices */
-	INTERRUPT_NAMES_INDOM,	/* 26 - persistent percpu interrupts IDs */
-	SOFTIRQS_NAMES_INDOM,	/* 27 - persistent percpu softirqs IDs */
+	INTERRUPT_NAMES_INDOM_DEPRECATED, /* deprecated 26 */
+	SOFTIRQ_NAMES_INDOM_DEPRECATED,  /* deprecated 27 */
 	IPC_STAT_INDOM,	        /* 28 - ipc shm_stat shmid */
 	IPC_MSG_INDOM,	        /* 29 - ipc msg_stat msgid */
 	IPC_SEM_INDOM,	        /* 30 - ipc sem_stat msgid */
@@ -178,9 +185,13 @@ enum {
 	ZONEINFO_PROTECTION_INDOM,	/* 33 - proc zoneinfo protection item */
 	TAPEDEV_INDOM,		/* 34 - tape devices */
 	TTY_INDOM,              /* 35 - serial tty devices */
-	SOFTIRQS_INDOM,		/* 36 - softirqs */
+	SOFTIRQ_INDOM,		/* 36 - softirqs */
 	PRESSUREAVG_INDOM,	/* 37 - 10, 60, 300 second pressure averages */
 	ZRAM_INDOM,		/* 38 - compressed RAM devices */
+	FCHOST_INDOM,		/* 39 - fibrechannel hosts */
+	INTERRUPT_CPU_INDOM,	/* 40 - per-CPU interrupt lines */
+	SOFTIRQ_CPU_INDOM,	/* 41 - per-CPU soft IRQs */
+	WWID_INDOM,		/* 42 - per-WWID multipath device */
 
 	NUM_INDOMS		/* one more than highest numbered cluster */
 };
@@ -196,9 +207,8 @@ extern pmdaIndom *linux_pmda_indom(int);
  */
 #define	LINUX_TEST_MODE		(1<<0)
 #define	LINUX_TEST_STATSPATH	(1<<1)
-#define	LINUX_TEST_MEMINFO	(1<<2)
-#define	LINUX_TEST_NCPUS	(1<<3)
-#define	LINUX_TEST_NNODES	(1<<4)
+#define	LINUX_TEST_NCPUS	(1<<2)
+#define	LINUX_TEST_NNODES	(1<<3)
 extern int linux_test_mode;
 
 /*  
@@ -276,6 +286,14 @@ typedef struct {
     unsigned int	cache_align;
 } cpuinfo_t;
 
+typedef struct {
+    unsigned int	flags;
+    uint64_t		count;
+    uint64_t		time;
+    uint32_t		max;
+    uint32_t		min;
+} cpufreq_t;
+
 typedef struct {
     unsigned int	flags;
     uint64_t		processed;
@@ -293,15 +311,18 @@ typedef struct {
     struct linux_table	*meminfo;
     struct linux_table	*memstat;
     double		bandwidth;
+    float		extfrag_unusable;  /* external fragmentation */
+    unsigned int	num_extfrag_index; /* memory allocator order */
 } pernode_t;
 
 typedef struct {
     unsigned int	cpuid;  /* cpu%d instance name */
-    unsigned int	instid; /* internal instance id */
+    unsigned int	flags;	/* refresh status */
     pernode_t		*node;
     char		*name;
     cpuacct_t		stat;
     cpuinfo_t		info;
+    cpufreq_t		freq;
     softnet_t		*softnet;
 } percpu_t;
 
