@@ -14,6 +14,15 @@
 #include "linux.h"
 #include "proc_net_netstat.h"
 
+/*
+ * used to control once per execution checking for buffer truncation and
+ * unknown fields in the "stat" file
+ * == 1 initially
+ * == 0 if checks done and nothing bad was found
+ * < 0 if parsing is botched in a way that makes refreshing non-sensical
+ */
+static int	onetrip = 1;
+
 extern proc_net_netstat_t	_pm_proc_net_netstat;
 
 typedef struct {
@@ -21,6 +30,8 @@ typedef struct {
     __uint64_t		*offset;
 } netstat_fields_t;
 
+__uint64_t	not_exported;
+
 netstat_fields_t netstat_ip_fields[] = {
     { .field = "InNoRoutes",
      .offset = &_pm_proc_net_netstat.ip[_PM_NETSTAT_IPEXT_INNOROUTES] },
@@ -56,6 +67,9 @@ netstat_fields_t netstat_ip_fields[] = {
      .offset = &_pm_proc_net_netstat.ip[_PM_NETSTAT_IPEXT_ECT0PKTS] },
     { .field = "InCEPkts",
      .offset = &_pm_proc_net_netstat.ip[_PM_NETSTAT_IPEXT_CEPKTS] },
+    { .field = "ReasmOverlaps",
+      .offset = &_pm_proc_net_netstat.ip[_PM_NETSTAT_IPEXT_REASMOVERLAPS] },
+
     { .field = NULL, .offset = NULL }
 };
 
@@ -239,7 +253,7 @@ netstat_fields_t netstat_tcp_fields[] = {
     { .field = "TCPFastOpenActive",
      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPFASTOPENACTIVE] },
     { .field = "TCPFastOpenPassive",
-     .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPFASTOPENACTIVEFAIL] },
+     .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPFASTOPENPASSIVE] },
     { .field = "TCPFastOpenPassiveFail",
      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPFASTOPENPASSIVEFAIL] },
     { .field = "TCPFastOpenListenOverflow",
@@ -262,6 +276,75 @@ netstat_fields_t netstat_tcp_fields[] = {
      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPSYNRETRANS] },
     { .field = "TCPOrigDataSent",
      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPORIGDATASENT] },
+    { .field = "TCPBacklogCoalesce",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPBACKLOGCOALESCE] },
+    { .field = "TCPMemoryPressuresChrono",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPMEMORYPRESSURESCHRONO] },
+    { .field = "TCPMD5Failure",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPMD5FAILURE] },
+    { .field = "PFMemallocDrop",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_PFMEMALLOCDROP] },
+    { .field = "TCPFastOpenActiveFail",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPFASTOPENACTIVEFAIL] },
+    { .field = "TCPFastOpenBlackhole",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPFASTOPENBLACKHOLE] },
+    { .field = "TCPHystartTrainDetect",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPHYSTARTTRAINDETECT] },
+    { .field = "TCPHystartTrainCwnd",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPHYSTARTTRAINCWND] },
+    { .field = "TCPHystartDelayDetect",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPHYSTARTDELAYDETECT] },
+    { .field = "TCPHystartDelayCwnd",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPHYSTARTDELAYCWND] },
+    { .field = "TCPACKSkippedSynRecv",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPACKSKIPPEDSYNRECV] },
+    { .field = "TCPACKSkippedPAWS",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPACKSKIPPEDPAWS] },
+    { .field = "TCPACKSkippedSeq",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPACKSKIPPEDSEQ] },
+    { .field = "TCPACKSkippedFinWait2",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPACKSKIPPEDFINWAIT2] },
+    { .field = "TCPACKSkippedTimeWait",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPACKSKIPPEDTIMEWAIT] },
+    { .field = "TCPACKSkippedChallenge",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPACKSKIPPEDCHALLENGE] },
+    { .field = "TCPWinProbe",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPWINPROBE] },
+    { .field = "TCPKeepAlive",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPKEEPALIVE] },
+    { .field = "TCPMTUPFail",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPMTUPFAIL] },
+    { .field = "TCPMTUPSuccess",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPMTUPSUCCESS] },
+    { .field = "TCPDelivered",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPDELIVERED] },
+    { .field = "TCPDeliveredCE",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPDELIVEREDCE] },
+    { .field = "TCPAckCompressed",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPACKCOMPRESSED] },
+    { .field = "TCPZeroWindowDrop",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPZEROWINDOWDROP] },
+    { .field = "TCPRcvQDrop",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPRCVQDROP] },
+    { .field = "TCPWqueueTooBig",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPWQUEUETOOBIG] },
+    { .field = "TCPFastOpenPassiveAltKey",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPFASTOPENPASSIVEALTKEY] },
+    { .field = "TcpTimeoutRehash",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPTIMEOUTREHASH] },
+    { .field = "TcpDuplicateDataRehash",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPDUPLICATEDATAREHASH] },
+    { .field = "TCPDSACKRecvSegs",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPDSACKRECVSEGS] },
+    { .field = "TCPDSACKIgnoredDubious",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPDSACKIGNOREDDUBIOUS] },
+    { .field = "TCPMigrateReqSuccess",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPMIGRATEREQSUCCESS] },
+    { .field = "TCPMigrateReqFailure",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPMIGRATEREQFAILURE] },
+    { .field = "TCPLoss",
+      .offset = &_pm_proc_net_netstat.tcp[_PM_NETSTAT_TCPEXT_TCPLOSS] },
+
     { .field = NULL, .offset = NULL }
 };
 
@@ -292,6 +375,83 @@ netstat_fields_t netstat_mptcp_fields[] = {
      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_DSSNOTMATCHING] },
     { .field = "InfiniteMapRx",
      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_INFINITEMAPRX] },
+    { .field = "MPCapableSYNTX",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_MPCAPABLESYNTX] },
+    { .field = "MPCapableSYNACKRX",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_MPCAPABLESYNACKRX] },
+    { .field = "MPFallbackTokenInit",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_MPFALLBACKTOKENINIT] },
+    { .field = "DSSNoMatchTCP",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_DSSNOMATCHTCP] },
+    { .field = "DataCsumErr",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_DATACSUMERR] },
+    { .field = "OFOQueueTail",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_OFOQUEUETAIL] },
+    { .field = "OFOQueue",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_OFOQUEUE] },
+    { .field = "OFOMerge",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_OFOMERGE] },
+    { .field = "NoDSSInWindow",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_NODSSINWINDOW] },
+    { .field = "DuplicateData",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_DUPLICATEDATA] },
+    { .field = "AddAddr",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_ADDADDR] },
+    { .field = "AddAddrDrop",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_ADDADDRDROPS] },
+    { .field = "EchoAdd",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_ECHOADD] },
+    { .field = "PortAdd",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_PORTADD] },
+    { .field = "MPJoinPortSynRx",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_MPJOINPORTSYNRX] },
+    { .field = "MPJoinPortSynAckRx",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_MPJOINPORTSYNACKRX] },
+    { .field = "MPJoinPortAckRx",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_MPJOINPORTACKRX] },
+    { .field = "MismatchPortSynRx",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_MISMATCHPORTSYNRX] },
+    { .field = "MismatchPortAckRx",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_MISMATCHPORTACKRX] },
+    { .field = "RmAddr",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_RMADDR] },
+    { .field = "RmAddrDrop",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_RMADDRDROPS] },
+    { .field = "RmSubflow",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_RMSUBFLOW] },
+    { .field = "MPPrioTx",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_MPPRIOTX] },
+    { .field = "MPPrioRx",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_MPPRIORX] },
+    { .field = "RcvPruned",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_RCVPRUNED] },
+    { .field = "MPFailTx",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_MPFAILTX] },
+    { .field = "MPFailRx",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_MPFAILRX] },
+    { .field = "SubflowStale",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_SUBFLOWSTALE] },
+    { .field = "SubflowRecover",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_SUBFLOWRECOVER] },
+    { .field = "MPFastcloseTx",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_MPFASTCLOSETX] },
+    { .field = "MPFastcloseRx",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_MPFASTCLOSERX] },
+    { .field = "MPRstTx",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_MPRSTTX] },
+    { .field = "MPRstRx",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_MPRSTRX] },
+    { .field = "InfiniteMapTx",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_INFINITEMAPTX] },
+    { .field = "SndWndShared",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_SNDWNDSHARED] },
+    { .field = "RcvWndShared",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_RCVWNDSHARED] },
+    { .field = "RcvWndConflictUpdate",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_RCVWNDCONFLICTUPDATE] },
+    { .field = "RcvWndConflict",
+      .offset = &_pm_proc_net_netstat.mptcp[_PM_NETSTAT_MPTCPEXT_RCVWNDCONFLICT] },
+
     { .field = NULL, .offset = NULL }
 };
 
@@ -310,6 +470,11 @@ get_fields(netstat_fields_t *fields, char *header, char *buffer)
 	indices[i] = p;
     }
     count = i;
+    while (p != NULL) {
+	if (onetrip == 1)
+	    pmNotifyErr(LOG_WARNING, "proc_net_netstat: %s extra field \"%s\" (increase NETSTAT_MAX_COLUMNS)\n", header, p);
+	p = strtok(NULL, " \n");
+    }
 
     /*
      * Extract values via back-referencing column headings.
@@ -320,20 +485,36 @@ get_fields(netstat_fields_t *fields, char *header, char *buffer)
      * kernel - but may be out-of-order for older kernels).
      */
     strtok(buffer, " ");
-    for (i = j = 0; j < count && fields[i].field; j++, i++) {
+    for (i = j = 0; j <= count; j++) {
         if ((p = strtok(NULL, " \n")) == NULL)
             break;
-        if (strcmp(fields[i].field, indices[j]) == 0)
-            *fields[i].offset = strtoull(p, NULL, 10);
+	if (fields[i].field == NULL)
+	    /* wrap search in fields table */
+	    i = 0;
+        if (strcmp(fields[i].field, indices[j]) == 0) {
+	    if (fields[i].offset != &not_exported)
+		*fields[i].offset = strtoull(p, NULL, 10);
+	    else if (onetrip)
+		pmNotifyErr(LOG_INFO, "proc_net_netstat: %s \"%s\" parsed but not exported\n", header, indices[j]);
+	    i++;
+	}
         else {
             for (i = 0; fields[i].field; i++) {
                 if (strcmp(fields[i].field, indices[j]) != 0)
                     continue;
-                *fields[i].offset = strtoull(p, NULL, 10);
+		if (fields[i].offset != &not_exported)
+		    *fields[i].offset = strtoull(p, NULL, 10);
+		else if (onetrip)
+		    pmNotifyErr(LOG_INFO, "proc_net_netstat: %s \"%s\" parsed but not exported\n", header, indices[j]);
                 break;
             }
-	    if (fields[i].field == NULL) /* not found, ignore */
-		i = 0;
+	    if (fields[i].field == NULL) {
+		/* not found, warn */
+		if (onetrip == 1)
+		    pmNotifyErr(LOG_WARNING, "proc_net_netstat: %s unknown field[#%d] \"%s\"\n", header, j, indices[j]);
+	    }
+	    else
+		i++;
 	}
     }
 }
@@ -360,19 +541,67 @@ init_refresh_proc_net_netstat(proc_net_netstat_t *netstat)
 	*(NETSTAT_MPTCP_OFFSET(i, netstat->mptcp)) = -1;
 }
 
+size_t
+check_read_trunc(char *buf, FILE *fp)
+{
+    char	*p;
+    size_t	lost;
+    int		c;
+
+    for (p = buf; *p; p++)
+	;
+    if (p > buf)
+	p--;
+    if (*p == '\n') {
+	return 0;
+    }
+
+    lost = 1;
+    while ((c = fgetc(fp)) != EOF) {
+	if (c == '\n')
+	    break;
+	lost++;
+    }
+
+    return lost;
+}
+
+#define MAXLINELEN 4192
+
 int
 refresh_proc_net_netstat(proc_net_netstat_t *netstat)
 {
     /* Need a sufficiently large value to hold a full line */
-    char	buf[MAXPATHLEN];
-    char	header[2048];
+    char	buf[MAXLINELEN];
+    char	header[MAXLINELEN];
     FILE	*fp;
 
+    if (onetrip < 0)
+	return onetrip;
+
     init_refresh_proc_net_netstat(netstat);
     if ((fp = linux_statsfile("/proc/net/netstat", buf, sizeof(buf))) == NULL)
 	return -oserror();
     while (fgets(header, sizeof(header), fp) != NULL) {
+	if (onetrip == 1) {
+	    size_t	lost;
+	    if ((lost = check_read_trunc(header, fp)) != 0) {
+		pmNotifyErr(LOG_ERR, "refresh_proc_net_netstat: header[] too small, need at least %zd more bytes\n", lost);
+		onetrip = PM_ERR_BOTCH;
+		fclose(fp);
+		return onetrip;
+	    }
+	}
 	if (fgets(buf, sizeof(buf), fp) != NULL) {
+	    if (onetrip == 1) {
+		size_t	lost;
+		if ((lost = check_read_trunc(buf, fp)) != 0) {
+		    pmNotifyErr(LOG_ERR, "refresh_proc_net_netstat: buf[] too small, need at least %zd more bytes\n", lost);
+		    onetrip = PM_ERR_BOTCH;
+		    fclose(fp);
+		    return onetrip;
+		}
+	    }
 	    if (strncmp(buf, "IpExt:", 6) == 0)
 		get_fields(netstat_ip_fields, header, buf);
 	    else if (strncmp(buf, "TcpExt:", 7) == 0)
@@ -383,6 +612,7 @@ refresh_proc_net_netstat(proc_net_netstat_t *netstat)
 		pmNotifyErr(LOG_ERR, "Unrecognised netstat row: %s\n", buf);
 	}
     }
+    onetrip = 0;	/* been thru the whole file, assume OK next time */
     fclose(fp);
     return 0;
 }
