@@ -1,13 +1,13 @@
 /*
  * Linux sysfs_kernel cluster
  *
- * Copyright (c) 2009, Red Hat.
- * 
+ * Copyright (c) 2009,2023 Red Hat.
+ *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation; either version 2 of the License, or (at your
  * option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
@@ -22,6 +22,6 @@ typedef struct {
 } sysfs_kernel_t;
 
 /* refresh sysfs_kernel */
-extern int refresh_sysfs_kernel(sysfs_kernel_t *);
+extern int refresh_sysfs_kernel(sysfs_kernel_t *, int *);
 
 #endif /* SYSFS_KERNEL_H */
