@@ -1,6 +1,7 @@
 /*
  * Trace PMDA - process level transaction monitoring for libpcp_trace processes
  *
+ * Copyright (c) 2012 Red Hat.
  * Copyright (c) 1997-2000 Silicon Graphics, Inc.  All Rights Reserved.
  * 
  * This program is free software; you can redistribute it and/or modify it
@@ -12,20 +13,17 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <ctype.h>
 #include "pmapi.h"
-#include "impl.h"
+#include "libpcp.h"
 #include "pmda.h"
 #include "domain.h"
 #include "trace.h"
 #include "trace_dev.h"
 #include "comms.h"
+#include "data.h"
 
 #define DEFAULT_TIMESPAN	60	/* one minute  */
 #define DEFAULT_BUFSIZE		5	/* twelve second update */
@@ -35,14 +33,16 @@ struct timeval	interval;
 unsigned int	rbufsize  = DEFAULT_BUFSIZE;
 int		ctlport	  = -1;
 char		*ctlsock;
+int		somedebug;
 
 static char	mypath[MAXPATHLEN];
+static char	*username;
 
 extern void traceInit(pmdaInterface *dispatch);
 extern void traceMain(pmdaInterface *dispatch);
 extern int  updateObserveValue(const char *);
 extern int  updateCounterValue(const char *);
-extern void debuglibrary(int);
+extern void debuglibrary(void);
 
 static void
 usage(void)
@@ -55,11 +55,12 @@ Options:\n\
   -l logfile  write log into logfile rather than using default file\n\
   -A access   host based access control\n\
   -I port     expect programs to connect on given inet port (number/name)\n\
+  -M username user account to run under (default \"pcp\")\n\
   -N buckets  number of historical data buffers maintained\n\
   -T period   time over which samples are considered (default 60 seconds)\n\
   -U units    export observation values using the given units\n\
   -V units    export counter values using the given units\n",
-	      pmProgname);
+	      pmGetProgname());
     exit(1);
 }
 
@@ -90,31 +91,32 @@ parseAuth(char *spec)
     char *p, *endnum;
 
     if (first) {
+	if (__pmAccAddOp(TR_OP_SEND) < 0) {
+	    pmNotifyErr(LOG_ERR, "failed to add send auth operation");
+	    return -1;
+	}
 	first = 0;
-	__pmAccAddOp(TR_OP_SEND);
     }
 
     if (strncasecmp(spec, "disallow:", 9) == 0) {
 	p = squash(&spec[9], &offset);
 	if (p == NULL || p[0] == '\0') {
-	    fprintf(stderr, "%s: invalid disallow (%s)\n", pmProgname, spec);
+	    fprintf(stderr, "%s: invalid disallow (%s)\n", pmGetProgname(), spec);
 	    if (p)
 	    	free(p);
 	    return -1;
 	}
-#ifdef PCP_DEBUG
-	if (pmDebug & DBG_TRACE_APPL0)
+	if (pmDebugOptions.appl0)
 	    fprintf(stderr, "deny: host '%s'\n", p);
-#endif
 	denyops = TR_OP_SEND;
 	if (__pmAccAddHost(p, specops, denyops, 0) < 0)
-	    __pmNotifyErr(LOG_ERR, "failed to add authorisation (%s)", p);
+	    pmNotifyErr(LOG_ERR, "failed to add authorisation (%s)", p);
 	free(p);
     }
     else if (strncasecmp(spec, "allow:", 6) == 0) {
 	p = squash(&spec[6], &offset);
 	if (p == NULL || p[0] == '\0') {
-	    fprintf(stderr, "%s: invalid allow (%s)\n", pmProgname, spec);
+	    fprintf(stderr, "%s: invalid allow (%s)\n", pmGetProgname(), spec);
 	    if (p)
 	    	free(p);
 	    return -1;
@@ -122,22 +124,20 @@ parseAuth(char *spec)
 	offset += 7;
 	maxconn = (int)strtol(&spec[offset], &endnum, 10);
 	if (*endnum != '\0' || maxconn < 0) {
-	    fprintf(stderr, "%s: bogus max connection in '%s'\n", pmProgname,
+	    fprintf(stderr, "%s: bogus max connection in '%s'\n", pmGetProgname(),
 		    &spec[offset]);
 	    free(p);
 	    return -1;
 	}
-#ifdef PCP_DEBUG
-	if (pmDebug & DBG_TRACE_APPL0)
+	if (pmDebugOptions.appl0)
 	    fprintf(stderr, "allow: host '%s', maxconn=%d\n", p, maxconn);
-#endif
 	denyops = TR_OP_NONE;
 	if (__pmAccAddHost(p, specops, denyops, maxconn) < 0)
-	    __pmNotifyErr(LOG_ERR, "failed to add authorisation (%s)", p);
+	    pmNotifyErr(LOG_ERR, "failed to add authorisation (%s)", p);
 	free(p);
     }
     else {
-	fprintf(stderr, "%s: access spec is invalid (%s)\n", pmProgname, spec);
+	fprintf(stderr, "%s: access spec is invalid (%s)\n", pmGetProgname(), spec);
 	return -1;
     }
     return 0;
@@ -149,18 +149,20 @@ main(int argc, char **argv)
     pmdaInterface	dispatch;
     char		*endnum;
     int			err = 0;
-    int			sep = __pmPathSeparator();
+    int			sep = pmPathSeparator();
     int			c = 0;
+    int			sts;
 
-    __pmSetProgname(argv[0]);
+    pmSetProgname(argv[0]);
+    pmGetUsername(&username);
 
-    snprintf(mypath, sizeof(mypath), "%s%c" "trace" "%c" "help",
+    pmsprintf(mypath, sizeof(mypath), "%s%c" "trace" "%c" "help",
 		pmGetConfig("PCP_PMDAS_DIR"), sep, sep);
-    pmdaDaemon(&dispatch, PMDA_INTERFACE_2, pmProgname, TRACE,
+    pmdaDaemon(&dispatch, PMDA_INTERFACE_2, pmGetProgname(), TRACE,
 		"trace.log", mypath);
 
     /* need - port, as well as time interval and time span for averaging */
-    while ((c = pmdaGetOpt(argc, argv, "A:D:d:I:l:T:N:U:V:?",
+    while ((c = pmdaGetOpt(argc, argv, "A:D:d:I:l:T:M:N:U:V:?",
 						&dispatch, &err)) != EOF) {
 	switch(c) {
 	case 'A':
@@ -168,22 +170,34 @@ main(int argc, char **argv)
 		err++;
 	    /* add optarg to access control list */
 	    break;
+	case 'D':
+	    sts = pmSetDebug(optarg);
+	    if (sts < 0) {
+		fprintf(stderr, "%s: unrecognized debug options specification (%s)\n",
+		    pmGetProgname(), optarg);
+		err++;
+	    }
+	    somedebug = 1;
+	    break;
 	case 'I':
 	    ctlport = (int)strtol(optarg, &endnum, 10);
 	    if (*endnum != '\0' || ctlport < 0)
 		ctlsock = optarg;
 	    break;
+	case 'M':
+	    username = optarg;
+	    break;
 	case 'N':
 	    rbufsize = (int)strtol(optarg, &endnum, 10);
 	    if (*endnum != '\0' || rbufsize < 1) {
-		fprintf(stderr, "%s: -N requires a positive number.\n", pmProgname);
+		fprintf(stderr, "%s: -N requires a positive number.\n", pmGetProgname());
 		err++;
 	    }
 	    break;
 	case 'T':
 	    if (pmParseInterval(optarg, &timespan, &endnum) < 0) {
 		fprintf(stderr, "%s: -T requires a time interval: %s\n",
-			pmProgname, endnum);
+			pmGetProgname(), endnum);
 		free(endnum);
 		err++;
 	    }
@@ -208,11 +222,10 @@ main(int argc, char **argv)
     interval.tv_usec = (long)((timespan.tv_sec % rbufsize) * 1000000);
     rbufsize++;		/* reserve space for the `working' buffer */
 
-#ifdef PCP_DEBUG
-    debuglibrary(pmDebug);
-#endif
+    debuglibrary();
 
     pmdaOpenLog(&dispatch);
+    pmSetProcessIdentity(username);
     traceInit(&dispatch);
     pmdaConnect(&dispatch);
     traceMain(&dispatch);
