@@ -2,7 +2,7 @@
  * pmie.c - performance inference engine
  ***********************************************************************
  *
- * Copyright (c) 2013-2015,2017 Red Hat.
+ * Copyright (c) 2013-2015,2017,2020,2022 Red Hat.
  * Copyright (c) 1995-2003 Silicon Graphics, Inc.  All Rights Reserved.
  * 
  * This program is free software; you can redistribute it and/or modify it
@@ -36,7 +36,6 @@
 #include "eval.h"
 #include "show.h"
 
-
 /***********************************************************************
  * constants
  ***********************************************************************/
@@ -96,7 +95,9 @@ static pmLongOptions longopts[] = {
     { "check", 0, 'C', 0, "parse configuration and exit" },
     { "config", 1, 'c', "FILE", "configuration file" },
     { "interact", 0, 'd', 0, "interactive debugging mode" },
+    { "primary", 0, 'P', 0, "execute as primary inference engine" },
     { "foreground", 0, 'f', 0, "run in the foreground, not as a daemon" },
+    { "systemd", 0, 'F', 0, "systemd mode - notify service manager (if any) when started and ready" },
     { "", 0, 'H', NULL }, /* was: no DNS lookup on the default hostname */
     { "", 1, 'j', "FILE", "stomp protocol (JMS) file" },
     { "logfile", 1, 'l', "FILE", "send status and error messages to FILE" },
@@ -115,7 +116,7 @@ static pmLongOptions longopts[] = {
 
 static pmOptions opts = {
     .flags = PM_OPTFLAG_STDOUT_TZ,
-    .short_options = "a:A:bc:CdD:efHh:j:l:n:O:qS:t:T:U:vVWXxzZ:?",
+    .short_options = "a:A:bc:CdD:efFHh:j:l:n:O:PqS:t:T:U:vVWXxzZ:?",
     .long_options = longopts,
     .short_usage = "[options] [filename ...]",
     .override = override,
@@ -351,7 +352,10 @@ startmonitor(void)
 	return;
     }
     /* seek to struct size and write one zero */
-    lseek(fd, sizeof(pmiestats_t)-1, SEEK_SET);
+    if (lseek(fd, sizeof(pmiestats_t)-1, SEEK_SET) < 0) {
+	fprintf(stderr, "%s: Warning: lseek failed for stats file %s: %s\n",
+		pmGetProgname(), perffile, osstrerror());
+    }
     if (write(fd, &zero, 1) != 1) {
 	fprintf(stderr, "%s: Warning: write failed for stats file %s: %s\n",
 		pmGetProgname(), perffile, osstrerror());
@@ -394,7 +398,7 @@ sigintproc(int sig)
 	doexit = sig;
     else {
 	/* for RH BZ 1327226 */
-	fprintf(stderr, "\nInterrupted!\n");
+	fprintf(stderr, "\nInterrupted! signal=%d\n", sig);
 	exit(1);
     }
 }
@@ -410,10 +414,13 @@ remap_stdout_stderr(void)
     setlinebuf(stdout);
     i = fileno(stdout);
     close(i);
-    if ((j = dup(fileno(stderr))) != i)
+    if ((j = dup(fileno(stderr))) != i) {
 	fprintf(stderr, "%s: Warning: failed to link stdout ... "
 			"dup() returns %d, expected %d (stderr=%d)\n",
 			pmGetProgname(), j, i, fileno(stderr));
+	if (j >= 0)
+	    close(j);
+    }
 }
 
 void
@@ -445,8 +452,8 @@ sigbadproc(int sig)
 {
     if (pmDebugOptions.desperate) {
 	pmNotifyErr(LOG_ERR, "Unexpected signal %d ...\n", sig);
-	fprintf(stderr, "\nProcedure call traceback ...\n");
-	__pmDumpStack(stderr);
+	fprintf(stderr, "\n");
+	__pmDumpStack();
 	fflush(stderr);
     }
     stopmonitor();
@@ -465,9 +472,10 @@ getargs(int argc, char *argv[])
     char		*commandlog = NULL;
     char		*subopts;
     char		*subopt;
-    char		*msg;
+    char		*msg = NULL;
     int			checkFlag = 0;
     int			foreground = 0;
+    int			primary = 0;
     int			sts;
     int			c;
     int			bflag = 0;
@@ -545,6 +553,16 @@ getargs(int argc, char *argv[])
 	    foreground = 1;
 	    break;
 
+	case 'P':			/* primary (local) pmie process */
+	    primary = 1;
+	    isdaemon = 1;
+	    break;
+
+	case 'F':			/* systemd mode */
+	    systemd = 1;
+	    isdaemon = 1;
+	    break;
+
 	case 'H': 			/* deprecated: no DNS lookups */
 	    break;
 
@@ -612,6 +630,11 @@ getargs(int argc, char *argv[])
 		pmGetProgname());
 	opts.errors++;
     }
+    if (!opts.errors && foreground && systemd) {
+	pmprintf("%s: the -f and -F options are mutually exclusive\n",
+		pmGetProgname());
+	opts.errors++;
+    }
     if (!opts.errors && bflag && agent) {
 	pmprintf("%s: the -b and -x options are incompatible\n",
 		pmGetProgname());
@@ -670,7 +693,7 @@ getargs(int argc, char *argv[])
 
     if (commandlog != NULL) {
 	logfp = pmOpenLog(pmGetProgname(), commandlog, stderr, &sts);
-	if (realpath(commandlog, logfile) == NULL) {
+	if (strcmp(commandlog, "-") != 0 && realpath(commandlog, logfile) == NULL) {
 	    fprintf(stderr, "%s: cannot find realpath for log %s: %s\n",
 		    pmGetProgname(), commandlog, osstrerror());
 	    exit(1);
@@ -686,6 +709,24 @@ getargs(int argc, char *argv[])
     if ((bflag || isdaemon) && !agent)
 	remap_stdout_stderr();
 
+    if (isdaemon) {			/* daemon mode */
+	/* Note: we can no longer unilaterally close stdin here, as it
+	 * can really confuse remap_stdout_stderr() during log rotation!
+	 */
+	if (agent)
+	    close(fileno(stdin));
+#ifndef IS_MINGW
+	if (!systemd)
+	    setsid();	/* not process group leader, lose controlling tty */
+#endif
+	if (primary) {
+	    if (systemd) {
+		__pmServerNotifyServiceManagerReady(getpid());
+	    }
+	    __pmServerCreatePIDFile(pmGetProgname(), 0);
+	}
+    }
+
     /* default host from leftmost archive on command line, or from
        discovery after a brief connection */
     if (archives) {
@@ -699,9 +740,13 @@ getargs(int argc, char *argv[])
     }
 
     if (!archives && !interactive) {
-	if (commandlog != NULL)
-            fprintf(stderr, "pmie: PID = %" FMT_PID ", via %s\n\n",
+	if (commandlog != NULL) {
+            fprintf(stderr, "pmie: PID = %" FMT_PID ", via %s",
                     (pid_t)getpid(), dfltHostConn);
+	    if (primary)
+		fprintf(stderr, " [primary]");
+	    fprintf(stderr, "\n\n");
+	}
 	startmonitor();
     }
 
@@ -715,7 +760,7 @@ getargs(int argc, char *argv[])
     if (archives) {
 	pmtimevalFromReal(last, &tv2);
     } else {
-	tv2.tv_sec = INT_MAX;		/* sizeof(time_t) == sizeof(int) */
+	tv2.tv_sec = PM_MAX_TIME_T;
 	tv2.tv_usec = 0;
     }
     if (pmParseTimeWindow(opts.start_optarg, opts.finish_optarg,
@@ -724,6 +769,7 @@ getargs(int argc, char *argv[])
                           &tv, &tv2, &tv1,
 		          &msg) < 0) {
 	fputs(msg, stderr);
+	free(msg);
         exit(1);
     }
     start = pmtimevalToReal(&tv1);
@@ -759,17 +805,6 @@ getargs(int argc, char *argv[])
 					 * Kevin Wang <kjw@rightsock.com>
 					 */
 
-    if (isdaemon) {			/* daemon mode */
-	/* Note: we can no longer unilaterally close stdin here, as it
-	 * can really confuse remap_stdout_stderr() during log rotation!
-	 */
-	if (agent)
-	    close(fileno(stdin));
-#ifndef IS_MINGW
-	setsid();	/* not process group leader, lose controlling tty */
-#endif
-    }
-
     if (stomping)
 	stompInit();			/* connect to our message server */
 
@@ -780,7 +815,7 @@ getargs(int argc, char *argv[])
     if (archives) {
 	pmtimevalFromReal(last, &tv2);
     } else {
-	tv2.tv_sec = INT_MAX;
+	tv2.tv_sec = PM_MAX_TIME_T;
 	tv2.tv_usec = 0;
     }
     if (pmParseTimeWindow(opts.start_optarg, opts.finish_optarg,
@@ -789,6 +824,7 @@ getargs(int argc, char *argv[])
                           &tv, &tv2, &tv1,
 		          &msg) < 0) {
 	fputs(msg, stderr);
+	free(msg);
 	exit(1);
     }
 
@@ -796,6 +832,9 @@ getargs(int argc, char *argv[])
     start = pmtimevalToReal(&tv1);
     stop = pmtimevalToReal(&tv2);
     runTime = stop - start;
+
+    if (msg != NULL)
+	free(msg);
 }
 
 /***********************************************************************
@@ -865,7 +904,7 @@ interact(void)
 		if (archives) {
 		    pmtimevalFromReal(last, &tv2);
 		} else {
-		    tv2.tv_sec = INT_MAX;
+		    tv2.tv_sec = PM_MAX_TIME_T;
 		    tv2.tv_usec = 0;
 		}
 		if (__pmParseTime(token, &tv1, &tv2, &tv1, &msg) < 0) {
@@ -921,6 +960,16 @@ interact(void)
 int
 main(int argc, char **argv)
 {
+#ifdef HAVE___EXECUTABLE_START
+    extern char		__executable_start;
+
+    /*
+     * optionally set address for start of my text segment, to be used
+     * in __pmDumpStack() if it is called later
+     */
+    __pmDumpStackInit((void *)&__executable_start);
+#endif
+
     pmGetUsername(&username);
     setlinebuf(stdout);
 
@@ -932,7 +981,10 @@ main(int argc, char **argv)
 
     if (interactive)
 	interact();
-    else
+    else {
 	run();
+	if (systemd)
+	    __pmServerNotifyServiceManagerStopping(getpid());
+    }
     exit(0);
 }
