@@ -33,6 +33,9 @@
 #if defined(HAVE_IEEEFP_H)
 #include <ieeefp.h>
 #endif
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
 
 extern char	*clientid;
 
@@ -125,34 +128,59 @@ newContext(Symbol *host, const char *hconn, int is_temp)
  * instance profile
  ***********************************************************************/
 
-/* equality (not symmetric) of instance names */
+/*
+ * equality of external instance names
+ * - try full match first, else
+ * - match up to first space of myname and indomname
+ */
 static int
-eqinst(char *i1, char *i2)
+eqinst(char *myname, char *indomname)
 {
-    int		n1 = (int) strlen(i1);
-    int		n2 = (int) strlen(i2);
+    char	*p;
+    int		mylen;
+    int		indomlen;
 
-    /* test equality of first word */
-    if ((strncmp(i1, i2, n1) == 0) && ((n1 == n2) || isspace((int)i2[n1])))
+    if (strcmp(myname, indomname) == 0) {
+	/* full match */
 	return 1;
+    }
 
-    do {	/* skip over first word */
-	i2++;
-	n2--;
-	if (n2 < n1)
-	    return 0;
-    } while (! isspace((int)*i2));
+    p = index(myname, ' ');
+    if (p == NULL) {
+	/* no space */
+	mylen = 0;
+    }
+    else {
+	mylen = p - myname;
+    }
 
-    do {	/* skip over spaces */
-	i2++;
-	n2--;
-	if (n2 < n1)
-	    return 0;
-    } while (isspace((int)*i2));
+    p = index(indomname, ' ');
+    if (p == NULL) {
+	/* no space */
+	indomlen = 0;
+    }
+    else {
+	indomlen = p - indomname;
+    }
 
-    /* test equality of second word */
-    if ((strncmp(i1, i2, n1) == 0) && ((n1 == n2) || isspace((int)i2[n1])))
+    if (mylen == 0 && indomlen == 0) {
+	/* no spaces in either */
+	return 0;
+    }
+
+    if (indomlen == 0) indomlen = strlen(indomname);
+    if (mylen == 0) mylen = strlen(myname);
+
+    if (mylen != indomlen) {
+	/* lengths to first space or end of string not equal */
+	return 0;
+    }
+
+    if (strncmp(myname, indomname, mylen) == 0) {
+	/* prefix match to first space */
 	return 1;
+    }
+
     return 0;
 }
 
@@ -621,14 +649,21 @@ scale(pmUnits in)
     return f;
 }
 
-
-/* initialize Metric */
-int      /* 1: ok, 0: try again later, -1: fail */
-initMetric(Metric *m)
+/*
+ * initialize / reinitialize Metric (m)
+ * reinit is 0 for init case, 1 for reinit case
+ *
+ * returns:
+ *	0	problem, need to retry this one again later
+ * 	1	all OK, can schedule fetching this one
+ *     -1	serious badness, don't even bother retrying
+ */
+static int
+init(Metric *m, int reinit)
 {
     char	*hname;
-    char	*hconn = symName(m->hconn);
-    char	*mname = symName(m->mname);
+    const char	*hconn = symName(m->hconn);
+    const char	*mname = symName(m->mname);
     char	**inames;
     int		*iids;
     int		handle;
@@ -636,29 +671,43 @@ initMetric(Metric *m)
     int		sts;
     int		i, j;
 
+    if (pmDebugOptions.appl1 && pmDebugOptions.desperate)
+	fprintf(stderr, "%sinitMetric(m=%p %s)\n", reinit ? "re" : "", m, mname);
+
     /* set up temporary context */
-    if ((handle = newContext(&m->hname, hconn, 1)) < 0)
+    if ((handle = newContext(&m->hname, hconn, 1)) < 0) {
+	if (pmDebugOptions.appl1 && pmDebugOptions.desperate)
+	    fprintf(stderr, "%sinitMetric: newContext failed: %s\n", reinit ? "re" : "", pmErrStr(handle));
 	return 0;
+    }
     hname = symName(m->hname);
 
     host_state_changed(hconn, STATE_RECONN);
 
     if ((sts = pmLookupName(1, &mname, &m->desc.pmid)) < 0) {
-	fprintf(stderr, "%s: metric %s not in namespace for %s\n"
+	if (!reinit)
+	    fprintf(stderr, "%s: metric %s not in namespace for %s\n"
 		"pmLookupName failed: %s\n",
 		pmGetProgname(), mname, findsource(hname, hconn), pmErrStr(sts));
+	if (pmDebugOptions.appl1 && pmDebugOptions.desperate)
+	    fprintf(stderr, "%sinitMetric: pmLookupName failed: %s\n", reinit ? "re" : "", pmErrStr(sts));
 	ret = 0;
 	goto end;
     }
 
     /* fill in performance metric descriptor */
     if ((sts = pmLookupDesc(m->desc.pmid, &m->desc)) < 0) {
-	fprintf(stderr, "%s: metric %s not currently available from %s\n"
+	if (!reinit)
+	    fprintf(stderr, "%s: metric %s not currently available from %s\n"
 		"pmLookupDesc failed: %s\n",
 		pmGetProgname(), mname, findsource(hname, hconn), pmErrStr(sts));
+	if (pmDebugOptions.appl1 && pmDebugOptions.desperate)
+	    fprintf(stderr, "%sinitMetric: pmLookupDesc(%s) failed: %s\n", reinit ? "re" : "", pmIDStr(m->desc.pmid), pmErrStr(sts));
 	ret = 0;
 	goto end;
     }
+    if (pmDebugOptions.appl1 && pmDebugOptions.desperate)
+	fprintf(stderr, "%sinitMetric: got metadata pmid=%s type=%s units=%s\n", reinit ? "re" : "", pmIDStr(m->desc.pmid), pmTypeStr(m->desc.type), pmUnitsStr(&m->desc.units));
 
     if (m->desc.type == PM_TYPE_AGGREGATE ||
 	m->desc.type == PM_TYPE_AGGREGATE_STATIC ||
@@ -691,8 +740,16 @@ initMetric(Metric *m)
 		    "pmGetIndom failed: %s\n", mname, findsource(hname, hconn), pmErrStr(sts));
 	    ret = 0;
 	}
-
-	if (ret == 1) {	/* got instance profile */
+	if (sts >= 0) {
+	    /* got instance domain */
+	    if (pmDebugOptions.appl1 && pmDebugOptions.desperate) {
+		fprintf(stderr, "%sinitMetric: got indom, %d instances", reinit ? "re" : "", sts);
+		if (sts > 0)
+		    fprintf(stderr, " \"%s\"", inames[0]);
+		if (sts > 1)
+		    fprintf(stderr, "...\"%s\"", inames[sts-1]);
+		fputc('\n', stderr);
+	    }
 	    if (m->specinst == 0) {
 		/* all instances */
 		m->iids = iids;
@@ -715,20 +772,14 @@ initMetric(Metric *m)
 			}
 		    }
 		    if (j == sts) {
-			pmNotifyErr(LOG_ERR, "metric %s from %s does not "
+			if (!reinit)
+			    pmNotifyErr(LOG_ERR, "metric %s from %s does not "
 				"(currently) have instance \"%s\"\n",
                                       mname, findsource(hname, hconn), m->inames[i]);
 			m->iids[i] = PM_IN_NULL;
 			ret = 0;
 		    }
 		}
-		if (sts > 0) {
-		    /*
-		     * pmGetInDom or pmGetInDomArchive returned some
-		     * instances above
-		     */
-		    free(iids);
-		}
 
 		/* 
 		 * if specinst != m_idom, then some not found ... move these
@@ -754,186 +805,28 @@ initMetric(Metric *m)
 
 	    if (pmDebugOptions.appl1) {
 		int	numinst;
-		fprintf(stderr, "initMetric: %s from %s: instance domain specinst=%d\n",
-			mname, hname, m->specinst);
+		fprintf(stderr, "%sinitMetric: %s from %s: instance domain specinst=%d\n",
+			reinit ? "re" : "", mname, hname, m->specinst);
 		if (m->m_idom < 1) fprintf(stderr, "  %d instances!\n", m->m_idom);
 		numinst =  m->specinst == 0 ? m->m_idom : m->specinst;
 		for (i = 0; i < numinst; i++) {
 		    fprintf(stderr, "  indom[%d]", i);
-		    if (m->iids[i] == PM_IN_NULL) 
+		    if (m->iids[i] == PM_IN_NULL) {
 			fprintf(stderr, " ?missing");
-		    else
-			fprintf(stderr, " %d", m->iids[i]);
-		    fprintf(stderr, " \"%s\"\n", m->inames[i]);
-		}
-	    }
-	    if (sts > 0) {
-		/*
-		 * pmGetInDom or pmGetInDomArchive returned some instances
-		 * above
-		 */
-		free(inames);
-	    }
-	}
-    }
-
-    if (ret == 1) {
-	/* compute conversion factor into canonical units
-	   - non-zero conversion factor flags initialized metric */
-	m->conv = scale(m->desc.units);
-
-	/* automatic rate computation */
-	if (m->desc.sem == PM_SEM_COUNTER) {
-	    m->vals = (double *) ralloc(m->vals, m->m_idom * sizeof(double));
-	    for (j = 0; j < m->m_idom; j++)
-		m->vals[j] = 0;
-	}
-    }
-
-end:
-    /* destroy temporary context */
-    pmDestroyContext(handle);
-
-    /* retry not meaningful for archives */
-    if (archives && (ret == 0))
-	ret = -1;
-
-    return ret;
-}
-
-
-/* reinitialize Metric - only for live host */
-int      /* 1: ok, 0: try again later, -1: fail */
-reinitMetric(Metric *m)
-{
-    char	*hname;
-    char	*hconn = symName(m->hconn);
-    char	*mname = symName(m->mname);
-    char	**inames;
-    int		*iids;
-    int		handle;
-    int		ret = 1;
-    int		sts;
-    int		i, j;
-
-    /* set up temporary context */
-    if ((handle = newContext(&m->hname, hconn, 1)) < 0)
-	return 0;
-    hname = symName(m->hname);
-
-    host_state_changed(hconn, STATE_RECONN);
-
-    if ((sts = pmLookupName(1, &mname, &m->desc.pmid)) < 0) {
-	ret = 0;
-	goto end;
-    }
-
-    /* fill in performance metric descriptor */
-    if ((sts = pmLookupDesc(m->desc.pmid, &m->desc)) < 0) {
-	ret = 0;
-        goto end;
-    }
-
-    if (m->desc.type == PM_TYPE_AGGREGATE ||
-	m->desc.type == PM_TYPE_AGGREGATE_STATIC ||
-	m->desc.type == PM_TYPE_EVENT ||
-	m->desc.type == PM_TYPE_HIGHRES_EVENT ||
-	m->desc.type == PM_TYPE_UNKNOWN) {
-	fprintf(stderr, "%s: metric %s has inappropriate type\n", pmGetProgname(), mname);
-	ret = -1;
-    }
-    else if (m->desc.indom == PM_INDOM_NULL) {
-	if (m->specinst != 0) {
-	    fprintf(stderr, "%s: metric %s has no instances\n", pmGetProgname(), mname);
-	    ret = -1;
-	}
-	else
-	    m->m_idom = 1;
-    }
-    else {
-	if ((sts = pmGetInDom(m->desc.indom, &iids, &inames)) < 0) { /* full profile */
-	    ret = 0;
-	}
-	else {
-	    if (m->specinst == 0) {
-		/* all instances */
-		m->iids = iids;
-		m->m_idom = sts;
-		m->inames = alloc(m->m_idom*sizeof(char *));
-		for (i = 0; i < m->m_idom; i++) {
-		    m->inames[i] = sdup(inames[i]);
-		}
-	    }
-	    else {
-		/* explicit instance profile */
-		m->m_idom = 0;
-		for (i = 0; i < m->specinst; i++) {
-		    /* look for first matching instance name */
-		    for (j = 0; j < sts; j++) {
-			if (eqinst(m->inames[i], inames[j])) {
-			    m->iids[i] = iids[j];
-			    m->m_idom++;
-			    break;
-			}
-		    }
-		    if (j == sts) {
-			m->iids[i] = PM_IN_NULL;
 			ret = 0;
 		    }
-		}
-		if (sts > 0) {
-		    /*
-		     * pmGetInDom or pmGetInDomArchive returned some
-		     * instances above
-		     */
-		    free(iids);
-		}
-
-		/* 
-		 * if specinst != m_idom, then some not found ... move these
-		 * to the end of the list
-		 */
-		for (j = m->specinst-1; j >= 0; j--) {
-		    if (m->iids[j] != PM_IN_NULL)
-			break;
-		}
-		for (i = 0; i < j; i++) {
-		    if (m->iids[i] == PM_IN_NULL) {
-			/* need to swap */
-			char	*tp;
-			tp = m->inames[i];
-			m->inames[i] = m->inames[j];
-			m->iids[i] = m->iids[j];
-			m->inames[j] = tp;
-			m->iids[j] = PM_IN_NULL;
-			j--;
-		    }
-		}
-	    }
-
-	    if (pmDebugOptions.appl1) {
-		int	numinst;
-		fprintf(stderr, "reinitMetric: %s from %s: instance domain specinst=%d\n",
-			mname, hname, m->specinst);
-		if (m->m_idom < 1) fprintf(stderr, "  %d instances!\n", m->m_idom);
-		if (m->specinst == 0) numinst = m->m_idom;
-		else numinst = m->specinst;
-		for (i = 0; i < numinst; i++) {
-		    fprintf(stderr, "  indom[%d]", i);
-		    if (m->iids[i] == PM_IN_NULL) 
-			fprintf(stderr, " ?missing");
 		    else
 			fprintf(stderr, " %d", m->iids[i]);
 		    fprintf(stderr, " \"%s\"\n", m->inames[i]);
 		}
 	    }
-	    if (sts > 0) {
-		/*
-		 * pmGetInDom or pmGetInDomArchive returned some instances
-		 * above
-		 */
-		free(inames);
-	    }
+	    /*
+	     * pmGetInDom or pmGetInDomArchive returned some instances
+	     * above
+	     */
+	    free(inames);
+	    if (m->specinst != 0)
+		free(iids);
 	}
     }
 
@@ -949,17 +842,17 @@ reinitMetric(Metric *m)
 		m->vals[j] = 0;
 	}
     }
-
-    if (ret >= 0) {
+    if (ret == 1 && reinit) {
 	/*
-	 * re-shape, starting here are working up the expression until
+	 * reshape, starting here are working up the expression until
 	 * we reach the top of the tree or the designated metrics
 	 * associated with the node are not the same
 	 */
 	Expr	*x = m->expr;
+	int	reshape = 0;
 	while (x) {
 	    /*
-	     * only re-shape expressions that may have set values
+	     * only reshape expressions that may have set values
 	     */
 	    if (x->op == CND_FETCH ||
 		x->op == CND_NEG || x->op == CND_ADD || x->op == CND_SUB ||
@@ -978,19 +871,40 @@ reinitMetric(Metric *m)
 		x->op == CND_NOT || x->op == CND_AND || x->op == CND_OR ||
 		x->op == CND_RISE || x->op == CND_FALL || x->op == CND_INSTANT ||
 		x->op == CND_MATCH || x->op == CND_NOMATCH) {
+		reshape++;
 		instFetchExpr(x);
 		findEval(x);
 		if (pmDebugOptions.appl1) {
-		    fprintf(stderr, "reinitMetric: re-shaped ...\n");
+		    fprintf(stderr, "reinitMetric: reshaped ...\n");
 		    dumpExpr(x);
 		}
 	    }
 	    if (x->parent) {
 		x = x->parent;
-		if (x->metrics == m)
-		    continue;
+		/*
+		 * used to stop if x->metrics != m, but this is wrong
+		 * when the same metric is used as the left and right
+		 * operator (with different instance specifiers), e.g.
+		 * all_inst(foo == foo #'magic') ...
+		 */
+		;
+		/* 
+		 * if operand is a set -> scalar function, like
+		 * CND_COUNT_INST, don't propagate instance reshaping
+		 * further up the tree
+		 */
+		if (isScalarResult(x))
+		    break;
 	    }
-	    break;
+	    else
+		break;		/* x is root of expression tree */
+	}
+	if (reshape && pmDebugOptions.appl1 && pmDebugOptions.desperate) {
+	    x = m->expr;
+	    while (x->parent)
+		x = x->parent;
+	    fprintf(stderr, "reinitMetric: enclosing tree after reshaping\n");
+	    dumpTree(x);
 	}
     }
 
@@ -998,9 +912,26 @@ end:
     /* destroy temporary context */
     pmDestroyContext(handle);
 
+    /* retry not meaningful for archives */
+    if (archives && (ret == 0))
+	ret = -1;
+
     return ret;
 }
 
+/* initialize Metric */
+int      /* 1: ok, 0: try again later, -1: fail */
+initMetric(Metric *m)
+{
+    return init(m, 0);
+}
+
+/* reinitialize Metric - only for live host */
+int      /* 1: ok, 0: try again later, -1: fail */
+reinitMetric(Metric *m)
+{
+    return init(m, 1);
+}
 
 /* put initialised Metric onto fetch list */
 void
