@@ -1,4 +1,4 @@
-#! /bin/sh
+#!/bin/sh
 #
 # Copyright (c) 2013-2016,2018,2020 Red Hat.
 # Copyright (c) 1995-2000,2003 Silicon Graphics, Inc.  All Rights Reserved.
@@ -162,6 +162,65 @@ _timespec()
     done
 }
 
+# replacement for fmt(1) that is the same on every platform ... mimics
+# the FSF version with a maximum line length of 75 columns
+#
+_fmt()
+{
+    $PCP_AWK_PROG '
+BEGIN		{ len = 0 }
+		{ for (i = 1; i <= NF; i++) {
+		    wlen = length($i)
+		    if (len + 1 + wlen > 75) {
+			printf "\n"
+			len = 0
+		    }
+		    if (len + 1 + wlen <= 75) {
+			if (len == 0) {
+			    printf "%s",$i
+			    len = wlen;
+			}
+			else {
+			    printf " %s",$i
+			    len += 1 + wlen
+			}
+		    }
+		  }
+		}
+END		{ if (len > 0) printf "\n" }'
+}
+
+# check callback
+# shell command is $1 possibly surrounded by quotes, so unpick
+# "executable program" (first word) and if it exists and is executable
+# append shell command to $2
+#
+_add_callback()
+{
+    case "$1"
+    in
+	\"*\")
+		_shell="`echo "$1" | sed -e 's/^"//' -e 's/"$//'`"
+		;;
+	\'*\')
+		_shell="`echo "$1" | sed -e "s/^'//" -e "s/'$//"`"
+		;;
+	*)
+		_shell="$1"
+		;;
+    esac
+    _prog=`echo "$_shell" | sed -e 's/ .*//'`
+
+    if which "$_prog" >/dev/null 2>&1
+    then
+	echo "$_shell" >>"$2"
+	return 0
+    else
+	_warning "$_prog: not a executable on PATH=$PATH"
+	return 1
+    fi
+}
+
 if is_chkconfig_on pmlogger
 then
     PMLOGGER_CTL=on
@@ -258,6 +317,7 @@ echo > $tmp/usage
 cat >> $tmp/usage <<EOF
 Options:
   -c=FILE,--control=FILE  pmlogger control file
+  -D,--noreport           do not run pmlogger_daily_report
   -E,--expunge            expunge metrics with metadata inconsistencies when merging archives
   -f,--force              force actions (intended for QA, not production)
   -k=TIME,--discard=TIME  remove archives after TIME (format DD[:HH[:MM]])
@@ -275,7 +335,9 @@ Options:
   -V,--verbose            verbose output (multiple times for very verbose)
   -x=TIME,--compress-after=TIME  compress archive data files after TIME (format DD[:HH[:MM]])
   -X=PROGRAM,--compressor=PROGRAM  use PROGRAM for archive data file compression
-  -Y=REGEX,--regex=REGEX  egrep filter when compressing files ["$COMPRESSREGEX_DEFAULT"]
+  -Y=REGEX,--regex=REGEX  grep -E filter when compressing files ["$COMPRESSREGEX_DEFAULT"]
+  -Z                      QA mode, force pmlogger re-exec
+  -z                      QA mode, do not re-exec pmlogger
   --help
 EOF
 
@@ -295,6 +357,7 @@ MYARGS=""
 COMPRESSONLY=false
 OFLAG=false
 PFLAG=false
+REEXEC_MODE=normal
 TRACE=0
 RFLAG=false
 REWRITEALL=false
@@ -302,6 +365,7 @@ MFLAG=false
 EXPUNGE=""
 FORCE=false
 KILL=pmsignal
+DO_DAILY_REPORT=true
 
 ARGS=`pmgetopt --progname=$prog --config=$tmp/usage -- "$@"`
 [ $? != 0 ] && exit 1
@@ -315,6 +379,8 @@ do
 		CONTROLDIR="$2.d"
 		shift
 		;;
+	-D)	DO_DAILY_REPORT=false
+		;;
 	-E)	EXPUNGE="-E"
 		;;
 	-f)	FORCE=true
@@ -449,6 +515,10 @@ do
 		    continue
 		fi
 		;;
+	-Z)	REEXEC_MODE=force
+		;;
+	-z)	REEXEC_MODE=skip
+		;;
 	--)	shift
 		break
 		;;
@@ -460,6 +530,87 @@ done
 
 [ $# -ne 0 ] && _usage
 
+if $SHOWME
+then
+    :
+else
+    # Salt away previous log, if any ...
+    #
+    _save_prev_file "$PROGLOG"
+    # After argument checking, everything must be logged to ensure no mail is
+    # accidentally sent from cron.  Close stdout and stderr, then open stdout
+    # as our logfile and redirect stderr there too.  Create the log file with
+    # correct ownership first.
+    #
+    # Exception ($SHOWME, above) is for -N where we want to see the output.
+    #
+    touch "$MYPROGLOG"
+    chown $PCP_USER:$PCP_GROUP "$MYPROGLOG" >/dev/null 2>&1
+    exec 1>"$MYPROGLOG" 2>&1
+fi
+
+if $COMPRESSONLY
+then
+    # no merge callbacks or autosave with -K
+    :
+else
+    # merge callback initialization ...
+    # pmlogger_daily_report goes first, then
+    # values (script names) set in the environment, then
+    # (later) any values (script names) set in the control files
+    #
+    touch $tmp/merge_callback
+    if $DO_DAILY_REPORT && [ -x "$PCP_BINADM_DIR/pmlogger_daily_report" ]
+    then
+	# pmlogger_daily_report script is present, need to see if the
+	# "zeroconf" pmlogconf(1) files are installed (the report needs
+	# metrics enabled by these files to be logged by the primary
+	# pmlogger) ... pick the "disk" one as a representative
+	#
+	if [ -f $PCP_VAR_DIR/config/pmlogconf/zeroconf/disk ]
+	then
+
+	    # Note: actual archive name will follow -a when callback
+	    # happens
+	    #
+	    if _add_callback "$PCP_BINADM_DIR/pmlogger_daily_report -o $PCP_SA_DIR -a" $tmp/merge_callback
+	    then
+		$VERBOSE && echo "Add merge callback for: pmlogger_daily_report"
+	    fi
+	fi
+    fi
+    if [ -n "$PCP_MERGE_CALLBACK" ]
+    then
+	if _add_callback "$PCP_MERGE_CALLBACK" $tmp/merge_callback
+	then
+	    $VERBOSE && echo "Add merge callback from environment: $PCP_MERGE_CALLBACK"
+	fi
+    fi
+
+    # autosave initialization ...
+    #
+    touch $tmp/autosave
+    touch $tmp/savefiles
+    if [ -n "$PCP_AUTOSAVE_DIR" ]
+    then
+	echo "$PCP_AUTOSAVE_DIR" >$tmp/autosave
+	$VERBOSE && echo "Using \$PCP_AUTOSAVE_DIR from environment: $PCP_AUTOSAVE_DIR"
+    fi
+fi
+
+# compress callback initialization ...
+# values (script names) set in the environment first, then
+# (later) any values (script names) set in the control files
+#
+touch $tmp/compress_callback
+if [ -n "$PCP_COMPRESS_CALLBACK" ]
+then
+    if _add_callback "$PCP_COMPRESS_CALLBACK" $tmp/compress_callback
+    then
+	$VERBOSE && echo "Add compress callback from environment: $PCP_COMPRESS_CALLBACK"
+    fi
+fi
+
 if $PFLAG
 then
     rm -f $tmp/ok
@@ -529,25 +680,6 @@ else
     fi
 fi
 
-if $SHOWME
-then
-    :
-else
-    # Salt away previous log, if any ...
-    #
-    _save_prev_file "$PROGLOG"
-    # After argument checking, everything must be logged to ensure no mail is
-    # accidentally sent from cron.  Close stdout and stderr, then open stdout
-    # as our logfile and redirect stderr there too.  Create the log file with
-    # correct ownership first.
-    #
-    # Exception ($SHOWME, above) is for -N where we want to see the output.
-    #
-    touch "$MYPROGLOG"
-    chown $PCP_USER:$PCP_GROUP "$MYPROGLOG" >/dev/null 2>&1
-    exec 1>"$MYPROGLOG" 2>&1
-fi
-
 if $VERY_VERBOSE
 then
     echo >&2 "Start: `date '+%F %T.%N'`"
@@ -565,25 +697,38 @@ then
     fi
 fi
 
-# if SaveLogs exists in the $PCP_LOG_DIR/pmlogger directory then save
-# $MYPROGLOG there as well with a unique name that contains the date and time
-# when we're run ... skip if -N (showme)
+# if SaveLogs exists in the $PCP_LOG_DIR/pmlogger directory and is writeable
+# then save $MYPROGLOG there as well with a unique name that contains the date
+# and time when we're run ... skip if -N (showme)
 #
-if $SHOWME
+if [ -d $PCP_LOG_DIR/pmlogger/SaveLogs -a -w $PCP_LOG_DIR/pmlogger/SaveLogs ]
 then
-    :
-else
-    if [ -d $PCP_LOG_DIR/pmlogger/SaveLogs ]
+    now="`date '+%Y%m%d.%H:%M:%S.%N'`"
+    link=`echo $MYPROGLOG | sed -e "s@.*$prog@$PCP_LOG_DIR/pmlogger/SaveLogs/$now-$prog@"`
+    if [ ! -f "$link" ]
     then
-	now="`date '+%Y%m%d.%H.%M.%S'`"
-	link=`echo $MYPROGLOG | sed -e "s/$prog/SaveLogs\/$prog.$now/"`
-	if [ ! -f "$link" ]
+	if $SHOWME
 	then
-	    if $SHOWME
+	    echo "+ ln $MYPROGLOG $link"
+	else
+	    ln $MYPROGLOG $link
+	    if [ -w $link ]
 	    then
-		echo "+ ln $MYPROGLOG $link"
-	    else
-		ln $MYPROGLOG $link
+		echo "--- Added by $prog when SaveLogs dir found ---" >>$link
+		echo "Start: `date '+%F %T.%N'`" >>$link
+		echo "Args: $ARGS" >>$link
+		if which pstree >/dev/null 2>&1
+		then
+		    if pstree -spa $$ >$tmp/tmp 2>&1
+		    then
+			echo "Called from:" >>$link
+			cat $tmp/tmp >>$link
+		    else
+			# pstree not functional for us ... -s not supported
+			# in older versions
+			:
+		    fi
+		fi
 	    fi
 	fi
     fi
@@ -1016,6 +1161,41 @@ s/^\([A-Za-z][A-Za-z0-9_]*\)=/export \1; \1=/p
 			    fi
 			    ;;
 
+			'export PCP_MERGE_CALLBACK;'*)
+			    if ! $COMPRESSONLY
+			    then
+				$SHOWME && echo "+ $cmd"
+				script="`echo "$cmd" | sed -e 's/.*BACK; PCP_MERGE_CALLBACK=//'`"
+				if _add_callback "$script" $tmp/merge_callback
+				then
+				    $VERBOSE && echo "Add merge callback: $script"
+				fi
+			    fi
+			    ;;
+
+			'export PCP_COMPRESS_CALLBACK;'*)
+			    $SHOWME && echo "+ $cmd"
+			    script="`echo "$cmd" | sed -e 's/.*BACK; PCP_COMPRESS_CALLBACK=//'`"
+			    if _add_callback "$script" $tmp/compress_callback
+			    then
+				$VERBOSE && echo "Add compress callback: $script"
+			    fi
+			    ;;
+
+			'export PCP_AUTOSAVE_DIR;'*)
+			    if ! $COMPRESSONLY
+			    then
+				$SHOWME && echo "+ $cmd"
+				eval $cmd
+				if [ -s $tmp/autosave ]
+				then
+				    _warning "\$PCP_AUTOSAVE_DIR ($PCP_AUTOSAVE_DIR) reset from control file, previous value (`cat $tmp/autosave`) ignored"
+				fi
+				echo "$PCP_AUTOSAVE_DIR" >$tmp/autosave
+				$VERBOSE && echo "Using \$PCP_AUTOSAVE_DIR: $PCP_AUTOSAVE_DIR"
+			    fi
+			    ;;
+
 			*)
 			    $SHOWME && echo "+ $cmd"
 			    echo eval $cmd >>$tmp/cmd
@@ -1171,7 +1351,7 @@ s/^\([A-Za-z][A-Za-z0-9_]*\)=/export \1; \1=/p
 		then
 		    echo >&2 "primary pmlogger process PID not found"
 		    ls >&2 -l "$PCP_TMP_DIR/pmlogger"
-		    $PCP_PS_PROG $PCP_PS_ALL_FLAGS | egrep >&2 '[P]ID|[p]mlogger'
+		    $PCP_PS_PROG $PCP_PS_ALL_FLAGS | grep -E >&2 '[P]ID|[p]mlogger'
 		fi
 	    elif _get_pids_by_name pmlogger | grep "^$pid\$" >/dev/null
 	    then
@@ -1206,32 +1386,64 @@ s/^\([A-Za-z][A-Za-z0-9_]*\)=/export \1; \1=/p
 	    _warning "skipping log rotation because we don't know which pmlogger to signal"
 	elif ! $COMPRESSONLY
 	then
-	    # send pmlogger a SIGUSR2 to "roll the archive logs"
-	    #
-	    if $SHOWME
+	    touch $tmp/reexec
+	    if [ "$REEXEC_MODE" = force ]
 	    then
-		echo "+ $KILL -s USR2 $pid"
-	    else
-		$VERY_VERBOSE && echo >&2 "Sending SIGUSR2 to $pid"
-		$KILL -s USR2 "$pid"
-		if $VERY_VERBOSE
+		# -Z, force reexec
+		:
+	    elif [ "$REEXEC_MODE" = skip ]
+	    then
+		# -z, skip reexec
+		rm -f $tmp/reexec
+	    elif [ -f "$PCP_TMP_DIR/pmlogger/$pid" ]
+	    then
+		# if pmlogger is already creating an archive with today's
+		# date then "reexec" has already been done, so we do not need
+		# to do it again ... this can happen when systemd decides
+		# to run pmlogger_daily at pmlogger service start, rather
+		# than just at 00:10 (by default) as originally intended
+		# and implemented in the cron-driven approach
+		#
+		current_archive=`sed -n -e '3s;.*/;;p' <"$PCP_TMP_DIR/pmlogger/$pid"`
+		case "$current_archive"
+		in
+		    `pmdate %Y%m%d`*)
+				$VERBOSE && echo "Skip sending SIGUSR2 to $pid (reexec already done)"
+				rm -f $tmp/reexec
+				;;
+		esac
+	    fi
+	    if [ -f $tmp/reexec ]
+	    then
+		#
+		# send pmlogger a SIGUSR2 to "roll the archive logs"
+		#
+		if $SHOWME
 		then
-		    # We have seen in qa/793, but never in a "real"
-		    # deployment, cases where the pmlogger process
-		    # vanishes a short time after the SIGUSR2 has been
-		    # sent and caught, the process has called exec() and
-		    # main() has restarted.  This check is intended
-		    # detect and report when this happens if -VV is
-		    # in play.
-		    #
-		    sleep 1
-		    if $PCP_PS_PROG -p "$pid" 2>&1 | grep "^$pid[ 	]" >/dev/null
+		    echo "+ $KILL -s USR2 $pid"
+		else
+		    $VERBOSE && echo "Sending SIGUSR2 to reexec $pid"
+		    $KILL -s USR2 "$pid"
+		    if $VERY_VERBOSE
 		    then
-			: still alive
-		    else
-			echo >&2 "Error: pmlogger process $pid has vanished"
+			# We have seen in qa/793, but never in a "real"
+			# deployment, cases where the pmlogger process
+			# vanishes a short time after the SIGUSR2 has been
+			# sent and caught, the process has called exec() and
+			# main() has restarted.  This check is intended
+			# detect and report when this happens if -VV is
+			# in play.
+			#
+			sleep 1
+			if $PCP_PS_PROG -p "$pid" 2>&1 | grep "^$pid[ 	]" >/dev/null
+			then
+			    : still alive
+			else
+			    echo >&2 "Error: pmlogger process $pid has vanished"
+			fi
 		    fi
 		fi
+		rm -f $tmp/reexec
 	    fi
 	fi
 
@@ -1297,7 +1509,7 @@ s/^\([A-Za-z][A-Za-z0-9_]*\)=/export \1; \1=/p
 		    if $VERBOSE
 		    then
 			echo "Archive files older than `_timespec $CULLAFTER` being removed ..."
-			fmt <$tmp/list | sed -e 's/^/    /'
+			_fmt <$tmp/list | sed -e 's/^/    /'
 		    fi
 		    if $SHOWME
 		    then
@@ -1430,6 +1642,7 @@ END	{ if (inlist != "") print lastdate,inlist }' >$tmp/list
 				done
 			    fi
 			    narch=`echo $inlist | wc -w | sed -e 's/ //g'`
+			    rm -f $tmp/mergefile
 			    if [ "$narch" = 1 ]
 			    then
 				# optimization - rename, don't merge, for one input archive
@@ -1437,6 +1650,7 @@ END	{ if (inlist != "") print lastdate,inlist }' >$tmp/list
 				if $SHOWME
 				then
 				    echo "+ pmlogmv$MYARGS $inlist $outfile"
+				    echo "$outfile" >$tmp/mergefile
 				elif pmlogmv$MYARGS $inlist $outfile
 				then
 				    if $VERY_VERBOSE
@@ -1444,6 +1658,7 @@ END	{ if (inlist != "") print lastdate,inlist }' >$tmp/list
 					echo >&2 "Renamed output archive $outfile ..."
 					pmdumplog >&2 -L $outfile
 				    fi
+				    echo "$outfile" >$tmp/mergefile
 				else
 				    _error "problems executing pmlogmv for host \"$host\""
 				fi
@@ -1453,6 +1668,7 @@ END	{ if (inlist != "") print lastdate,inlist }' >$tmp/list
 				if $SHOWME
 				then
 				    echo "+ pmlogger_merge$MYARGS $EXPUNGE -f $inlist $outfile"
+				    echo "$outfile" >$tmp/mergefile
 				elif pmlogger_merge$MYARGS $EXPUNGE -f $inlist $outfile
 				then
 				    if $VERY_VERBOSE
@@ -1460,10 +1676,51 @@ END	{ if (inlist != "") print lastdate,inlist }' >$tmp/list
 					echo >&2 "Merged output archive $outfile ..."
 					pmdumplog >&2 -L $outfile
 				    fi
+				    echo "$outfile" >$tmp/mergefile
 				else
 				    _error "problems executing pmlogger_merge for host \"$host\""
 				fi
 			    fi
+			    if [ -s $tmp/mergefile -a -s $tmp/merge_callback ]
+			    then
+				# Do merge callbacks
+				#
+				$VERBOSE && echo "Merge callbacks ..."
+				mergefile="$dir/`cat $tmp/mergefile`"
+				cat $tmp/merge_callback \
+				| while read exec
+				do
+				    case "$exec"
+				    in
+					$PCP_BINADM_DIR/pmlogger_daily_report)
+					    # only do this one for the primary
+					    # pmlogger instance
+					    #
+					    [ "$primary" != y ] && continue
+					    ;;
+				    esac
+				    if $SHOWME
+				    then
+					echo "+ $exec $mergefile"
+				    else
+					$VERBOSE && echo "callback: $exec $mergefile"
+					eval $exec $mergefile
+					sts=$?
+					if [ "$sts" = 0 ]
+					then
+					    :
+					else
+					    _warning "$exec $mergefile returned status $sts"
+					fi
+				    fi
+				done
+			    fi
+			    if [ -s $tmp/mergefile -a -s $tmp/autosave ]
+			    then
+				# save archive basename for possible AUTOSAVE use later
+				#
+				cat $tmp/mergefile >>$tmp/savefiles
+			    fi
 			fi
 		    done
 		fi
@@ -1561,7 +1818,7 @@ p
 			find . -type f -mtime +$mtime
 		    fi \
 		    | _filter_filename \
-		    | egrep -v "$COMPRESSREGEX" \
+		    | grep -E -v "$COMPRESSREGEX" \
 		    | sort >$tmp/list
 		    if [ -s $tmp/list -a -n "$current_base" -a -n "$current_vol" ]
 		    then
@@ -1579,7 +1836,7 @@ p
 			# ...DDMM.HH.MM-seq# variants to get the base name
 			# separated from the other part of the file name, but
 			# on the upside compressed file names were stripped out
-			# above by the egrep -v "$COMPRESSREGEX"
+			# above by the grep -E -v "$COMPRESSREGEX"
 			#
 			sed -n <$tmp/list \
 			    -e '/\./s/\.\([^.][^.]*\)$/ \1/p' \
@@ -1616,7 +1873,7 @@ p
 			    else
 				echo "Archive files older than $COMPRESSAFTER days being compressed ..."
 			    fi
-			    fmt <$tmp/list | sed -e 's/^/    /'
+			    _fmt <$tmp/list | sed -e 's/^/    /'
 			fi
 			if $SHOWME
 			then
@@ -1624,6 +1881,44 @@ p
 			else
 			    cat $tmp/list | xargs $COMPRESS
 			fi
+			if [ -s $tmp/compress_callback ]
+			then
+			    # Do compress callbacks, but only for full
+			    # day's archives
+			    #
+			    $VERBOSE && echo "Compress callbacks ..."
+			    cat $tmp/list \
+			    | sed -n \
+				-e 's/\.[0-9][0-9]*$//' \
+				-e 's/\.meta$//' \
+				-e 's/\.index$//' \
+				-e '/^[12][0-9][0-9][0-9][0-1][0-9][0-3][0-9]$/p' \
+				-e '/^[0-9][0-9][0-1][0-9][0-3][0-9]$/p' \
+			    | sort -n \
+			    | uniq \
+			    | while read compressfile
+			    do
+				compressfile="$dir/$compressfile"
+				cat $tmp/compress_callback \
+				| while read exec
+				do
+				    if $SHOWME
+				    then
+					echo "+ $exec $compressfile"
+				    else
+					$VERBOSE && echo "callback: $exec $compressfile"
+					eval $exec $compressfile
+					sts=$?
+					if [ "$sts" = 0 ]
+					then
+					    :
+					else
+					    _warning "$exec $compressfile returned status $sts"
+					fi
+				    fi
+				done
+			    done
+			fi
 		    else
 			$VERY_VERBOSE && echo >&2 "Warning: no archive files found to compress"
 		    fi
@@ -1639,6 +1934,50 @@ p
 	    fi
 	fi
 
+	# autosave any newly merged (and possibly compressed) archives
+	# if PCP_AUTOSAVE_DIR is in play
+	#
+	if [ -s $tmp/autosave -a -s $tmp/savefiles ]
+	then
+	    if $VERBOSE
+	    then
+		( echo "Autosave ... "; _fmt <$tmp/savefiles ) \
+		| sed -e 's/^/    /'
+	    fi
+	    last_mkdir=''
+	    cat $tmp/savefiles \
+	    | while read savefile
+	    do
+		# make sure destination directory hierarchy exists and $PCP_USER
+		# user can write there
+		#
+		DATEYYYY=`echo "$savefile" | sed -e 's/^\(....\).*/\1/'`
+		DATEMM=`echo "$savefile" | sed -e 's/^....\(..\).*/\1/'`
+		DATEDD=`echo "$savefile" | sed -e 's/^......\(..\)/\1/'`
+		auto_dir="`cat $tmp/autosave \
+			   | sed \
+			       -e s/DATEYYYY/$DATEYYYY/g \
+			       -e s/DATEMM/$DATEMM/g \
+			       -e s/DATEDD/$DATEDD/g \
+			       -e s/LOCALHOSTNAME/$dirhostname/`"
+		if [ ! -d "$auto_dir" -a "$auto_dir" != "$last_mkdir" ]
+		then
+		    mkdir_and_chown "$auto_dir" 755 $PCP_USER:$PCP_GROUP >$tmp/tmp 2>&1
+		    if [ ! -d "$auto_dir" ]
+		    then
+			cat $tmp/tmp
+			_error "cannot create directory ($auto_dir) for autosave"
+		    else
+			_warning "creating directory ($auto_dir) for autosave"
+			# fall through and another warning will come from
+			# _autosave()
+		    fi
+		    last_mkdir="$auto_dir"
+		fi
+		_autosave "$dir" "$savefile" "$auto_dir"
+	    done
+	fi
+
 	# and cull old trace files (from -t option)
 	#
 	if [ "$TRACE" -gt 0 ] && ! $COMPRESSONLY
@@ -1659,7 +1998,7 @@ p
 		if $VERBOSE
 		then
 		    echo "Trace files older than $TRACE days being removed ..."
-		    fmt <$tmp/list | sed -e 's/^/    /'
+		    _fmt <$tmp/list | sed -e 's/^/    /'
 		fi
 		if $SHOWME
 		then
@@ -1676,6 +2015,28 @@ p
     done
 }
 
+# Paranoid archive saving
+#
+# Usage: _autosave src_dir archive_base dest_dir
+#
+_autosave()
+{
+    _src="$1"
+    _base="$2"
+    _dest="$3"
+    if [ ! -d "$_dest" ]
+    then
+	_warning "$_base: AUTOSAVE skipped: directory $_dest does not exist"
+	return
+    fi
+    if pmlogmv -c "$_src/$_base" "$_dest/$_base"
+    then
+	:
+    else
+	_warning "$_base: AUTOSAVE failed"
+    fi
+}
+
 
 # .NeedRewrite in $PCP_LOG_DIR/pmlogger is just like -R, but needs
 # only be done once, e.g. after software upgrade with new pmlogrewrite(1)
