@@ -140,6 +140,114 @@ done
 
 [ $# -ne 0 ] && _usage
 
+# optionally uncompress a compressed archive
+#
+_uncompress()
+{
+    ls -d $1.* >$tmp/list 2>&1
+
+    rm -f $tmp/uncompress
+    for _suff in `pmconfig -L compress_suffixes | sed -e 's/^compress_suffixes=//'`
+    do
+	if grep "$1.*.$_suff\$" $tmp/list >/dev/null
+	then
+	    touch $tmp/uncompress
+	    break
+	fi
+    done
+
+    if [ -f $tmp/uncompress ]
+    then
+	# at least one of the archive files is compressed, copy and
+	# uncompress
+	#
+	$VERBOSE && echo >&2 "Uncompressing $1"
+	touch $tmp/ok
+	for _file in `cat $tmp/list`
+	do
+	    case "$_file"
+	    in
+		*.xz)
+			_dest=$tmp/`basename "$_file" .xz`
+			;;
+		*.lzma)
+			_dest=$tmp/`basename "$_file" .lzma`
+			;;
+		*.bz2)
+			_dest=$tmp/`basename "$_file" .bz2`
+			;;
+		*.bz)
+			_dest=$tmp/`basename "$_file" .bz`
+			;;
+		*.gz)
+			_dest=$tmp/`basename "$_file" .gz`
+			;;
+		*.Z)
+			_dest=$tmp/`basename "$_file" .Z`
+			;;
+		*.z)
+			_dest=$tmp/`basename "$_file" .z`
+			;;
+	    esac
+	    case "$_file"
+	    in
+		*.xz|*.lzma)
+			if xzcat "$_file" >"$_dest"
+			then
+			    :
+			else
+			    echo >&2 "Warning: xzcat $_file failed"
+			    rm -f $tmp/ok
+			    break
+			fi
+			;;
+		*.bz2|*.bz)
+			if bzcat "$_file" >"$_dest"
+			then
+			    :
+			else
+			    echo >&2 "Warning: bzcat $_file failed"
+			    rm -f $tmp/ok
+			    break
+			fi
+			;;
+		*.gz|*.Z|*.z)
+			if zcat "$_file" >"$_dest"
+			then
+			    :
+			else
+			    echo >&2 "Warning: zcat $_file failed"
+			    rm -f $tmp/ok
+			    break
+			fi
+			;;
+		*)      # not compressed
+			_dest=$tmp/`basename "$_file"`
+			if cp "$_file" "$_dest"
+			then
+			    :
+			else
+			    echo >&2 "Warning: cp $_file failed"
+			    rm -f $tmp/ok
+			    break
+			fi
+			;;
+	    esac
+	done
+	if [ -f $tmp/ok ]
+	then
+	    echo $tmp/`basename "$1"`
+	else
+	    $VERBOSE && echo >&2 "Uncompressing failed, reverting to compressed archive"
+	    echo "$1"
+	fi
+    else
+	# no compression, nothing to be done
+	#
+	echo "$1"
+    fi
+}
+
 if $PFLAG
 then
     rm -f $tmp/ok
@@ -179,23 +287,31 @@ then
     fi
 fi
 
-# write date-and-timestamp to be checked by -p polling
+# if it looks like we can create/update the date-and-timestamp
+# then do it ... this will be checked by -p polling
 #
-if _save_prev_file $PCP_LOG_DIR/pmlogger/pmlogger_daily_report.stamp
+if [ -w $PCP_LOG_DIR/pmlogger ]
 then
-    :
-else
-    echo "Warning: cannot save previous date-and-timestamp" >&2
-fi
-# only update date-and-timestamp if we can write the file
-#
-pmdate '# %Y-%m-%d %H:%M:%S
+    if [ ! -f $PCP_LOG_DIR/pmlogger/pmlogger_daily_report.stamp \
+           -o -w $PCP_LOG_DIR/pmlogger/pmlogger_daily_report.stamp ]
+    then
+	if _save_prev_file $PCP_LOG_DIR/pmlogger/pmlogger_daily_report.stamp
+	then
+	    :
+	else
+	    echo "Warning: cannot save previous date-and-timestamp" >&2
+	fi
+	# only update date-and-timestamp if we can write the file
+	#
+	pmdate '# %Y-%m-%d %H:%M:%S
 %s' >$tmp/stamp
-if cp $tmp/stamp $PCP_LOG_DIR/pmlogger/pmlogger_daily_report.stamp
-then
-    :
-else
-    echo "Warning: cannot install new date-and-timestamp" >&2
+	if cp $tmp/stamp $PCP_LOG_DIR/pmlogger/pmlogger_daily_report.stamp
+	then
+	    :
+	else
+	    echo "Warning: cannot install new date-and-timestamp" >&2
+	fi
+    fi
 fi
 
 # After argument checking, everything must be logged to ensure no mail is
@@ -218,6 +334,13 @@ exec 1>"$PROGLOG" 2>&1
 # take a long time to run as a result.
 #
 [ -z "$ARCHIVEPATH" ] && ARCHIVEPATH=$PCP_ARCHIVE_DIR/$HOSTNAME/`pmdate -1d %Y%m%d`
+
+# If input archive has been compressed, then uncompress it
+# into temporary files and use these, to avoid repeated
+# uncompressing for pmrep below.
+#
+ORIG_ARCHIVEPATH="$ARCHIVEPATH"
+ARCHIVEPATH=`_uncompress $ARCHIVEPATH`
 $VERBOSE && echo ARCHIVEPATH=$ARCHIVEPATH
 
 # Default output directory
@@ -241,9 +364,9 @@ $VERBOSE && echo REPORTFILE=$REPORTFILE
 
 # If the input archive doesn't exist, we exit.
 #
-if [ ! -f "$ARCHIVEPATH.index" -a ! -f "$ARCHIVEPATH.index.xz" ]; then
+if [ ! -f "`echo $ARCHIVEPATH.meta*`" ]; then
     # report this to the log
-    echo "$prog: FATAL error: Failed to find input archive \"$ARCHIVEPATH\"."
+    echo "$prog: FATAL error: Failed to find input archive \"$ARCHIVEPATH\""
     exit 1
 fi
 
@@ -268,7 +391,7 @@ then
 fi
 
 #
-# Common reporting funtion
+# Common reporting function
 #
 _report()
 {
@@ -280,14 +403,36 @@ _report()
     pmdumplog -z -l $ARCHIVEPATH | awk '/commencing/ {print "# ",$2,$3,$4,$5,$6}' >>$REPORTFILE
     echo $_comment >>$REPORTFILE
     $VERBOSE && echo pmrep $REPORT_OPTIONS $_conf
-    pmrep $REPORT_OPTIONS $_conf >$tmp/out
-    if [ -s $tmp/out ]; then
+    pmrep $REPORT_OPTIONS $_conf >$tmp/out 2>$tmp/err
+    if [ -s $tmp/out ]
+    then
     	cat $tmp/out >>$REPORTFILE
     else
-    	echo "-- no report for config \"$_conf\"" >>$REPORTFILE
+	if grep 'PM_ERR_NAME' $tmp/err >/dev/null 2>&1
+	then
+	    metric=`$PCP_AWK_PROG <$tmp/err '{ print $3 }'`
+	    echo "-- no report for config \"$_conf\" because the metric \"$metric\" is not in the archive" >>$REPORTFILE
+	elif grep 'PM_ERR_INDOM_LOG' $tmp/err >/dev/null 2>&1
+	then
+	    metric=`$PCP_AWK_PROG <$tmp/err '{ print $3 }'`
+	    echo "-- no report for config \"$_conf\" because there are no values for any instance of the metric \"$metric\" in the archive" >>$REPORTFILE
+	elif grep 'PM_ERR_BADDERIVE' $tmp/err >/dev/null 2>&1
+	then
+	    metric=`$PCP_AWK_PROG <$tmp/err '{ print $3 }'`
+	    echo "-- no report for config \"$_conf\" because one or more metrics for the derived metric \"$metric\" is not in the archive" >>$REPORTFILE
+	else
+	    cat $tmp/err >>$REPORTFILE
+	    echo "-- no report for config \"$_conf\"" >>$REPORTFILE
+	fi
     fi
 }
 
+echo "System Activity Report" >>$REPORTFILE
+echo >>$REPORTFILE
+echo "Host:            $HOSTNAME" >>$REPORTFILE
+echo "Archive:         $ORIG_ARCHIVEPATH" >>$REPORTFILE
+echo "Report created:  `date`" >>$REPORTFILE
+
 _report :sar-u-ALL '# CPU Utilization statistics, all CPUS'
 _report :sar-u-ALL-P-ALL '# CPU Utilization statistics, per-CPU'
 _report :vmstat '# virtual memory (vmstat) statistics'
@@ -315,8 +460,6 @@ _report :numa-hint-faults '# NUMA hint fault statistics'
 _report :numa-per-node-cpu '# NUMA per-node CPU statistics'
 _report :numa-pgmigrate-per-node '# NUMA per-node page migration statistics'
 
-[ -f $tmp/err ] && status=1
-
 # optional end logging to $PCP_LOG_DIR/NOTICES
 #
 if $PCP_LOG_RC_SCRIPTS
