@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013,2018,2021 Red Hat.
+ * Copyright (c) 2013,2018,2021-2022 Red Hat.
  * Copyright (c) 2004 Silicon Graphics, Inc.  All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
@@ -28,14 +28,18 @@ extern int	wflag;		/* -w from command line */
 typedef struct {
     int			flags;		/* GLOBAL_* flags */
     __pmTimestamp	time;		/* timestamp shift */
-    char		hostname[PM_LOG_MAXHOSTLEN];
-    char		tz[PM_TZ_MAXLEN]; 
+    char		*hostname;
+    char		*timezone;
+    char		*zoneinfo;
+    int			features;
 } global_t;
 
 /* values for global_t flags */
 #define GLOBAL_CHANGE_TIME	1
 #define GLOBAL_CHANGE_HOSTNAME	2
-#define GLOBAL_CHANGE_TZ	4
+#define GLOBAL_CHANGE_TIMEZONE	4
+#define GLOBAL_CHANGE_ZONEINFO	8
+#define GLOBAL_CHANGE_FEATURES	16
 
 extern global_t global;
 
@@ -65,6 +69,7 @@ typedef struct indomspec {
 #define INST_DELETE		64
 
 extern indomspec_t	*indom_root;
+extern __pmHashCtl	indom_hash;
 
 /*
  * Rewrite specifications for a metric
@@ -164,15 +169,15 @@ extern labelspec_t	*label_root;
  *  Input archive control
  */
 typedef struct {
-    int		ctx;
-    int		version;	/* input log version (2/3) */
-    __pmContext	*ctxp;
-    char	*name;
-    pmLogLabel	label;
-    __int32_t	*metarec;
-    __int32_t	*logrec;
-    pmResult	*rp;
-    int		mark;		/* need EOL marker */
+    int			ctx;
+    int			version;	/* input log version (2/3) */
+    __pmContext		*ctxp;
+    char		*name;
+    __pmLogLabel	label;
+    __int32_t		*metarec;
+    __int32_t		*logrec;
+    __pmResult		*rp;
+    int			mark;		/* need EOL marker */
 } inarch_t;
 
 extern inarch_t		inarch;		/* input archive */
@@ -211,11 +216,8 @@ extern int	yyparse(void);
 #define W_NEXT	2
 #define W_NONE	3
 
-extern int	_pmLogGet(__pmArchCtl *, int, __int32_t **);
-extern int	_pmLogPut(FILE *, __int32_t *);
 extern int	_pmLogRename(const char *, const char *);
 extern int	_pmLogRemove(const char *, int);
-extern pmUnits	ntoh_pmUnits(pmUnits);
 #define ntoh_pmInDom(indom) ntohl(indom)
 #define ntoh_pmID(pmid)     ntohl(pmid)
 
@@ -237,7 +239,7 @@ extern char	*dupcat(const char *, const char *);
 extern void	newvolume(int);
 
 extern void	do_desc(void);
-extern void	do_indom(void);
+extern void	do_indom(int);
 extern void	do_labelset(void);
 extern void	do_text(void);
 extern void	do_result(void);
