@@ -37,9 +37,9 @@ typedef struct {
     PmTime::State	state;
     QIcon 		*stop;
     QIcon 		*play;
-} IconStateMap;
+} LiveIconStateMap;
 
-static void setup(IconStateMap *map, PmTime::State state,
+static void setup(LiveIconStateMap *map, PmTime::State state,
 		  QIcon *stop, QIcon *play)
 {
     map->state = state;
@@ -49,7 +49,7 @@ static void setup(IconStateMap *map, PmTime::State state,
 
 void PmTimeLive::setControl(PmTime::State state)
 {
-    static IconStateMap maps[2];
+    static LiveIconStateMap maps[2];
     static int nmaps;
 
     if (!nmaps) {
@@ -205,15 +205,15 @@ void PmTimeLive::changeDelta(int value)
 
 void PmTimeLive::displayDeltaText()
 {
-    QString text;
+    char text[64];
     double delta = PmTime::secondsFromTimeval(&my.pmtime.delta);
 
     delta = PmTime::secondsToUnits(delta, my.units);
     if ((double)(int)delta == delta)
-	text.sprintf("%.2f", delta);
+	pmsprintf(text, sizeof(text), "%.2f", delta);
     else
-	text.sprintf("%.6f", delta);
-    lineEditDelta->setText(text);
+	pmsprintf(text, sizeof(text), "%.6f", delta);
+    lineEditDelta->setText(QString(text));
 }
 
 void PmTimeLive::disableConsole()
