@@ -1,6 +1,6 @@
 """Wrapper module for libpcp_pmda - Performace Co-Pilot Domain Agent API
 #
-# Copyright (C) 2013-2015,2017-2019 Red Hat.
+# Copyright (C) 2013-2015,2017-2021 Red Hat.
 #
 # This file is part of the "pcp" module, the python interfaces for the
 # Performance Co-Pilot toolkit.
@@ -17,7 +17,7 @@
 #
 # See pmdasimple.py for an example use of this module.
 """
-# pylint: disable=too-many-arguments
+# pylint: disable=too-many-arguments,consider-using-dict-items,no-member
 
 import os
 
@@ -61,6 +61,8 @@ LIBPCP_PMDA.pmdaAddLabels.restype = c_int
 LIBPCP_PMDA.pmdaAddLabels.argtypes = [POINTER(POINTER(pmLabelSet)), c_char_p]
 LIBPCP_PMDA.pmdaAddLabelFlags.restype = c_int
 LIBPCP_PMDA.pmdaAddLabelFlags.argtypes = [POINTER(pmLabelSet), c_int]
+LIBPCP_PMDA.pmdaAddNotes.restype = c_int
+LIBPCP_PMDA.pmdaAddNotes.argtypes = [POINTER(POINTER(pmLabelSet)), c_char_p]
 
 LIBPCP_PMDA.pmdaGetContext.restype = c_int
 LIBPCP_PMDA.pmdaGetContext.argtypes = []
@@ -78,6 +80,13 @@ def pmdaAddLabelFlags(labels, flags):
         raise pmErr(status)
     return status
 
+def pmdaAddNotes(label):
+    result_p = POINTER(pmLabelSet)()
+    status = LIBPCP_PMDA.pmdaAddNotes(byref(result_p), label)
+    if status < 0:
+        raise pmErr(status)
+    return result_p
+
 def pmdaGetContext():
     status = LIBPCP_PMDA.pmdaGetContext()
     if status < 0:
@@ -366,8 +375,9 @@ class MetricDispatch(object):
             replacement = pmdaIndom(it_indom, insts)
         # list indoms need to keep the table up-to-date for libpcp_pmda
         if isinstance(insts, list):
-            for i in range(len(self._indomtable)):  # _indomtable is persistently shared with pmda.c
-                if self._indomtable[i].it_indom == it_indom:
+            # _indomtable is persistently shared with pmda.c
+            for i, entry in enumerate(self._indomtable):
+                if entry.it_indom == it_indom:
                     self._indomtable[i] = replacement # replace in place
                     break
         self._indoms[it_indom] = replacement
@@ -508,6 +518,10 @@ class PMDA(MetricDispatch):
     def set_label(label):
         return cpmda.set_label(label)
 
+    @staticmethod
+    def set_notes(notes):
+        return cpmda.set_notes(notes)
+
     @staticmethod
     def set_refresh(refresh):
         return cpmda.set_refresh(refresh)
@@ -524,6 +538,10 @@ class PMDA(MetricDispatch):
     def set_label_callback(label_callback):
         return cpmda.set_label_callback(label_callback)
 
+    @staticmethod
+    def set_notes_callback(notes_callback):
+        return cpmda.set_notes_callback(notes_callback)
+
     @staticmethod
     def set_attribute_callback(attribute_callback):
         return cpmda.set_attribute_callback(attribute_callback)
