@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2015,2017-2019 Red Hat.
+ * Copyright (C) 2013-2015,2017-2021 Red Hat.
  *
  * This file is part of the "pcp" module, the python interfaces for the
  * Performance Co-Pilot toolkit.
@@ -20,11 +20,13 @@
  * and macros needed to implement PMDAs in python.  These are exported to
  * python PMDAs via the pmda.py module, using ctypes.
  */
-
+#define PY_SSIZE_T_CLEAN
 #include <Python.h>
-#include "pmapi.h"
-#include "pmda.h"
-#include "libpcp.h"
+#include <pcp/pmapi.h>
+#include <pcp/pmda.h>
+
+/* libpcp internal routines used by this module */
+PCP_CALL extern int __pmAddLabels(pmLabelSet **, const char *, int);
 
 #if PY_MAJOR_VERSION >= 3
 #define MOD_ERROR_VAL NULL
@@ -56,11 +58,13 @@ static PyObject *indom_longtext_dict;	/* indom pmid:long help */
 
 static PyObject *fetch_func;
 static PyObject *label_func;
+static PyObject *notes_func;
 static PyObject *refresh_func;
 static PyObject *instance_func;
 static PyObject *store_cb_func;
 static PyObject *fetch_cb_func;
 static PyObject *label_cb_func;
+static PyObject *notes_cb_func;
 static PyObject *attribute_cb_func;
 static PyObject *endcontext_cb_func;
 static PyObject *refresh_all_func;
@@ -100,7 +104,7 @@ maybe_refresh_all(void)
 	arglist = Py_BuildValue("()");
 	if (arglist == NULL)
 	    return;
-	result = PyEval_CallObject(refresh_metrics_func, arglist);
+	result = PyObject_Call(refresh_metrics_func, arglist, NULL);
 	Py_DECREF(arglist);
 	if (result == NULL)
 	    PyErr_Print();
@@ -371,7 +375,7 @@ prefetch(void)
     arglist = Py_BuildValue("()");
     if (arglist == NULL)
 	return -ENOMEM;
-    result = PyEval_CallObject(fetch_func, arglist);
+    result = PyObject_Call(fetch_func, arglist, NULL);
     Py_DECREF(arglist);
     if (result == NULL)
 	return callback_error("prefetch");
@@ -387,7 +391,7 @@ refresh_cluster(int cluster)
     arglist = Py_BuildValue("(i)", cluster);
     if (arglist == NULL)
 	return -ENOMEM;
-    result = PyEval_CallObject(refresh_func, arglist);
+    result = PyObject_Call(refresh_func, arglist, NULL);
     Py_DECREF(arglist);
     if (result == NULL)
 	return callback_error("refresh_cluster");
@@ -414,7 +418,7 @@ refresh_all_clusters(int numclusters, int *clusters)
     arglist = Py_BuildValue("(N)", list);
     if (arglist == NULL)
 	return -ENOMEM;
-    result = PyEval_CallObject(refresh_all_func, arglist);
+    result = PyObject_Call(refresh_all_func, arglist, NULL);
     Py_DECREF(list);
     Py_DECREF(arglist);
     if (result == NULL)
@@ -498,37 +502,65 @@ label(int ident, int type, pmLabelSet **lp, pmdaExt *ep)
     int id, sts = 0;
     char *s = NULL;
 
-    if (label_func) {
-	PyObject *arglist, *result;
+    if (label_func || notes_func) {
+	PyObject *arglist, *label_result, *notes_result;
 
 	id = (type == PM_LABEL_CLUSTER) ? (int)pmID_cluster(ident) : ident;
 
 	arglist = Py_BuildValue("(ii)", id, type);
 	if (arglist == NULL)
 	    return -ENOMEM;
-	result = PyEval_CallObject(label_func, arglist);
+	if (label_func)
+	   label_result = PyObject_Call(label_func, arglist, NULL);
+	else
+	   label_result = NULL;
+	if (notes_func)
+	   notes_result = PyObject_Call(notes_func, arglist, NULL);
+	else
+	   notes_result = NULL;
 	Py_DECREF(arglist);
 
-	if (!result) {
-	    PyErr_Print();
-	    return -EAGAIN;
+	if (label_func) {
+	    if (!label_result) {
+		PyErr_Print();
+		return -EAGAIN;
+	    }
+
+	    if (PyArg_Parse(label_result, "s:label", &s) == 0 || s == NULL) {
+		pmNotifyErr(LOG_ERR, "bad labels result (expected string)");
+		Py_DECREF(label_result);
+		return -EINVAL;
+	    }
+
+	    if (!empty_labelset(s) && (sts = __pmAddLabels(lp, s, type)) < 0)
+		pmNotifyErr(LOG_ERR, "__pmAddLabels failed: %s", pmErrStr(sts));
+
+	    Py_DECREF(label_result);
 	}
 
-	if (PyArg_Parse(result, "s:label", &s) == 0 || s == NULL) {
-	    pmNotifyErr(LOG_ERR, "label gave bad result (expected string)");
-	    Py_DECREF(result);
-	    return -EINVAL;
+	if (notes_func) {
+	    if (!notes_result) {
+		PyErr_Print();
+		return -EAGAIN;
+	    }
+
+	    if (PyArg_Parse(notes_result, "s:notes", &s) == 0 || s == NULL) {
+		pmNotifyErr(LOG_ERR, "bad notes result (expected string)");
+		Py_DECREF(notes_result);
+		return -EINVAL;
+	    }
+
+	    type |= PM_LABEL_OPTIONAL;
+	    if (!empty_labelset(s) && (sts = __pmAddLabels(lp, s, type)) < 0)
+		pmNotifyErr(LOG_ERR, "__pmAddLabels failed: %s", pmErrStr(sts));
+
+	    Py_DECREF(notes_result);
 	}
-
-	if (!empty_labelset(s) && (sts = __pmAddLabels(lp, s, type)) < 0)
-	    pmNotifyErr(LOG_ERR, "__pmAddLabels failed: %s", pmErrStr(sts));
-
-	Py_DECREF(result);
-
-	if (sts < 0)
-   	    return sts;
     }
 
+    if (sts < 0)
+	return sts;
+
     return pmdaLabel(ident, type, lp, ep);
 }
 
@@ -540,7 +572,7 @@ preinstance(pmInDom indom)
     arglist = Py_BuildValue("(i)", pmInDom_serial(indom));
     if (arglist == NULL)
 	return -ENOMEM;
-    result = PyEval_CallObject(instance_func, arglist);
+    result = PyObject_Call(instance_func, arglist, NULL);
     Py_DECREF(arglist);
     if (result == NULL)
 	return callback_error("preinstance");
@@ -548,7 +580,7 @@ preinstance(pmInDom indom)
     return 0;
 }
 
-int
+static int
 instance(pmInDom indom, int a, char *b, pmInResult **rp, pmdaExt *pmda)
 {
     int sts;
@@ -559,23 +591,24 @@ instance(pmInDom indom, int a, char *b, pmInResult **rp, pmdaExt *pmda)
     return pmdaInstance(indom, a, b, rp, pmda);
 }
 
-int
+static int
 fetch_callback(pmdaMetric *metric, unsigned int inst, pmAtomValue *atom)
 {
     char *s;
     int rc, sts, code;
     PyObject *arglist, *result;
-    __pmID_int *pmid = (__pmID_int *)&metric->m_desc.pmid;
+    unsigned int item = pmID_item(metric->m_desc.pmid);
+    unsigned int cluster = pmID_cluster(metric->m_desc.pmid);
 
     if (fetch_cb_func == NULL)
 	return PM_ERR_VALUE;
 
-    arglist = Py_BuildValue("(iiI)", pmid->cluster, pmid->item, inst);
+    arglist = Py_BuildValue("(iiI)", cluster, item, inst);
     if (arglist == NULL) {
 	pmNotifyErr(LOG_ERR, "fetch callback cannot alloc parameters");
 	return -EINVAL;
     }
-    result = PyEval_CallObject(fetch_cb_func, arglist);
+    result = PyObject_Call(fetch_cb_func, arglist, NULL);
     Py_DECREF(arglist);
     if (result == NULL)
 	return callback_error("fetch_callback");
@@ -651,47 +684,76 @@ fetch_callback(pmdaMetric *metric, unsigned int inst, pmAtomValue *atom)
     return sts;
 }
 
-int
+static int
 label_callback(pmInDom indom, unsigned int inst, pmLabelSet **lp)
 {
     int sts = 0;
     int type = PM_LABEL_INSTANCES;
     char *s = NULL;
-    PyObject *arglist, *result;
+    PyObject *arglist, *label_result, *notes_result;
 
-    if (label_cb_func == NULL)
+    if (label_cb_func == NULL && notes_cb_func == NULL)
 	return PM_ERR_VALUE;
 
     arglist = Py_BuildValue("(II)", indom, inst);
     if (arglist == NULL) {
-	pmNotifyErr(LOG_ERR, "fetch callback cannot alloc parameters");
+	pmNotifyErr(LOG_ERR, "label callback cannot alloc parameters");
 	return -EINVAL;
     }
-    result = PyEval_CallObject(label_cb_func, arglist);
+    if (label_cb_func)
+	label_result = PyObject_Call(label_cb_func, arglist, NULL);
+    else
+	label_result = NULL;
+    if (notes_cb_func)
+	notes_result = PyObject_Call(notes_cb_func, arglist, NULL);
+    else
+	notes_result = NULL;
     Py_DECREF(arglist);
-    if (result == NULL) {
-	PyErr_Print();
-	return -EAGAIN; /* exception thrown */
-    }
-    if (PyArg_Parse(result, "s:label_callback", &s) == 0 || s == NULL) {
-	pmNotifyErr(LOG_ERR, "label callback gave bad result (expected string)");
-	Py_DECREF(result);
-	return -EINVAL;
+
+    if (label_cb_func) {
+	if (label_result == NULL) {
+	    PyErr_Print();
+	    return -EAGAIN; /* exception thrown */
+	}
+	if (PyArg_Parse(label_result, "s:label_callback", &s) == 0 || !s) {
+	    pmNotifyErr(LOG_ERR, "bad label callback result (expected string)");
+	    Py_DECREF(label_result);
+	    return -EINVAL;
+	}
+
+	if (!empty_labelset(s) && (sts = __pmAddLabels(lp, s, type)) < 0)
+	    pmNotifyErr(LOG_ERR, "__pmAddLabels failed: %s", pmErrStr(sts));
+
+	Py_DECREF(label_result);
     }
 
-    if (!empty_labelset(s) && (sts = __pmAddLabels(lp, s, type)) < 0)
-	pmNotifyErr(LOG_ERR, "__pmAddLabels failed: %s", pmErrStr(sts));
+    if (sts >= 0 && notes_cb_func) {
+	if (notes_result == NULL) {
+	    PyErr_Print();
+	    return -EAGAIN; /* exception thrown */
+	}
+	if (PyArg_Parse(notes_result, "s:notes_callback", &s) == 0 || !s) {
+	    pmNotifyErr(LOG_ERR, "bad notes callback result (expected string)");
+	    Py_DECREF(notes_result);
+	    return -EINVAL;
+	}
+
+	type |= PM_LABEL_OPTIONAL;
+	if (!empty_labelset(s) && (sts = __pmAddLabels(lp, s, type)) < 0)
+	    pmNotifyErr(LOG_ERR, "__pmAddLabels failed: %s", pmErrStr(sts));
+
+	Py_DECREF(notes_result);
+    }
 
-    Py_DECREF(result);
     return sts;
 }
 
-int
-store_callback(__pmID_int *pmid, unsigned int inst, pmAtomValue av, int type)
+static int
+store_callback(pmID pmid, unsigned int inst, pmAtomValue av, int type)
 {
     int rc, code;
-    int item = pmid->item;
-    int cluster = pmid->cluster;
+    int item = pmID_item(pmid);
+    int cluster = pmID_cluster(pmid);
     PyObject *arglist, *result;
 
     switch (type) {
@@ -720,7 +782,7 @@ store_callback(__pmID_int *pmid, unsigned int inst, pmAtomValue av, int type)
 	    pmNotifyErr(LOG_ERR, "unsupported type in store callback");
 	    return -EINVAL;
     }
-    result = PyEval_CallObject(store_cb_func, arglist);
+    result = PyObject_Call(store_cb_func, arglist, NULL);
     Py_DECREF(arglist);
     if (result == NULL)
 	return callback_error("store_callback");
@@ -734,30 +796,31 @@ store_callback(__pmID_int *pmid, unsigned int inst, pmAtomValue av, int type)
 }
 
 static pmdaMetric *
-lookup_metric(__pmID_int *pmid, pmdaExt *pmda)
+lookup_metric(pmID pmid, pmdaExt *pmda)
 {
     int		i;
     pmdaMetric	*mp;
+    unsigned int item = pmID_item(pmid);
+    unsigned int cluster = pmID_cluster(pmid);
 
     for (i = 0; i < pmda->e_nmetrics; i++) {
 	mp = &pmda->e_metrics[i];
-	if (pmid->item != pmID_item(mp->m_desc.pmid))
+	if (item != pmID_item(mp->m_desc.pmid))
 	    continue;
-	if (pmid->cluster != pmID_cluster(mp->m_desc.pmid))
+	if (cluster != pmID_cluster(mp->m_desc.pmid))
 	    continue;
 	return mp;
     }
     return NULL;
 }
 
-int
+static int
 store(pmResult *result, pmdaExt *pmda)
 {
     int		i, j, sts, type;
     pmAtomValue	av;
     pmdaMetric  *mp;
     pmValueSet  *vsp;
-    __pmID_int  *pmid;
 
     maybe_refresh_all();
 
@@ -768,10 +831,9 @@ store(pmResult *result, pmdaExt *pmda)
 
     for (i = 0; i < result->numpmid; i++) {
 	vsp = result->vset[i];
-	pmid = (__pmID_int *)&vsp->pmid;
 
 	/* find the type associated with this PMID */
-	if ((mp = lookup_metric(pmid, pmda)) == NULL)
+	if ((mp = lookup_metric(vsp->pmid, pmda)) == NULL)
 	    return PM_ERR_PMID;
 	type = mp->m_desc.type;
 
@@ -779,7 +841,7 @@ store(pmResult *result, pmdaExt *pmda)
 	    sts = pmExtractValue(vsp->valfmt, &vsp->vlist[j],type, &av, type);
 	    if (sts < 0)
 		return sts;
-	    sts = store_callback(pmid, vsp->vlist[j].inst, av, type);
+	    sts = store_callback(vsp->pmid, vsp->vlist[j].inst, av, type);
 	    if (sts < 0)
 		return sts;
 	}
@@ -787,7 +849,7 @@ store(pmResult *result, pmdaExt *pmda)
     return 0;
 }
 
-int
+static int
 text(int ident, int type, char **buffer, pmdaExt *pmda)
 {
     PyObject *dict, *value, *key;
@@ -822,7 +884,7 @@ text(int ident, int type, char **buffer, pmdaExt *pmda)
     return 0;
 }
 
-int
+static int
 attribute(int ctx, int attr, const char *value, int length, pmdaExt *pmda)
 {
     PyObject *arglist, *result;
@@ -834,11 +896,12 @@ attribute(int ctx, int attr, const char *value, int length, pmdaExt *pmda)
     if (attribute_cb_func == NULL)
         return 0;
 
-    arglist = Py_BuildValue("(iis#)", ctx, attr, value, length-1); // length includes NULL byte
+    // 'length' parameter includes the terminating NULL byte here
+    arglist = Py_BuildValue("(iis#)", ctx, attr, value, (Py_ssize_t)length-1);
     if (arglist == NULL)
         return -ENOMEM;
 
-    result = PyEval_CallObject(attribute_cb_func, arglist);
+    result = PyObject_Call(attribute_cb_func, arglist, NULL);
     Py_DECREF(arglist);
     if (result == NULL)
         return callback_error("attribute");
@@ -847,7 +910,7 @@ attribute(int ctx, int attr, const char *value, int length, pmdaExt *pmda)
     return 0;
 }
 
-void
+static void
 endContextCallBack(int ctx)
 {
     PyObject *arglist, *result;
@@ -859,7 +922,7 @@ endContextCallBack(int ctx)
     if (arglist == NULL)
         return;
 
-    result = PyEval_CallObject(endcontext_cb_func, arglist);
+    result = PyObject_Call(endcontext_cb_func, arglist, NULL);
     Py_DECREF(arglist);
     if (result == NULL) {
         callback_error("endcontext");
@@ -1472,6 +1535,12 @@ set_label(PyObject *self, PyObject *args)
     return set_callback(self, args, "O:set_label", &label_func);
 }
 
+static PyObject *
+set_notes(PyObject *self, PyObject *args)
+{
+    return set_callback(self, args, "O:set_notes", &notes_func);
+}
+
 static PyObject *
 set_refresh(PyObject *self, PyObject *args)
 {
@@ -1502,6 +1571,12 @@ set_label_callback(PyObject *self, PyObject *args)
     return set_callback(self, args, "O:set_label_callback", &label_cb_func);
 }
 
+static PyObject *
+set_notes_callback(PyObject *self, PyObject *args)
+{
+    return set_callback(self, args, "O:set_notes_callback", &notes_cb_func);
+}
+
 static PyObject *
 set_attribute_callback(PyObject *self, PyObject *args)
 {
@@ -1580,12 +1655,16 @@ static PyMethodDef methods[] = {
 	.ml_flags = METH_VARARGS|METH_KEYWORDS },
     { .ml_name = "set_label", .ml_meth = (PyCFunction)set_label,
 	.ml_flags = METH_VARARGS|METH_KEYWORDS },
+    { .ml_name = "set_notes", .ml_meth = (PyCFunction)set_notes,
+	.ml_flags = METH_VARARGS|METH_KEYWORDS },
     { .ml_name = "set_store_callback", .ml_meth = (PyCFunction)set_store_callback,
 	.ml_flags = METH_VARARGS|METH_KEYWORDS },
     { .ml_name = "set_fetch_callback", .ml_meth = (PyCFunction)set_fetch_callback,
 	.ml_flags = METH_VARARGS|METH_KEYWORDS },
     { .ml_name = "set_label_callback", .ml_meth = (PyCFunction)set_label_callback,
 	.ml_flags = METH_VARARGS|METH_KEYWORDS },
+    { .ml_name = "set_notes_callback", .ml_meth = (PyCFunction)set_notes_callback,
+	.ml_flags = METH_VARARGS|METH_KEYWORDS },
     { .ml_name = "set_attribute_callback", .ml_meth = (PyCFunction)set_attribute_callback,
 	.ml_flags = METH_VARARGS|METH_KEYWORDS },
     { .ml_name = "set_endcontext_callback", .ml_meth = (PyCFunction)set_endcontext_callback,
